@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: apic.cc,v 1.73 2006/01/15 17:38:40 sshwarts Exp $
+// $Id: apic.cc,v 1.94 2006/06/20 16:51:03 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -27,6 +27,7 @@
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #include "iodev/iodev.h"
 
 #if BX_SUPPORT_APIC
@@ -39,9 +40,9 @@
 
 int apic_bus_deliver_interrupt(Bit8u vector, Bit8u dest, Bit8u delivery_mode, Bit8u dest_mode, bx_bool level, bx_bool trig_mode)
 {
-  if (delivery_mode == APIC_DM_LOWPRI)
+  if(delivery_mode == APIC_DM_LOWPRI)
   {
-     if (dest_mode == 0) {
+     if(dest_mode == 0) {
        // I/O subsytem initiated interrupt with lowest priority delivery
        // mode is not supported in physical destination mode
 //     BX_ERROR(("Ignoring lowest priority interrupt in physical dest mode !"));
@@ -53,8 +54,8 @@ int apic_bus_deliver_interrupt(Bit8u vector, Bit8u dest, Bit8u delivery_mode, Bi
   }
 
   // determine destination local apics and deliver
-  if (dest_mode == 0) {
-    if (dest == APIC_BROADCAST_PHYSICAL_DESTINATION_MODE)
+  if(dest_mode == 0) {
+    if(dest == APIC_BROADCAST_PHYSICAL_DESTINATION_MODE)
     {
        return apic_bus_broadcast_interrupt(vector, delivery_mode, trig_mode, APIC_MAX_ID);
     }
@@ -73,12 +74,12 @@ int apic_bus_deliver_interrupt(Bit8u vector, Bit8u dest, Bit8u delivery_mode, Bi
   } 
   else {
     // logical destination mode
-    if (dest == 0) return 0;
+    if(dest == 0) return 0;
 
     bx_bool interrupt_delivered = 0;
 
     for (int i=0; i<BX_NUM_LOCAL_APICS; i++) {
-      if (BX_CPU_APIC(i)->match_logical_addr(dest)) {
+      if(BX_CPU_APIC(i)->match_logical_addr(dest)) {
         BX_CPU_APIC(i)->deliver(vector, delivery_mode, trig_mode);
         interrupt_delivered = 1;
       }
@@ -93,7 +94,7 @@ int apic_bus_deliver_lowest_priority(Bit8u vector, Bit8u dest, bx_bool trig_mode
 #ifndef BX_IMPLEMENT_XAPIC
   // search for if focus processor exists
   for (int i=0; i<BX_NUM_LOCAL_APICS; i++) {
-    if (BX_CPU_APIC(i)->is_focus(vector)) {
+    if(BX_CPU_APIC(i)->is_focus(vector)) {
       BX_CPU_APIC(i)->deliver(vector, APIC_DM_LOWPRI, trig_mode);
       return 1;
     }
@@ -104,20 +105,20 @@ int apic_bus_deliver_lowest_priority(Bit8u vector, Bit8u dest, bx_bool trig_mode
   int lowest_priority_agent = -1, lowest_priority = 0x100;
 
   for (int i=0; i<BX_NUM_LOCAL_APICS; i++) {
-    if (broadcast || BX_CPU_APIC(i)->match_logical_addr(dest)) {
+    if(broadcast || BX_CPU_APIC(i)->match_logical_addr(dest)) {
 #ifndef BX_IMPLEMENT_XAPIC
       int priority = BX_CPU_APIC(i)->get_apr();
 #else
       int priority = BX_CPU_APIC(i)->get_tpr();
 #endif
-      if (priority < lowest_priority) {
+      if(priority < lowest_priority) {
         lowest_priority = priority;
         lowest_priority_agent = i;
       }
     }
   }
 
-  if (lowest_priority_agent >= 0)
+  if(lowest_priority_agent >= 0)
   {
     BX_CPU_APIC(lowest_priority_agent)->deliver(vector, APIC_DM_LOWPRI, trig_mode);
     return 1;
@@ -128,14 +129,14 @@ int apic_bus_deliver_lowest_priority(Bit8u vector, Bit8u dest, bx_bool trig_mode
 
 int apic_bus_broadcast_interrupt(Bit8u vector, Bit8u delivery_mode, bx_bool trig_mode, int exclude_cpu)
 {
-  if (delivery_mode == APIC_DM_LOWPRI)
+  if(delivery_mode == APIC_DM_LOWPRI)
   {
     return apic_bus_deliver_lowest_priority(vector, 0 /* doesn't matter */, trig_mode, 1);
   }
 
   // deliver to all bus agents except 'exclude_cpu'
   for (int i=0; i<BX_NUM_LOCAL_APICS; i++) {
-    if (i == exclude_cpu) continue;
+    if(i == exclude_cpu) continue;
     BX_CPU_APIC(i)->deliver(vector, delivery_mode, trig_mode);
   }
 
@@ -149,91 +150,108 @@ static void apic_bus_broadcast_eoi(Bit8u vector)
 
 ////////////////////////////////////
 
-bx_generic_apic_c::bx_generic_apic_c()
+bx_generic_apic_c::bx_generic_apic_c(bx_phy_address base)
 {
-  id = APIC_UNKNOWN_ID;
   put("APIC?");
   settype(APICLOG);
+  id = APIC_UNKNOWN_ID;
+  set_base(base);
 }
 
-void bx_generic_apic_c::set_base (bx_address newbase)
+void bx_generic_apic_c::set_base(bx_phy_address newbase)
 {
   newbase &= (~0xfff);
-  BX_INFO(("relocate APIC id=%d to " FMT_ADDRX, id, newbase));
   base_addr = newbase;
+  if (id != APIC_UNKNOWN_ID)
+    BX_INFO(("relocate APIC id=%d to 0x%08x", id, newbase));
 }
 
-void bx_generic_apic_c::set_id (Bit8u newid) 
+void bx_generic_apic_c::set_id(Bit8u newid) 
 {
-  BX_INFO (("set APIC ID to %d", newid));
+  BX_INFO(("set APIC ID to %d", newid));
   id = newid;
 }
 
-bx_bool bx_generic_apic_c::is_selected (bx_address addr, Bit32u len)
+bx_bool bx_generic_apic_c::is_selected(bx_phy_address addr, unsigned len)
 {
-  if ((addr & ~0xfff) == get_base()) {
-    if (((addr & 0xf) != 0) || (len != 4))
-      BX_INFO(("warning: misaligned or wrong-size APIC access. len=%d, addr=" FMT_ADDRX "", len, addr));
+  if((addr & ~0xfff) == get_base()) {
+    if((addr & 0xf) != 0)
+      BX_INFO(("warning: misaligned APIC access. addr=0x%08x, len=%d", addr, len));
     return 1;
   }
   return 0;
 }
 
-void bx_generic_apic_c::read (Bit32u addr, void *data, unsigned len)
+void bx_generic_apic_c::read(bx_phy_address addr, void *data, unsigned len)
 {
-  if ((addr & ~0xf) != ((addr+len-1) & ~0xf))
-    BX_PANIC(("APIC read spans 32-bit boundary"));
+  if((addr & ~0x3) != ((addr+len-1) & ~0x3)) {
+    BX_PANIC(("APIC read at address 0x%08x spans 32-bit boundary !", addr));
+    return;
+  }
   Bit32u value;
-  read_aligned (addr, &value, 4);
-  if ((addr&3) == 0 && len == 4) {
+  read_aligned(addr & ~0x3, &value);
+  if(len == 4) { // must be 32-bit aligned
     *((Bit32u *)data) = value;
     return;
   }
-  // handle partial word read, independent of endian-ness.
-  Bit8u bytes[4];
-  bytes[0] = value & 0xff;
-  bytes[1] = (value >> 8) & 0xff;
-  bytes[2] = (value >> 16) & 0xff;
-  bytes[3] = (value >> 24) & 0xff;
-  Bit8u *p1 = bytes+(addr&3);
-  Bit8u *p2 = (Bit8u *)data;
-  for (unsigned i=0; i<len; i++) {
-    if (bx_dbg.apic)
-      BX_INFO(("apic: Copying byte %02x", (unsigned int) *p1));
-    *p2++ = *p1++;
+  // handle partial read, independent of endian-ness
+  value >>= (addr&3)*8;
+  if (len == 1)
+    *((Bit8u *) data) = value & 0xff;
+  else if (len == 2)
+    *((Bit16u *)data) = value & 0xffff;
+  else
+    BX_PANIC(("Unsupported APIC read at address 0x%08x, len=%d", addr, len));
+}
+
+void bx_generic_apic_c::write(bx_phy_address addr, void *data, unsigned len)
+{
+  if((addr & ~0x3) != ((addr+len-1) & ~0x3)) {
+    BX_PANIC(("APIC write at address 0x%08x spans 32-bit boundary !", addr));
+    return;
   }
+  bx_phy_address addr_aligned = addr & ~0x3;
+  if(len == 4) { // must be 32-bit aligned
+    write_aligned(addr_aligned, (Bit32u*) data);
+    return;
+  }
+  // partial write to the apic register, need to update some bytes 
+  // and do not touch the others, i.e. to do RMW operation
+  Bit32u value;
+  read_aligned(addr_aligned, &value);  // apic read has no side effects
+  // handle partial write, independent of endian-ness
+  unsigned shift = (addr&3)*8;
+  if (len == 1) {
+    value &= ~(0xff << shift);
+    value |= (*((Bit8u *) data) << shift);
+  }
+  else if (len == 2) {
+    value &= ~(0xffff << shift);
+    value |= (*((Bit16u *)data) << shift);
+  }
+  else {
+    BX_PANIC(("Unsupported APIC write at address 0x%08x, len=%d", addr, len));
+  }
+  write_aligned(addr_aligned, &value);
 }
 
 bx_local_apic_c::bx_local_apic_c(BX_CPU_C *mycpu)
-  : bx_generic_apic_c(), cpu(mycpu), cpu_id(cpu->which_cpu())
+  : bx_generic_apic_c(BX_LAPIC_BASE_ADDR), cpu(mycpu), cpu_id(cpu->which_cpu())
 {
-  reset();
-
   // KPL: Register a non-active timer for use when the timer is started.
   timer_handle = bx_pc_system.register_timer_ticks(this,
             BX_CPU(0)->local_apic.periodic_smf, 0, 0, 0, "lapic");
+  timer_active = 0;
+
+  reset();
 	
   INTR = 0;
 }
 
-/*
-Bit32u bx_local_apic_c::get_arb_id (void)
-{
-  return arb_id;
-}
-
-void bx_local_apic_c::set_arb_id (Bit32u new_arb_id)
-{
-  BX_DEBUG (("set arbitration ID to %d", new_arb_id));
-  arb_id = new_arb_id;
-}
-*/
-
 void bx_local_apic_c::reset() 
 {
   /* same as INIT but also sets arbitration ID and APIC ID */
   init();
-//arb_id = id;
 }
 
 void bx_local_apic_c::init()
@@ -243,11 +261,10 @@ void bx_local_apic_c::init()
   bx_generic_apic_c::init();
 
   BX_INFO(("local apic in %s initializing",
-      (cpu && cpu->name) ? cpu->name : "?"));
+     (cpu && cpu->name) ? cpu->name : "?"));
 
   // default address for a local APIC, can be moved
-  base_addr = APIC_BASE_ADDR;
-  bypass_irr_isr = 0;
+  base_addr = BX_LAPIC_BASE_ADDR;
   error_status = shadow_error_status = 0;
   log_dest = 0;
   dest_format = 0xf;
@@ -256,7 +273,7 @@ void bx_local_apic_c::init()
   log_dest = 0;
   task_priority = 0;
 
-  for (i=0; i<BX_LOCAL_APIC_MAX_INTS; i++) {
+  for(i=0; i<BX_LOCAL_APIC_MAX_INTS; i++) {
     irr[i] = isr[i] = tmr[i] = 0;
   }
 
@@ -265,59 +282,48 @@ void bx_local_apic_c::init()
   timer_initial = 0;
   timer_current = 0;
 
-  if (timer_active) {
+  if(timer_active) {
     bx_pc_system.deactivate_timer(timer_handle);
     timer_active = 0;
   }
 
-  for (i=0; i<APIC_LVT_ENTRIES; i++) {
+  for(i=0; i<APIC_LVT_ENTRIES; i++) {
     lvt[i] = 0x10000;	// all LVT are masked
   }
 
-  spurious_vector  = 0xff;   // software disabled (bit 8)
+  spurious_vector  = 0xff;   // software disabled(bit 8)
   software_enabled = 0;
   focus_disable    = 0;
 }
 
-void bx_local_apic_c::set_id (Bit8u newid)
+void bx_local_apic_c::set_id(Bit8u newid)
 {
-  bx_generic_apic_c::set_id (newid);
-  sprintf (cpu->name, "CPU apicid=%02x", (Bit32u)id);
-  if (id < APIC_MAX_ID) {
+  bx_generic_apic_c::set_id(newid);
+  sprintf(cpu->name, "CPU apicid=%02x",(Bit32u)id);
+  if(id < APIC_MAX_ID) {
     char buffer[16];
-    sprintf (buffer, "APIC%x", id);
+    sprintf(buffer, "APIC%x", id);
     put(buffer);
     settype(CPU0LOG + id);
-    sprintf (buffer, "CPU%x", id);
-    cpu->put (buffer);
+    sprintf(buffer, "CPU%x", id);
+    cpu->put(buffer);
   } else {
-    BX_INFO (("naming convention for apics requires id=0-%d only", APIC_MAX_ID));
+    BX_INFO(("naming convention for apics requires id=0-%d only", APIC_MAX_ID));
   }
   if(BX_CPU_LEVEL<2)
-    BX_INFO(( "8086" ));
+    BX_INFO(("8086"));
   else
-    BX_INFO(( "80%d86", BX_CPU_LEVEL ));
+    BX_INFO(("80%d86", BX_CPU_LEVEL));
 }
 
-void bx_local_apic_c::set_divide_configuration (Bit32u value)
+// APIC write: 4 byte write to 16-byte aligned APIC address
+void bx_local_apic_c::write_aligned(bx_phy_address addr, Bit32u *data)
 {
-  BX_ASSERT (value == (value & 0x0b));
-  // move bit 3 down to bit 0.
-  value = ((value & 8) >> 1) | (value & 3);
-  BX_ASSERT (value >= 0 && value <= 7);
-  timer_divide_factor = (value==7)? 1 : (2 << value);
-  BX_INFO(("%s: set timer divide factor to %d", cpu->name, timer_divide_factor));
-}
-
-void bx_local_apic_c::write (Bit32u addr, Bit32u *data, unsigned len)
-{
-  if (len != 4) {
-    BX_PANIC (("local apic write with len=%d (should be 4)", len));
-  }
-  BX_DEBUG(("%s: write 0x%08x to APIC address %08x", cpu->name, *data, addr));
+  BX_DEBUG(("%s: LAPIC write 0x%08x to address %08x", cpu->name, *data, addr));
+  BX_ASSERT((addr & 0xf) == 0);
   addr &= 0xff0;
   Bit32u value = *data;
-  switch (addr) {
+  switch(addr) {
     case 0x20: // local APIC id
       id = (value>>24) & APIC_ID_MASK;
       break;
@@ -329,11 +335,11 @@ void bx_local_apic_c::write (Bit32u addr, Bit32u *data, unsigned len)
       break;
     case 0xd0: // logical destination
       log_dest = (value >> 24) & APIC_ID_MASK;
-      BX_DEBUG (("set logical destination to %02x", log_dest));
+      BX_DEBUG(("set logical destination to %02x", log_dest));
       break;
     case 0xe0: // destination format
       dest_format = (value >> 28) & 0xf;
-      BX_DEBUG (("set destination format to %02x", dest_format));
+      BX_DEBUG(("set destination format to %02x", dest_format));
       break;
     case 0xf0: // spurious interrupt vector
       write_spurious_interrupt_register(value);
@@ -347,7 +353,7 @@ void bx_local_apic_c::write (Bit32u addr, Bit32u *data, unsigned len)
       shadow_error_status = 0;
       break;
     case 0x300: // interrupt command reg 0-31
-      icr_lo = value & ~(1<<12);  // force delivery status bit = 0 (idle)
+      icr_lo = value & ~(1<<12);  // force delivery status bit = 0(idle)
       send_ipi();
       break;
     case 0x310: // interrupt command reg 31-63
@@ -355,27 +361,27 @@ void bx_local_apic_c::write (Bit32u addr, Bit32u *data, unsigned len)
       break;
     case 0x320: // LVT Timer Reg
       lvt[APIC_LVT_TIMER] = value & 0x300ff;
-      if (! software_enabled) lvt[APIC_LVT_TIMER] |= 0x10000;
+      if(! software_enabled) lvt[APIC_LVT_TIMER] |= 0x10000;
       break;
     case 0x330: // LVT Thermal Monitor
       lvt[APIC_LVT_THERMAL] = value & 0x107ff;
-      if (! software_enabled) lvt[APIC_LVT_THERMAL] |= 0x10000;
+      if(! software_enabled) lvt[APIC_LVT_THERMAL] |= 0x10000;
       break;
     case 0x340: // LVT Performance Counter
       lvt[APIC_LVT_PERFORM] = value & 0x107ff;
-      if (! software_enabled) lvt[APIC_LVT_PERFORM] |= 0x10000;
+      if(! software_enabled) lvt[APIC_LVT_PERFORM] |= 0x10000;
       break;
     case 0x350: // LVT LINT0 Reg
       lvt[APIC_LVT_LINT0] = value & 0x1a7ff;
-      if (! software_enabled) lvt[APIC_LVT_LINT0] |= 0x10000;
+      if(! software_enabled) lvt[APIC_LVT_LINT0] |= 0x10000;
       break;
     case 0x360: // LVT Lint1 Reg
       lvt[APIC_LVT_LINT1] = value & 0x1a7ff;
-      if (! software_enabled) lvt[APIC_LVT_LINT1] |= 0x10000;
+      if(! software_enabled) lvt[APIC_LVT_LINT1] |= 0x10000;
       break;
     case 0x370: // LVT Error Reg
       lvt[APIC_LVT_ERROR] = value & 0x100ff;
-      if (! software_enabled) lvt[APIC_LVT_ERROR] |= 0x10000;
+      if(! software_enabled) lvt[APIC_LVT_ERROR] |= 0x10000;
       break;
     case 0x380: // initial count for timer
       set_initial_timer_count(value);
@@ -383,7 +389,7 @@ void bx_local_apic_c::write (Bit32u addr, Bit32u *data, unsigned len)
     case 0x3e0: // timer divide configuration
       // only bits 3, 1, and 0 are writable
       timer_divconf = value & 0xb;
-      set_divide_configuration (timer_divconf);
+      set_divide_configuration(timer_divconf);
       break;
     /* all read-only registers go here */
     case 0x30: // local APIC version
@@ -421,19 +427,19 @@ void bx_local_apic_c::send_ipi(void)
   int vector = (icr_lo & 0xff);
   int accepted = 0;
 
-  if (delivery_mode == APIC_DM_INIT)
+  if(delivery_mode == APIC_DM_INIT)
   {
-    if (level == 0 && trig_mode == 1) {
+    if(level == 0 && trig_mode == 1) {
       // special mode in local apic.  See "INIT Level Deassert" in the
       // Intel Soft. Devel. Guide Vol 3, page 7-34.  This magic code
-      // causes all APICs (regardless of dest address) to set their
+      // causes all APICs(regardless of dest address) to set their
       // arbitration ID to their APIC ID. Not supported by Pentium 4
       // and Intel Xeon processors.
       return; // we not model APIC bus arbitration ID anyway
     }
   }
 
-  switch (dest_shorthand) {
+  switch(dest_shorthand) {
   case 0:  // no shorthand, use real destination value
     accepted = apic_bus_deliver_interrupt(vector, dest, delivery_mode, dest_mode, level, trig_mode);
     break;
@@ -451,7 +457,7 @@ void bx_local_apic_c::send_ipi(void)
     BX_PANIC(("Invalid desination shorthand %#x\n", dest_shorthand));
   }
 
-  if (! accepted) {
+  if(! accepted) {
     BX_DEBUG(("An IPI wasn't accepted, raise APIC_ERR_TX_ACCEPT_ERR"));
     shadow_error_status |= APIC_ERR_TX_ACCEPT_ERR;
   }
@@ -471,8 +477,8 @@ void bx_local_apic_c::write_spurious_interrupt_register(Bit32u value)
   software_enabled = (value >> 8) & 1;
   focus_disable    = (value >> 9) & 1;
 
-  if (! software_enabled) {
-    for (unsigned i=0; i<APIC_LVT_ENTRIES; i++) {
+  if(! software_enabled) {
+    for(unsigned i=0; i<APIC_LVT_ENTRIES; i++) {
       lvt[i] |= 0x10000;	// all LVT are masked
     }
   }
@@ -481,14 +487,14 @@ void bx_local_apic_c::write_spurious_interrupt_register(Bit32u value)
 void bx_local_apic_c::receive_EOI(Bit32u value)
 {
   BX_DEBUG(("%s: Wrote 0x%x to EOI", cpu->name, value));
-  int vec = highest_priority_int (isr);
-  if (vec < 0) {
+  int vec = highest_priority_int(isr);
+  if(vec < 0) {
     BX_DEBUG(("EOI written without any bit in ISR"));
   } else {
-      if ((Bit32u) vec != spurious_vector) {
+      if((Bit32u) vec != spurious_vector) {
         BX_DEBUG(("%s: local apic received EOI, hopefully for vector 0x%02x", cpu->name, vec));
         isr[vec] = 0; 
-        if (tmr[vec]) {
+        if(tmr[vec]) {
             apic_bus_broadcast_eoi(vec);
             tmr[vec] = 0;
         }
@@ -496,34 +502,34 @@ void bx_local_apic_c::receive_EOI(Bit32u value)
       }
   }
 
-  if (bx_dbg.apic)
+  if(bx_dbg.apic)
       print_status();
 }
 
-void bx_local_apic_c::startup_msg (Bit32u vector)
+void bx_local_apic_c::startup_msg(Bit32u vector)
 {
-  if (cpu->debug_trap & 0x80000000) {
-    cpu->debug_trap &= ~0x80000000;
+  if(cpu->debug_trap & BX_DEBUG_TRAP_HALT_STATE) {
+    cpu->debug_trap &= ~BX_DEBUG_TRAP_HALT_STATE;
     cpu->dword.eip = 0;
-    cpu->load_seg_reg (&cpu->sregs[BX_SEG_REG_CS], vector*0x100);
+    cpu->load_seg_reg(&cpu->sregs[BX_SEG_REG_CS], vector*0x100);
     BX_INFO(("%s started up at %04X:%08X by APIC", cpu->name, vector*0x100, cpu->dword.eip));
   } else {
     BX_INFO(("%s started up by APIC, but was not halted at the time", cpu->name));
   }
 }
 
-void bx_local_apic_c::read_aligned (Bit32u addr, Bit32u *data, unsigned len)
+// APIC read: 4 byte read from 16-byte aligned APIC address
+void bx_local_apic_c::read_aligned(bx_phy_address addr, Bit32u *data)
 {
-  if (len != 4) {
-    BX_PANIC (("local apic read with len=%d (should be 4)", len));
-  }
+  BX_DEBUG(("%s: LAPIC read from address %08x", cpu->name, addr));
+  BX_ASSERT((addr & 0xf) == 0);
   *data = 0;  // default value for unimplemented registers
-  Bit32u addr2 = addr & 0xff0;
-  switch (addr2) {
+  bx_phy_address addr2 = addr & 0xff0;
+  switch(addr2) {
   case 0x20: // local APIC id
     *data = (id) << 24; break;
   case 0x30: // local APIC version
-    *data = APIC_VERSION_ID; break;
+    *data = BX_LAPIC_VERSION_ID; break;
   case 0x80: // task priority
     *data = task_priority & 0xff; break;
   case 0x90: // arbitration priority
@@ -544,8 +550,8 @@ void bx_local_apic_c::read_aligned (Bit32u addr, Bit32u *data, unsigned len)
   case 0xf0: // spurious interrupt vector
     {
       Bit32u reg = spurious_vector;
-      if (software_enabled) reg |= 0x100;
-      if (focus_disable) reg |= 0x200;
+      if(software_enabled) reg |= 0x100;
+      if(focus_disable) reg |= 0x200;
       *data = reg; 
     }
     break;
@@ -554,10 +560,10 @@ void bx_local_apic_c::read_aligned (Bit32u addr, Bit32u *data, unsigned len)
   case 0x140: case 0x150:
   case 0x160: case 0x170:
     {
-      unsigned index = (addr2 - 0x100) >> 2;
+      unsigned index = (addr2 - 0x100) << 1;
       Bit32u value = 0, mask = 1;
-      for (int i=0;i<32;i++) {
-        if (isr[index+i]) value |= mask;
+      for(int i=0;i<32;i++) {
+        if(isr[index+i]) value |= mask;
         mask <<= 1;
       }
       *data = value;
@@ -568,10 +574,10 @@ void bx_local_apic_c::read_aligned (Bit32u addr, Bit32u *data, unsigned len)
   case 0x1c0: case 0x1d0:
   case 0x1e0: case 0x1f0:
     {
-      unsigned index = (addr2 - 0x180) >> 2;
+      unsigned index = (addr2 - 0x180) << 1;
       Bit32u value = 0, mask = 1;
-      for (int i=0;i<32;i++) {
-        if (tmr[index+i]) value |= mask;
+      for(int i=0;i<32;i++) {
+        if(tmr[index+i]) value |= mask;
         mask <<= 1;
       }
       *data = value;
@@ -582,10 +588,10 @@ void bx_local_apic_c::read_aligned (Bit32u addr, Bit32u *data, unsigned len)
   case 0x240: case 0x250:
   case 0x260: case 0x270:
     {
-      unsigned index = (addr2 - 0x200) >> 2;
+      unsigned index = (addr2 - 0x200) << 1;
       Bit32u value = 0, mask = 1;
-      for (int i=0;i<32;i++) {
-        if (irr[index+i]) value |= mask;
+      for(int i=0;i<32;i++) {
+        if(irr[index+i]) value |= mask;
         mask <<= 1;
       }
       *data = value;
@@ -610,17 +616,16 @@ void bx_local_apic_c::read_aligned (Bit32u addr, Bit32u *data, unsigned len)
     }
   case 0x380: // initial count for timer
     *data = timer_initial;
-    //fprintf(stderr, "APIC: R(Initial Count Register) = %u\n", *data);
     break;
   case 0x390: // current count for timer
-    if (timer_active==0) {
+    if(timer_active==0) {
       *data = timer_current;
     } else {
       Bit64u delta64;
       Bit32u delta32;
       delta64 = (bx_pc_system.time_ticks() - ticksInitial) / timer_divide_factor;
       delta32 = (Bit32u) delta64;
-      if (delta32 > timer_initial)
+      if(delta32 > timer_initial)
         BX_PANIC(("APIC: R(curr timer count): delta < initial"));
       timer_current = timer_initial - delta32;
       *data = timer_current;
@@ -636,31 +641,31 @@ void bx_local_apic_c::read_aligned (Bit32u addr, Bit32u *data, unsigned len)
   BX_DEBUG(("%s: read from APIC address %08x = %08x", cpu->name, addr, *data));
 }
 
-int bx_local_apic_c::highest_priority_int (Bit8u *array)
+int bx_local_apic_c::highest_priority_int(Bit8u *array)
 {
-  for (int i=BX_LOCAL_APIC_MAX_INTS-1; i>=0; i--)
-    if (array[i]) return i;
+  for(int i=BX_APIC_LAST_VECTOR; i>=BX_APIC_FIRST_VECTOR; i--)
+    if(array[i]) return i;
 
   return -1;
 }
 
 void bx_local_apic_c::service_local_apic(void)
 {
-  if (bx_dbg.apic) {
+  if(bx_dbg.apic) {
     BX_INFO(("service_local_apic()"));
     print_status();
   }
-  if (INTR) return;  // INTR already up; do nothing
+  if(INTR) return;  // INTR already up; do nothing
   // find first interrupt in irr.
   int first_irr = highest_priority_int(irr);
-  int first_isr = highest_priority_int(isr);
   if (first_irr < 0) return;   // no interrupts, leave INTR=0
-  if (first_isr >= 0 && first_irr >= first_isr) {
-    BX_DEBUG(("local apic (%s): not delivering int%02x because int%02x is in service", cpu->name, first_irr, first_isr));
+  int first_isr = highest_priority_int(isr);
+  if (first_isr >= 0 && first_irr <= first_isr) {
+    BX_DEBUG(("local apic(%s): not delivering int 0x%02x because int 0x%02x is in service", cpu->name, first_irr, first_isr));
     return;
   }
-  if (((Bit32u)(first_irr) & 0x00f0) <= (task_priority & 0x00f0)) {
-    BX_DEBUG(("local apic (%s): not delivering int%02X because task_priority is %X", cpu->name, first_irr, task_priority));
+  if(((Bit32u)(first_irr) & 0xf0) <= (task_priority & 0xf0)) {
+    BX_DEBUG(("local apic(%s): not delivering int 0x%02X because task_priority is 0x%02X", cpu->name, first_irr, task_priority));
     return;
   }
   // interrupt has appeared in irr. Raise INTR. When the CPU
@@ -671,20 +676,22 @@ void bx_local_apic_c::service_local_apic(void)
   cpu->async_event = 1;
 }
 
-bx_bool bx_local_apic_c::deliver (Bit8u vector, Bit8u delivery_mode, Bit8u trig_mode)
+bx_bool bx_local_apic_c::deliver(Bit8u vector, Bit8u delivery_mode, Bit8u trig_mode)
 {
-  switch (delivery_mode) {
+  switch(delivery_mode) {
   case APIC_DM_FIXED:
   case APIC_DM_LOWPRI:
     BX_DEBUG(("Deliver lowest priority of fixed interrupt vector %02x", vector));
     trigger_irq(vector, trig_mode);
     break;
   case APIC_DM_SMI:
-    BX_PANIC(("Delivery if SMI still not implemented !"));
-    return 0;
+    BX_PANIC(("Delivery of SMI still not implemented !"));
+    cpu->deliver_SMI();
+    return 1;
   case APIC_DM_NMI:
-    BX_PANIC(("Delivery if SMI still not implemented !"));
-    return 0;
+    BX_PANIC(("Delivery of NMI still not implemented !"));
+    cpu->deliver_NMI();
+    return 1;
   case APIC_DM_INIT:
     BX_DEBUG(("Deliver INIT IPI"));
     init();
@@ -695,21 +702,20 @@ bx_bool bx_local_apic_c::deliver (Bit8u vector, Bit8u delivery_mode, Bit8u trig_
     break;
   case APIC_DM_EXTINT:
     BX_DEBUG(("Deliver EXTINT vector %02x", vector));
-    bypass_irr_isr = 1;
-    trigger_irq(vector, trig_mode);
+    trigger_irq(vector, trig_mode, 1);
     break;
   default:
-    break;
+    return 0;
   }
 
   return 1;
 }
 
-void bx_local_apic_c::trigger_irq (unsigned vector, unsigned trigger_mode)
+void bx_local_apic_c::trigger_irq(unsigned vector, unsigned trigger_mode, bx_bool bypass_irr_isr)
 {
   BX_DEBUG(("Local apic on %s: trigger interrupt vector=0x%x", cpu->name, vector));
   
-  if (vector > BX_APIC_LAST_VECTOR || vector < BX_APIC_FIRST_VECTOR) {
+  if(vector > BX_APIC_LAST_VECTOR || vector < BX_APIC_FIRST_VECTOR) {
     shadow_error_status |= APIC_ERR_RX_ILLEGAL_VEC;
     BX_INFO(("bogus vector %#x, ignoring ...", vector));
     return;
@@ -717,12 +723,11 @@ void bx_local_apic_c::trigger_irq (unsigned vector, unsigned trigger_mode)
 
   BX_DEBUG(("triggered vector %#02x", vector));
 
-  if (bypass_irr_isr) {
-    bypass_irr_isr = 0;
+  if(bypass_irr_isr) {
     goto service_vector;
   }
 
-  if (irr[vector] != 0) {
+  if(irr[vector] != 0) {
     BX_DEBUG(("triggered vector %#02x not accepted", vector));
     return;
   }
@@ -733,36 +738,30 @@ service_vector:
   service_local_apic();
 }
 
-void bx_local_apic_c::untrigger_irq (unsigned vector, unsigned trigger_mode)
+void bx_local_apic_c::untrigger_irq(unsigned vector, unsigned trigger_mode)
 {
   BX_DEBUG(("Local apic on %s: untrigger interrupt vector=0x%x", cpu->name, vector));
   // hardware says "no more".  clear the bit.  If the CPU hasn't yet
   // acknowledged the interrupt, it will never be serviced.
-  BX_ASSERT (irr[vector] == 1);
+  BX_ASSERT(irr[vector] == 1);
   irr[vector] = 0;
-  if (bx_dbg.apic) print_status();
+  if(bx_dbg.apic) print_status();
 }
 
 Bit8u bx_local_apic_c::acknowledge_int(void)
 {
   // CPU calls this when it is ready to service one interrupt
-  if (!INTR)
+  if(!INTR)
     BX_PANIC(("%s: acknowledged an interrupt, but INTR=0", cpu->name));
-  BX_ASSERT (INTR);
+  BX_ASSERT(INTR);
   int vector = highest_priority_int(irr);
   if (vector < 0) goto spurious;
-  if ((vector & 0xf0) <= get_ppr()) goto spurious;
-  if (irr[vector] != 1) {
-    BX_PANIC(("IRR was not 1! irr[%d]=%#x", vector, irr[vector]));
-    irr[vector]=1;
-  }
-  BX_ASSERT (irr[vector] == 1);
+  if((vector & 0xf0) <= get_ppr()) goto spurious;
+  BX_ASSERT(irr[vector] == 1);
   BX_DEBUG(("%s: acknowledge_int returning vector 0x%x", cpu->name, vector));
-  // currently isr never gets cleared, so no point
-  //BX_ASSERT (isr[vector] == 0);
   irr[vector] = 0;
   isr[vector] = 1;
-  if (bx_dbg.apic) {
+  if(bx_dbg.apic) {
     BX_INFO(("Status after setting isr:"));
     print_status();
   }
@@ -775,36 +774,35 @@ spurious:
   INTR = 0;
   cpu->async_event = 1;
   return spurious_vector;
-
 }
 
-void bx_local_apic_c::print_status (void)
+void bx_local_apic_c::print_status(void)
 {
   BX_INFO(("%s local apic: status is {:", cpu->name));
-  for (int vec=0; vec<BX_LOCAL_APIC_MAX_INTS; vec++) {
-    if (irr[vec] || isr[vec]) {
-      BX_INFO(("vec 0x%x: irr=%d, isr=%d", vec, (int)irr[vec], (int)isr[vec]));
+  for(int vec=0; vec<BX_LOCAL_APIC_MAX_INTS; vec++) {
+    if(irr[vec] || isr[vec]) {
+      BX_INFO(("vec 0x%x: irr=%d, isr=%d", vec,(int)irr[vec],(int)isr[vec]));
     }
   }
   BX_INFO(("}"));
 }
 
-bx_bool bx_local_apic_c::match_logical_addr (Bit8u address) 
+bx_bool bx_local_apic_c::match_logical_addr(Bit8u address) 
 {
-  if (dest_format != 0xf) {
+  if(dest_format != 0xf) {
     BX_PANIC(("bx_local_apic_c::match_logical_addr: cluster model addressing not implemented"));
   }
   bx_bool match = ((address & log_dest) != 0);
-  BX_DEBUG (("%s: comparing MDA %02x to my LDR %02x -> %s", cpu->name,
+  BX_DEBUG(("%s: comparing MDA %02x to my LDR %02x -> %s", cpu->name,
     address, log_dest, match? "Match" : "Not a match"));
   return match;
 }
 
-Bit8u bx_local_apic_c::get_ppr (void)
+Bit8u bx_local_apic_c::get_ppr(void)
 {
-  int ppr = highest_priority_int (isr);
+  int ppr = highest_priority_int(isr);
 
-  if ((ppr < 0) || ((task_priority & 0xF0) >= ((Bit32u) ppr & 0xF0))) 
+  if((ppr < 0) || ((task_priority & 0xF0) >= ((Bit32u) ppr & 0xF0))) 
     ppr = task_priority;
   else 
     ppr &= 0xF0;
@@ -812,14 +810,14 @@ Bit8u bx_local_apic_c::get_ppr (void)
   return ppr;
 }
 
-Bit8u bx_local_apic_c::get_tpr (void)
+Bit8u bx_local_apic_c::get_tpr(void)
 {
   return task_priority;
 }
 
-void  bx_local_apic_c::set_tpr (Bit8u priority)
+void  bx_local_apic_c::set_tpr(Bit8u priority)
 {
-  if (priority < task_priority) {
+  if(priority < task_priority) {
     task_priority = priority;
     service_local_apic();
   } else {
@@ -834,26 +832,26 @@ Bit8u bx_local_apic_c::get_apr(void)
   Bit32u irrv = (highest_priority_int(irr) >> 4) & 0xf;
   Bit8u  apr;
 
-  if (isrv < 0) isrv = 0;
-  if (irrv < 0) irrv = 0;
+  if(isrv < 0) isrv = 0;
+  if(irrv < 0) irrv = 0;
 
-  if ((tpr >= irrv) && (tpr > isrv)) {
+  if((tpr >= irrv) && (tpr > isrv)) {
     apr = task_priority & 0xff;
   }
   else {
-    apr = ((tpr & isrv) > irrv) ? (tpr & isrv) : irrv;
+    apr = ((tpr & isrv) > irrv) ?(tpr & isrv) : irrv;
     apr <<= 4;
   }
 
   BX_DEBUG(("apr = %d\n", apr));
 
-  return (Bit8u) apr;
+  return(Bit8u) apr;
 }
 
 bx_bool bx_local_apic_c::is_focus(Bit8u vector)
 {
-  if (focus_disable) return 0;
-  return (irr[vector] || isr[vector]) ? 1 : 0;
+  if(focus_disable) return 0;
+  return(irr[vector] || isr[vector]) ? 1 : 0;
 }
 
 void bx_local_apic_c::periodic_smf(void *this_ptr)
@@ -864,41 +862,57 @@ void bx_local_apic_c::periodic_smf(void *this_ptr)
 
 void bx_local_apic_c::periodic(void)
 {
-  if (!timer_active) {
+  if(!timer_active) {
     BX_ERROR(("%s: bx_local_apic_c::periodic called, timer_active==0", cpu->name));
     return;
   }
 
   // timer reached zero since the last call to periodic.
   Bit32u timervec = lvt[APIC_LVT_TIMER];
-  if (timervec & 0x20000) {
+  if(timervec & 0x20000) {
     // Periodic mode.
     // If timer is not masked, trigger interrupt.
-    if ((timervec & 0x10000)==0) {
-      trigger_irq (timervec & 0xff, APIC_EDGE_TRIGGERED);
+    if((timervec & 0x10000)==0) {
+      trigger_irq(timervec & 0xff, APIC_EDGE_TRIGGERED);
+    }
+    else {
+      BX_DEBUG(("%s: local apic timer LVT masked", cpu->name));
     }
     // Reload timer values.
     timer_current = timer_initial;
     ticksInitial = bx_pc_system.time_ticks(); // Take a reading.
-    BX_DEBUG(("%s: local apic timer (periodic) triggered int, reset counter to 0x%08x", cpu->name, timer_current));
+    BX_DEBUG(("%s: local apic timer(periodic) triggered int, reset counter to 0x%08x", cpu->name, timer_current));
   }
   else {
     // one-shot mode
     timer_current = 0;
     // If timer is not masked, trigger interrupt.
-    if ((timervec & 0x10000)==0) {
-      trigger_irq (timervec & 0xff, APIC_EDGE_TRIGGERED);
+    if((timervec & 0x10000)==0) {
+      trigger_irq(timervec & 0xff, APIC_EDGE_TRIGGERED);
+    }
+    else {
+      BX_DEBUG(("%s: local apic timer LVT masked", cpu->name));
     }
     timer_active = 0;
-    BX_DEBUG (("%s: local apic timer (one-shot) triggered int", cpu->name));
+    BX_DEBUG(("%s: local apic timer(one-shot) triggered int", cpu->name));
     bx_pc_system.deactivate_timer(timer_handle);
   }
 }
 
+void bx_local_apic_c::set_divide_configuration(Bit32u value)
+{
+  BX_ASSERT(value == (value & 0x0b));
+  // move bit 3 down to bit 0.
+  value = ((value & 8) >> 1) | (value & 3);
+  BX_ASSERT(value >= 0 && value <= 7);
+  timer_divide_factor = (value==7)? 1 : (2 << value);
+  BX_INFO(("%s: set timer divide factor to %d", cpu->name, timer_divide_factor));
+}
+
 void bx_local_apic_c::set_initial_timer_count(Bit32u value)
 {
   // If active before, deactive the current timer before changing it.
-  if (timer_active) {
+  if(timer_active) {
     bx_pc_system.deactivate_timer(timer_handle);
     timer_active = 0;
   }
@@ -906,11 +920,11 @@ void bx_local_apic_c::set_initial_timer_count(Bit32u value)
   timer_initial = value;
   timer_current = 0;
 
-  if (timer_initial != 0)  // terminate the counting if timer_initial = 0
+  if(timer_initial != 0)  // terminate the counting if timer_initial = 0
   {
     // This should trigger the counter to start.  If already started,
     // restart from the new start value.
-    BX_DEBUG(("APIC: Initial Timer Count Register = %u\n", value));
+    BX_DEBUG(("APIC: Initial Timer Count Register = %u", value));
     timer_current = timer_initial;
     timer_active = 1;
     Bit32u timervec = lvt[APIC_LVT_TIMER];
@@ -921,4 +935,53 @@ void bx_local_apic_c::set_initial_timer_count(Bit32u value)
   }
 }
 
+#if BX_SUPPORT_SAVE_RESTORE
+void bx_local_apic_c::register_state(bx_param_c *parent)
+{
+  unsigned i;
+  char name[6];
+
+  bx_list_c *lapic = new bx_list_c(parent, "local_apic", 25);
+
+  BXRS_HEX_PARAM_SIMPLE(lapic, base_addr);
+  BXRS_HEX_PARAM_SIMPLE(lapic, id);
+  BXRS_HEX_PARAM_SIMPLE(lapic, spurious_vector);
+  BXRS_PARAM_BOOL(lapic, software_enabled, software_enabled);
+  BXRS_PARAM_BOOL(lapic, focus_disable, focus_disable);
+  BXRS_HEX_PARAM_SIMPLE(lapic, task_priority);
+  BXRS_HEX_PARAM_SIMPLE(lapic, spurious_vector);
+  BXRS_HEX_PARAM_SIMPLE(lapic, log_dest);
+  BXRS_HEX_PARAM_SIMPLE(lapic, dest_format);
+  
+  bx_list_c *ISR = new bx_list_c(lapic, "isr", BX_LOCAL_APIC_MAX_INTS);
+  bx_list_c *TMR = new bx_list_c(lapic, "tmr", BX_LOCAL_APIC_MAX_INTS);
+  bx_list_c *IRR = new bx_list_c(lapic, "irr", BX_LOCAL_APIC_MAX_INTS);
+  for (i=0; i<BX_LOCAL_APIC_MAX_INTS; i++) {
+    sprintf(name, "0x%02x", i);
+    new bx_shadow_num_c(ISR, name, &isr[i]);
+    new bx_shadow_num_c(TMR, name, &tmr[i]);
+    new bx_shadow_num_c(IRR, name, &irr[i]);
+  }
+
+  BXRS_HEX_PARAM_SIMPLE(lapic, error_status);
+  BXRS_HEX_PARAM_SIMPLE(lapic, shadow_error_status);
+  BXRS_HEX_PARAM_SIMPLE(lapic, icr_hi);
+  BXRS_HEX_PARAM_SIMPLE(lapic, icr_lo);
+
+  bx_list_c *LVT = new bx_list_c(lapic, "lvt", APIC_LVT_ENTRIES);
+  for (i=0; i<APIC_LVT_ENTRIES; i++) {
+    sprintf(name, "%d", i);
+    new bx_shadow_num_c(LVT, name, &lvt[i], BASE_HEX);
+  }
+
+  BXRS_HEX_PARAM_SIMPLE(lapic, timer_initial);
+  BXRS_HEX_PARAM_SIMPLE(lapic, timer_current);
+  BXRS_HEX_PARAM_SIMPLE(lapic, timer_divconf);
+  BXRS_DEC_PARAM_SIMPLE(lapic, timer_divide_factor);
+  BXRS_PARAM_BOOL(lapic, timer_active, timer_active);
+  BXRS_HEX_PARAM_SIMPLE(lapic, ticksInitial);
+  BXRS_PARAM_BOOL(lapic, INTR, INTR);
+}
+#endif
+
 #endif /* if BX_SUPPORT_APIC */
