@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: arith64.cc,v 1.26 2005/05/19 20:25:15 sshwarts Exp $
+// $Id: arith64.cc,v 1.30 2006/06/09 22:29:06 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -28,6 +28,7 @@
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
@@ -50,7 +51,7 @@ void BX_CPU_C::ADD_EqGq(bxInstruction_c *i)
     /* pointer, segment address pair */
     read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
     sum_64 = op1_64 + op2_64;
-    Write_RMW_virtual_qword(sum_64);
+    write_RMW_virtual_qword(sum_64);
   }
 
   SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_INSTR_ADD64);
@@ -114,11 +115,10 @@ void BX_CPU_C::ADC_EqGq(bxInstruction_c *i)
     /* pointer, segment address pair */
     read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
     sum_64 = op1_64 + op2_64 + temp_CF;
-    Write_RMW_virtual_qword(sum_64);
+    write_RMW_virtual_qword(sum_64);
   }
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64,
-	       (temp_CF) ? BX_INSTR_ADC64 : BX_INSTR_ADD64);
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_INSTR_ADD_ADC64(temp_CF));
 }
 
 void BX_CPU_C::ADC_GqEq(bxInstruction_c *i)
@@ -144,8 +144,7 @@ void BX_CPU_C::ADC_GqEq(bxInstruction_c *i)
   /* now write sum back to destination */
   BX_WRITE_64BIT_REG(i->nnn(), sum_64);
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64,
-	       (temp_CF) ? BX_INSTR_ADC64 : BX_INSTR_ADD64);
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_INSTR_ADD_ADC64(temp_CF));
 }
 
 void BX_CPU_C::ADC_RAXId(bxInstruction_c *i)
@@ -162,8 +161,7 @@ void BX_CPU_C::ADC_RAXId(bxInstruction_c *i)
   /* now write sum back to destination */
   RAX = sum_64;
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64,
-	       (temp_CF) ? BX_INSTR_ADC64 : BX_INSTR_ADD64);
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_INSTR_ADD_ADC64(temp_CF));
 }
 
 void BX_CPU_C::SBB_EqGq(bxInstruction_c *i)
@@ -185,11 +183,10 @@ void BX_CPU_C::SBB_EqGq(bxInstruction_c *i)
     /* pointer, segment address pair */
     read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
     diff_64 = op1_64 - (op2_64 + temp_CF);
-    Write_RMW_virtual_qword(diff_64);
+    write_RMW_virtual_qword(diff_64);
   }
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64,
-	       (temp_CF) ? BX_INSTR_SBB64 : BX_INSTR_SUB64);
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_SUB_SBB64(temp_CF));
 }
 
 void BX_CPU_C::SBB_GqEq(bxInstruction_c *i)
@@ -215,8 +212,7 @@ void BX_CPU_C::SBB_GqEq(bxInstruction_c *i)
   /* now write diff back to destination */
   BX_WRITE_64BIT_REG(i->nnn(), diff_64);
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64,
-	       (temp_CF) ? BX_INSTR_SBB64 : BX_INSTR_SUB64);
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_SUB_SBB64(temp_CF));
 }
 
 void BX_CPU_C::SBB_RAXId(bxInstruction_c *i)
@@ -233,8 +229,7 @@ void BX_CPU_C::SBB_RAXId(bxInstruction_c *i)
   /* now write diff back to destination */
   RAX = diff_64;
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64,
-	       (temp_CF) ? BX_INSTR_SBB64 : BX_INSTR_SUB64);
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_SUB_SBB64(temp_CF));
 }
 
 void BX_CPU_C::SBB_EqId(bxInstruction_c *i)
@@ -256,11 +251,10 @@ void BX_CPU_C::SBB_EqId(bxInstruction_c *i)
     /* pointer, segment address pair */
     read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
     diff_64 = op1_64 - (op2_64 + temp_CF);
-    Write_RMW_virtual_qword(diff_64);
+    write_RMW_virtual_qword(diff_64);
   }
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64,
-	       (temp_CF) ? BX_INSTR_SBB64 : BX_INSTR_SUB64);
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_SUB_SBB64(temp_CF));
 }
 
 void BX_CPU_C::SUB_EqGq(bxInstruction_c *i)
@@ -280,7 +274,7 @@ void BX_CPU_C::SUB_EqGq(bxInstruction_c *i)
     /* pointer, segment address pair */
     read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
     diff_64 = op1_64 - op2_64;
-    Write_RMW_virtual_qword(diff_64);
+    write_RMW_virtual_qword(diff_64);
   }
 
   SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_SUB64);
@@ -428,7 +422,7 @@ void BX_CPU_C::XADD_EqGq(bxInstruction_c *i)
     BX_WRITE_64BIT_REG(i->rm(), sum_64);
   }
   else {
-    Write_RMW_virtual_qword(sum_64);
+    write_RMW_virtual_qword(sum_64);
     /* and write destination into source */
     BX_WRITE_64BIT_REG(i->nnn(), op1_64);
   }
@@ -453,7 +447,7 @@ void BX_CPU_C::ADD_EqId(bxInstruction_c *i)
     /* pointer, segment address pair */
     read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
     sum_64 = op1_64 + op2_64;
-    Write_RMW_virtual_qword(sum_64);
+    write_RMW_virtual_qword(sum_64);
   }
 
   SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_INSTR_ADD64);
@@ -478,11 +472,10 @@ void BX_CPU_C::ADC_EqId(bxInstruction_c *i)
     /* pointer, segment address pair */
     read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
     sum_64 = op1_64 + op2_64 + temp_CF;
-    Write_RMW_virtual_qword(sum_64);
+    write_RMW_virtual_qword(sum_64);
   }
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64,
-	       (temp_CF) ? BX_INSTR_ADC64 : BX_INSTR_ADD64);
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_INSTR_ADD_ADC64(temp_CF));
 }
 
 void BX_CPU_C::SUB_EqId(bxInstruction_c *i)
@@ -502,7 +495,7 @@ void BX_CPU_C::SUB_EqId(bxInstruction_c *i)
     /* pointer, segment address pair */
     read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
     diff_64 = op1_64 - op2_64;
-    Write_RMW_virtual_qword(diff_64);
+    write_RMW_virtual_qword(diff_64);
   }
 
   SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_SUB64);
@@ -544,7 +537,7 @@ void BX_CPU_C::NEG_Eq(bxInstruction_c *i)
     /* pointer, segment address pair */
     read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
     diff_64 = -op1_64;
-    Write_RMW_virtual_qword(diff_64);
+    write_RMW_virtual_qword(diff_64);
   }
 
   SET_FLAGS_OSZAPC_RESULT_64(diff_64, BX_INSTR_NEG64);
@@ -564,7 +557,7 @@ void BX_CPU_C::INC_Eq(bxInstruction_c *i)
     /* pointer, segment address pair */
     read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
     op1_64++;
-    Write_RMW_virtual_qword(op1_64);
+    write_RMW_virtual_qword(op1_64);
   }
 
   SET_FLAGS_OSZAP_RESULT_64(op1_64, BX_INSTR_INC64);
@@ -584,7 +577,7 @@ void BX_CPU_C::DEC_Eq(bxInstruction_c *i)
     /* pointer, segment address pair */
     read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
     op1_64--;
-    Write_RMW_virtual_qword(op1_64);
+    write_RMW_virtual_qword(op1_64);
   }
 
   SET_FLAGS_OSZAP_RESULT_64(op1_64, BX_INSTR_DEC64);
@@ -615,7 +608,7 @@ void BX_CPU_C::CMPXCHG_EqGq(bxInstruction_c *i)
       BX_WRITE_64BIT_REG(i->rm(), op2_64);
     }
     else {
-      Write_RMW_virtual_qword(op2_64);
+      write_RMW_virtual_qword(op2_64);
     }
   }
   else {
@@ -629,12 +622,12 @@ void BX_CPU_C::CMPXCHG16B(bxInstruction_c *i)
   Bit64u op1_64_lo, op1_64_hi, diff;
 
   if (i->modC0()) {
-    BX_INFO(("CMPXCHG16B: dest is not memory location (#UD)"));
+    BX_ERROR(("CMPXCHG16B: dest is not memory location (#UD)"));
     UndefinedOpcode(i);
   }
 
   if (RMAddr(i) & 0xf) {
-    BX_INFO(("CMPXCHG16B: not aligned memory location (#GP)"));
+    BX_ERROR(("CMPXCHG16B: not aligned memory location (#GP)"));
     exception(BX_GP_EXCEPTION, 0, 0);
   }
 
@@ -648,7 +641,7 @@ void BX_CPU_C::CMPXCHG16B(bxInstruction_c *i)
     // ZF = 1
     set_ZF(1);
     // dest <-- src
-    Write_RMW_virtual_qword(RCX);
+    write_RMW_virtual_qword(RCX);
     write_virtual_qword(i->seg(), RMAddr(i), &RBX);
   }
   else {
