@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: data_xfer64.cc,v 1.22 2005/06/21 17:01:18 sshwarts Exp $
+// $Id: data_xfer64.cc,v 1.24 2006/03/26 18:58:01 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -27,6 +27,7 @@
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
@@ -247,7 +248,7 @@ void BX_CPU_C::XCHG_EqGq(bxInstruction_c *i)
   else {
     /* pointer, segment address pair */
     read_RMW_virtual_qword(i->seg(), RMAddr(i), &op1_64);
-    Write_RMW_virtual_qword(op2_64);
+    write_RMW_virtual_qword(op2_64);
   }
 
   BX_WRITE_64BIT_REG(i->nnn(), op1_64);
