@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: debugstuff.cc,v 1.55 2006/01/27 19:50:00 sshwarts Exp $
+// $Id: debugstuff.cc,v 1.73 2006/08/25 19:56:03 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -27,31 +27,48 @@
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
+
 #if BX_DISASM
 void BX_CPU_C::debug_disasm_instruction(bx_address offset)
 {
-  bx_bool valid;
-  Bit32u  phy_addr;
+  bx_phy_address phy_addr;
   Bit8u   instr_buf[16];
-  char    char_buf[256];
-  unsigned isize;
+  char    char_buf[512];
+  unsigned i=0;
 
+  static char letters[] = "0123456789ABCDEF";
   static disassembler bx_disassemble;
+  unsigned remainsInPage = 0x1000 - (offset & 0xfff);
 
-  dbg_xlate_linear2phy(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_CS) + offset,
-                       &phy_addr, &valid);
+  bx_bool valid = dbg_xlate_linear2phy(BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_CS) + offset, &phy_addr);
   if (valid && BX_CPU_THIS_PTR mem!=NULL) {
     BX_CPU_THIS_PTR mem->dbg_fetch_mem(phy_addr, 16, instr_buf);
-    isize = bx_disassemble.disasm(
+    char_buf[i++] = '>';
+    char_buf[i++] = '>';
+    char_buf[i++] = ' ';
+    unsigned isize = bx_disassemble.disasm(
         BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.d_b,
         BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64,
         BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_CS), offset,
-        instr_buf, char_buf);
-    for (unsigned j=0; j<isize; j++)
-      BX_INFO((">> %02x", (unsigned) instr_buf[j]));
-    BX_INFO((">> : %s", char_buf));
+        instr_buf, char_buf+i);
+    if (isize <= remainsInPage) {
+      i=strlen(char_buf);
+      char_buf[i++] = ' ';
+      char_buf[i++] = ':';
+      char_buf[i++] = ' ';
+      for (unsigned j=0; j<isize; j++) {
+        char_buf[i++] = letters[(instr_buf[j] >> 4) & 0xf];
+        char_buf[i++] = letters[(instr_buf[j] >> 0) & 0xf];
+      }
+      char_buf[i] = 0;
+      BX_INFO(("%s", char_buf));
+    }
+    else {
+      BX_INFO(("(instruction unavailable) page split instruction"));
+    }
   }
   else {
     BX_INFO(("(instruction unavailable) page not present"));
@@ -204,10 +221,8 @@ void BX_CPU_C::debug(bx_address offset)
     (unsigned) (BX_CPU_THIS_PTR cr0.val32), 0,
     (unsigned) (BX_CPU_THIS_PTR cr2 >> 32),
     (unsigned) (BX_CPU_THIS_PTR cr2 & 0xffffffff)));
-  BX_INFO(("| CR3=0x%08x%08x CR4=0x%08x",
-    (unsigned) (BX_CPU_THIS_PTR cr3 >> 32),
-    (unsigned) (BX_CPU_THIS_PTR cr3 & 0xffffffff),
-    BX_CPU_THIS_PTR cr4.getRegister()));
+  BX_INFO(("| CR3=0x%08x CR4=0x%08x",
+    (unsigned) BX_CPU_THIS_PTR cr3, BX_CPU_THIS_PTR cr4.getRegister()));
 #else
   BX_INFO(("| EIP=%08x (%08x)", (unsigned) EIP,
     (unsigned) BX_CPU_THIS_PTR prev_eip));
@@ -243,7 +258,7 @@ Bit32u BX_CPU_C::dbg_get_reg(unsigned reg)
   switch (reg) {
     case BX_DBG_REG_EIP: return(EIP);
     case BX_DBG_REG_EFLAGS:
-      return_val32 = dbg_get_eflags();
+      return_val32 = BX_CPU_THIS_PTR read_eflags();
       return(return_val32);
     case BX_DBG_REG_CS: return(BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value);
     case BX_DBG_REG_SS: return(BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].selector.value);
@@ -251,6 +266,16 @@ Bit32u BX_CPU_C::dbg_get_reg(unsigned reg)
     case BX_DBG_REG_ES: return(BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].selector.value);
     case BX_DBG_REG_FS: return(BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].selector.value);
     case BX_DBG_REG_GS: return(BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].selector.value);
+    case BX_DBG_REG_CR0:
+      return BX_CPU_THIS_PTR cr0.val32;
+    case BX_DBG_REG_CR2:
+      return BX_CPU_THIS_PTR cr2;
+    case BX_DBG_REG_CR3:
+      return BX_CPU_THIS_PTR cr3;
+#if BX_CPU_LEVEL >= 4
+    case BX_DBG_REG_CR4:
+      return BX_CPU_THIS_PTR cr4.getRegister();
+#endif
     default:
       BX_PANIC(("get_reg: request for unknown register"));
       return(0);
@@ -267,7 +292,7 @@ bx_bool BX_CPU_C::dbg_set_reg(unsigned reg, Bit32u val)
     case BX_DBG_REG_EIP: EIP = val; return(1);
     case BX_DBG_REG_EFLAGS:
       BX_INFO(("dbg_set_reg: can not handle eflags yet."));
-      if ( val & 0xffff0000 ) {
+      if (val & 0xffff0000) {
         BX_INFO(("dbg_set_reg: can not set upper 16 bits of eflags."));
         return(0);
       }
@@ -275,7 +300,7 @@ bx_bool BX_CPU_C::dbg_set_reg(unsigned reg, Bit32u val)
       current_sys_bits = ((BX_CPU_THIS_PTR getB_NT()) << 14) |
                          (BX_CPU_THIS_PTR get_IOPL () << 12) |
                          ((BX_CPU_THIS_PTR getB_TF()) << 8);
-      if ( current_sys_bits != (val & 0x0000f100) ) {
+      if (current_sys_bits != (val & 0x0000f100)) {
         BX_INFO(("dbg_set_reg: can not modify NT, IOPL, or TF."));
         return(0);
       }
@@ -287,7 +312,7 @@ bx_bool BX_CPU_C::dbg_set_reg(unsigned reg, Bit32u val)
       BX_CPU_THIS_PTR set_IF(val & 0x01); val >>= 1;
       BX_CPU_THIS_PTR set_DF(val & 0x01); val >>= 1;
       BX_CPU_THIS_PTR set_OF(val & 0x01);
-      if (BX_CPU_THIS_PTR get_IF ())
+      if (BX_CPU_THIS_PTR get_IF())
         BX_CPU_THIS_PTR async_event = 1;
       return(1);
     case BX_DBG_REG_CS:
@@ -319,15 +344,10 @@ bx_bool BX_CPU_C::dbg_set_reg(unsigned reg, Bit32u val)
     seg->cache.p = 1;
     seg->cache.dpl = 0;
     seg->cache.segment = 1; // regular segment
-    if (reg == BX_DBG_REG_CS) {
-      seg->cache.u.segment.executable = 1; // code segment
-    }
-    else {
-      seg->cache.u.segment.executable = 0; // data segment
-    }
-    seg->cache.u.segment.c_ed = 0;       // expand up/non-conforming
-    seg->cache.u.segment.r_w = 1;        // writeable
-    seg->cache.u.segment.a = 1;          // accessed
+    if (reg == BX_DBG_REG_CS)
+      seg->cache.type = BX_CODE_EXEC_READ_ACCESSED;
+    else
+      seg->cache.type = BX_DATA_READ_WRITE_ACCESSED;
     seg->cache.u.segment.base = val << 4;
     seg->cache.u.segment.limit        = 0xffff;
     seg->cache.u.segment.limit_scaled = 0xffff;
@@ -344,193 +364,108 @@ unsigned BX_CPU_C::dbg_query_pending(void)
 {
   unsigned ret = 0;
 
-  if ( BX_HRQ ) {  // DMA Hold Request
+  if (BX_HRQ) {  // DMA Hold Request
     ret |= BX_DBG_PENDING_DMA;
   }
 
-  if ( BX_CPU_THIS_PTR INTR && BX_CPU_THIS_PTR get_IF () ) {
+  if (BX_CPU_THIS_PTR INTR && BX_CPU_THIS_PTR get_IF()) {
     ret |= BX_DBG_PENDING_IRQ;
   }
 
   return(ret);
 }
 
-Bit32u BX_CPU_C::dbg_get_eflags(void)
-{
-  return (BX_CPU_THIS_PTR read_eflags());
-}
-
-Bit32u BX_CPU_C::dbg_get_descriptor_l(bx_descriptor_t *d)
-{
-  Bit32u val;
-
-  if (d->valid == 0) {
-    return(0);
-  }
-
-  if (d->segment) {
-    val = ((d->u.segment.base & 0xffff) << 16) |
-          (d->u.segment.limit & 0xffff);
-    return(val);
-  }
-  else {
-    switch (d->type) {
-      case 0: // Reserved (not defined)
-        BX_ERROR(("#get_descriptor_l(): type %d not finished", d->type));
-        return(0);
-
-      case BX_SYS_SEGMENT_AVAIL_286_TSS:
-      case BX_SYS_SEGMENT_BUSY_286_TSS:
-        val = ((d->u.tss286.base & 0xffff) << 16) |
-               (d->u.tss286.limit & 0xffff);
-        return(val);
-
-      case BX_SYS_SEGMENT_LDT:
-        val = ((d->u.ldt.base & 0xffff) << 16) | d->u.ldt.limit;
-        return(val);
-
-      case BX_SYS_SEGMENT_AVAIL_386_TSS:
-      case BX_SYS_SEGMENT_BUSY_386_TSS:
-        val = ((d->u.tss386.base & 0xffff) << 16) |
-               (d->u.tss386.limit & 0xffff);
-        return(val);
-
-      default:
-        BX_ERROR(("#get_descriptor_l(): type %d not finished", d->type));
-        return(0);
-    }
-  }
-}
-
-Bit32u BX_CPU_C::dbg_get_descriptor_h(bx_descriptor_t *d)
-{
-  Bit32u val;
-
-  if (d->valid == 0) {
-    return(0);
-  }
-
-  if (d->segment) {
-    val = (d->u.segment.base & 0xff000000) |
-          ((d->u.segment.base >> 16) & 0x000000ff) |
-          (d->u.segment.executable << 11) |
-          (d->u.segment.c_ed << 10) |
-          (d->u.segment.r_w << 9) |
-          (d->u.segment.a << 8) |
-          (d->segment << 12) |
-          (d->dpl << 13) |
-          (d->p << 15) |
-          (d->u.segment.limit & 0xf0000) |
-          (d->u.segment.avl << 20) |
-          (d->u.segment.d_b << 22) |
-          (d->u.segment.g << 23);
-    return(val);
-  }
-  else {
-    switch (d->type) {
-      case 0: // Reserved (not yet defined)
-        BX_ERROR(("#get_descriptor_h(): type %d not finished", d->type));
-        return(0);
-
-      case BX_SYS_SEGMENT_AVAIL_286_TSS:
-      case BX_SYS_SEGMENT_BUSY_286_TSS:
-        val = ((d->u.tss286.base >> 16) & 0xff) |
-              (d->type << 8) |
-              (d->dpl << 13) |
-              (d->p << 15);
-        return(val);
-
-      case BX_SYS_SEGMENT_LDT:
-        val = ((d->u.ldt.base >> 16) & 0xff) |
-              (d->type << 8) |
-              (d->dpl << 13) |
-              (d->p << 15) |
-              (d->u.ldt.base & 0xff000000);
-        return(val);
-
-      case BX_SYS_SEGMENT_AVAIL_386_TSS:
-      case BX_SYS_SEGMENT_BUSY_386_TSS:
-        val = ((d->u.tss386.base >> 16) & 0xff) |
-              (d->type << 8) |
-              (d->dpl << 13) |
-              (d->p << 15) |
-              (d->u.tss386.limit & 0xf0000) |
-              (d->u.tss386.avl << 20) |
-              (d->u.tss386.g << 23) |
-              (d->u.tss386.base & 0xff000000);
-        return(val);
-
-      default:
-        BX_ERROR(("#get_descriptor_h(): type %d not finished", d->type));
-        return(0);
-    }
-  }
-}
-
 bx_bool BX_CPU_C::dbg_get_sreg(bx_dbg_sreg_t *sreg, unsigned sreg_no)
 {
   if (sreg_no > 5)
     return(0);
   sreg->sel   = BX_CPU_THIS_PTR sregs[sreg_no].selector.value;
-  sreg->des_l = dbg_get_descriptor_l(&BX_CPU_THIS_PTR sregs[sreg_no].cache);
-  sreg->des_h = dbg_get_descriptor_h(&BX_CPU_THIS_PTR sregs[sreg_no].cache);
+  sreg->des_l = get_descriptor_l(&BX_CPU_THIS_PTR sregs[sreg_no].cache);
+  sreg->des_h = get_descriptor_h(&BX_CPU_THIS_PTR sregs[sreg_no].cache);
   sreg->valid = BX_CPU_THIS_PTR sregs[sreg_no].cache.valid;
   return(1);
 }
 
+void BX_CPU_C::dbg_get_tr(bx_dbg_sreg_t *sreg)
+{
+  sreg->sel   = BX_CPU_THIS_PTR tr.selector.value;
+  sreg->des_l = get_descriptor_l(&BX_CPU_THIS_PTR tr.cache);
+  sreg->des_h = get_descriptor_h(&BX_CPU_THIS_PTR tr.cache);
+  sreg->valid = BX_CPU_THIS_PTR tr.cache.valid;
+}
+
+void BX_CPU_C::dbg_get_ldtr(bx_dbg_sreg_t *sreg)
+{
+  sreg->sel   = BX_CPU_THIS_PTR ldtr.selector.value;
+  sreg->des_l = get_descriptor_l(&BX_CPU_THIS_PTR ldtr.cache);
+  sreg->des_h = get_descriptor_h(&BX_CPU_THIS_PTR ldtr.cache);
+  sreg->valid = BX_CPU_THIS_PTR ldtr.cache.valid;
+}
+
+void BX_CPU_C::dbg_get_gdtr(bx_dbg_global_sreg_t *sreg)
+{
+  sreg->base  = BX_CPU_THIS_PTR gdtr.base;
+  sreg->limit = BX_CPU_THIS_PTR gdtr.limit;
+}
+
+void BX_CPU_C::dbg_get_idtr(bx_dbg_global_sreg_t *sreg)
+{
+  sreg->base  = BX_CPU_THIS_PTR idtr.base;
+  sreg->limit = BX_CPU_THIS_PTR idtr.limit;
+}
+
 bx_bool BX_CPU_C::dbg_get_cpu(bx_dbg_cpu_t *cpu)
 {
   cpu->eax = EAX;
   cpu->ebx = EBX;
   cpu->ecx = ECX;
   cpu->edx = EDX;
-
   cpu->ebp = EBP;
   cpu->esi = ESI;
   cpu->edi = EDI;
   cpu->esp = ESP;
+  cpu->eip = EIP;
 
-  cpu->eflags = dbg_get_eflags();
-  cpu->eip    = EIP;
+  cpu->eflags = BX_CPU_THIS_PTR read_eflags();
 
   cpu->cs.sel   = BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value;
-  cpu->cs.des_l = dbg_get_descriptor_l(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache);
-  cpu->cs.des_h = dbg_get_descriptor_h(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache);
+  cpu->cs.des_l = get_descriptor_l(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache);
+  cpu->cs.des_h = get_descriptor_h(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache);
   cpu->cs.valid = BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.valid;
 
   cpu->ss.sel   = BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].selector.value;
-  cpu->ss.des_l = dbg_get_descriptor_l(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache);
-  cpu->ss.des_h = dbg_get_descriptor_h(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache);
+  cpu->ss.des_l = get_descriptor_l(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache);
+  cpu->ss.des_h = get_descriptor_h(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache);
   cpu->ss.valid = BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.valid;
 
   cpu->ds.sel   = BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].selector.value;
-  cpu->ds.des_l = dbg_get_descriptor_l(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache);
-  cpu->ds.des_h = dbg_get_descriptor_h(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache);
+  cpu->ds.des_l = get_descriptor_l(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache);
+  cpu->ds.des_h = get_descriptor_h(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache);
   cpu->ds.valid = BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.valid;
 
   cpu->es.sel   = BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].selector.value;
-  cpu->es.des_l = dbg_get_descriptor_l(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache);
-  cpu->es.des_h = dbg_get_descriptor_h(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache);
+  cpu->es.des_l = get_descriptor_l(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache);
+  cpu->es.des_h = get_descriptor_h(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache);
   cpu->es.valid = BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.valid;
 
   cpu->fs.sel   = BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].selector.value;
-  cpu->fs.des_l = dbg_get_descriptor_l(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache);
-  cpu->fs.des_h = dbg_get_descriptor_h(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache);
+  cpu->fs.des_l = get_descriptor_l(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache);
+  cpu->fs.des_h = get_descriptor_h(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache);
   cpu->fs.valid = BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.valid;
 
   cpu->gs.sel   = BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].selector.value;
-  cpu->gs.des_l = dbg_get_descriptor_l(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache);
-  cpu->gs.des_h = dbg_get_descriptor_h(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache);
+  cpu->gs.des_l = get_descriptor_l(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache);
+  cpu->gs.des_h = get_descriptor_h(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache);
   cpu->gs.valid = BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.valid;
 
   cpu->ldtr.sel   = BX_CPU_THIS_PTR ldtr.selector.value;
-  cpu->ldtr.des_l = dbg_get_descriptor_l(&BX_CPU_THIS_PTR ldtr.cache);
-  cpu->ldtr.des_h = dbg_get_descriptor_h(&BX_CPU_THIS_PTR ldtr.cache);
+  cpu->ldtr.des_l = get_descriptor_l(&BX_CPU_THIS_PTR ldtr.cache);
+  cpu->ldtr.des_h = get_descriptor_h(&BX_CPU_THIS_PTR ldtr.cache);
   cpu->ldtr.valid = BX_CPU_THIS_PTR ldtr.cache.valid;
 
   cpu->tr.sel   = BX_CPU_THIS_PTR tr.selector.value;
-  cpu->tr.des_l = dbg_get_descriptor_l(&BX_CPU_THIS_PTR tr.cache);
-  cpu->tr.des_h = dbg_get_descriptor_h(&BX_CPU_THIS_PTR tr.cache);
+  cpu->tr.des_l = get_descriptor_l(&BX_CPU_THIS_PTR tr.cache);
+  cpu->tr.des_h = get_descriptor_h(&BX_CPU_THIS_PTR tr.cache);
   cpu->tr.valid = BX_CPU_THIS_PTR tr.cache.valid;
 
   cpu->gdtr.base  = BX_CPU_THIS_PTR gdtr.base;
@@ -546,14 +481,7 @@ bx_bool BX_CPU_C::dbg_get_cpu(bx_dbg_cpu_t *cpu)
   cpu->dr6 = BX_CPU_THIS_PTR dr6;
   cpu->dr7 = BX_CPU_THIS_PTR dr7;
 
-  cpu->tr3 = 0;
-  cpu->tr4 = 0;
-  cpu->tr5 = 0;
-  cpu->tr6 = 0;
-  cpu->tr7 = 0;
-
 #if BX_CPU_LEVEL >= 2
-  // cr0:32=pg,cd,nw,am,wp,ne,ts,em,mp,pe
   cpu->cr0 = BX_CPU_THIS_PTR cr0.val32;
   cpu->cr1 = 0;
   cpu->cr2 = BX_CPU_THIS_PTR cr2;
@@ -571,7 +499,6 @@ bx_bool BX_CPU_C::dbg_get_cpu(bx_dbg_cpu_t *cpu)
 bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
 {
   // returns 1=OK, 0=Error
-  Bit32u val;
   Bit32u type;
 
   // =================================================
@@ -580,75 +507,75 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
 
   // CS, SS, DS, ES, FS, GS descriptor checks
   if (!cpu->cs.valid) {
-    BX_ERROR(( "Error: CS not valid" ));
+    BX_ERROR(("Error: CS not valid"));
     return(0); // error
   }
-  if ( (cpu->cs.des_h & 0x1000) == 0 ) {
-    BX_ERROR(( "Error: CS not application type" ));
+  if ((cpu->cs.des_h & 0x1000) == 0) {
+    BX_ERROR(("Error: CS not application type"));
     return(0); // error
   }
-  if ( (cpu->cs.des_h & 0x0800) == 0 ) {
-    BX_ERROR(( "Error: CS not executable" ));
+  if ((cpu->cs.des_h & 0x0800) == 0) {
+    BX_ERROR(("Error: CS not executable"));
     return(0); // error
   }
 
   if (!cpu->ss.valid) {
-    BX_ERROR(( "Error: SS not valid" ));
+    BX_ERROR(("Error: SS not valid"));
     return(0); // error
   }
-  if ( (cpu->ss.des_h & 0x1000) == 0 ) {
-    BX_ERROR(( "Error: SS not application type" ));
+  if ((cpu->ss.des_h & 0x1000) == 0) {
+    BX_ERROR(("Error: SS not application type"));
     return(0); // error
   }
 
   if (cpu->ds.valid) {
-    if ( (cpu->ds.des_h & 0x1000) == 0 ) {
-      BX_ERROR(( "Error: DS not application type" ));
+    if ((cpu->ds.des_h & 0x1000) == 0) {
+      BX_ERROR(("Error: DS not application type"));
       return(0); // error
     }
   }
 
   if (cpu->es.valid) {
-    if ( (cpu->es.des_h & 0x1000) == 0 ) {
-      BX_ERROR(( "Error: ES not application type" ));
+    if ((cpu->es.des_h & 0x1000) == 0) {
+      BX_ERROR(("Error: ES not application type"));
       return(0); // error
     }
   }
 
   if (cpu->fs.valid) {
-    if ( (cpu->fs.des_h & 0x1000) == 0 ) {
-      BX_ERROR(( "Error: FS not application type" ));
+    if ((cpu->fs.des_h & 0x1000) == 0) {
+      BX_ERROR(("Error: FS not application type"));
       return(0); // error
     }
   }
 
   if (cpu->gs.valid) {
-    if ( (cpu->gs.des_h & 0x1000) == 0 ) {
-      BX_ERROR(( "Error: GS not application type" ));
+    if ((cpu->gs.des_h & 0x1000) == 0) {
+      BX_ERROR(("Error: GS not application type"));
       return(0); // error
     }
   }
 
   if (cpu->ldtr.valid) {
-    if ( cpu->ldtr.des_h & 0x1000 ) {
-      BX_ERROR(( "Error: LDTR not system type" ));
+    if (cpu->ldtr.des_h & 0x1000) {
+      BX_ERROR(("Error: LDTR not system type"));
       return(0); // error
     }
-    if ( ((cpu->ldtr.des_h >> 8) & 0x0f) != 2 ) {
-      BX_ERROR(( "Error: LDTR descriptor type not LDT" ));
+    if (((cpu->ldtr.des_h >> 8) & 0x0f) != BX_SYS_SEGMENT_LDT) {
+      BX_ERROR(("Error: LDTR descriptor type not LDT"));
       return(0); // error
     }
   }
 
   if (cpu->tr.valid) {
-    if ( cpu->tr.des_h & 0x1000 ) {
-      BX_ERROR(( "Error: TR not system type"));
+    if (cpu->tr.des_h & 0x1000) {
+      BX_ERROR(("Error: TR not system type"));
       return(0); // error
     }
     type = (cpu->tr.des_h >> 8) & 0x0f;
 
-    if ( (type != 1) && (type != 9) ) {
-      BX_ERROR(( "Error: TR descriptor type not TSS" ));
+    if ((type != 1) && (type != 9)) {
+      BX_ERROR(("Error: TR descriptor type not TSS"));
       return(0); // error
     }
   }
@@ -665,33 +592,10 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
   ESI = cpu->esi;
   EDI = cpu->edi;
   ESP = cpu->esp;
-
-  // eflags
-  val = cpu->eflags;
-  BX_CPU_THIS_PTR set_CF(val & 0x01); val >>= 2;
-  BX_CPU_THIS_PTR set_PF(val & 0x01); val >>= 2;
-  BX_CPU_THIS_PTR set_AF(val & 0x01); val >>= 2;
-  BX_CPU_THIS_PTR set_ZF(val & 0x01); val >>= 1;
-  BX_CPU_THIS_PTR set_SF(val & 0x01); val >>= 1;
-  BX_CPU_THIS_PTR set_TF (val & 0x01); val >>= 1;
-  BX_CPU_THIS_PTR set_IF (val & 0x01); val >>= 1;
-  BX_CPU_THIS_PTR set_DF (val & 0x01); val >>= 1;
-  BX_CPU_THIS_PTR set_OF(val & 0x01); val >>= 1;
-  BX_CPU_THIS_PTR set_IOPL (val & 0x03); val >>= 2;
-  BX_CPU_THIS_PTR set_NT (val & 0x01); val >>= 2;
-  BX_CPU_THIS_PTR set_RF (val & 0x01); val >>= 1;
-  BX_CPU_THIS_PTR set_VM (val & 0x01); val >>= 1;
-#if BX_CPU_LEVEL >= 4
-  BX_CPU_THIS_PTR set_AC (val & 0x01); val >>= 1;
-  //BX_CPU_THIS_PTR eflags.set_VIF (val & 0x01);
-  val >>= 1;
-  //BX_CPU_THIS_PTR eflags.set_VIP (val & 0x01);
-  val >>= 1;
-  BX_CPU_THIS_PTR set_ID (val & 0x01);
-#endif
-
   EIP = cpu->eip;
 
+  setEFlags(cpu->eflags);
+
   // CS:
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value = cpu->cs.sel;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.index = cpu->cs.sel >> 3;
@@ -703,10 +607,6 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.dpl              = (cpu->cs.des_h >> 13) & 0x03;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.segment          = (cpu->cs.des_h >> 12) & 0x01;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.type             = (cpu->cs.des_h >> 8) & 0x0f;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.executable = (cpu->cs.des_h >> 11) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.c_ed   = (cpu->cs.des_h >> 10) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.r_w    = (cpu->cs.des_h >> 9) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.a      = (cpu->cs.des_h >> 8) & 0x01;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.base   = (cpu->cs.des_l >> 16);
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.base  |= (cpu->cs.des_h & 0xff) << 16;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.base  |= (cpu->cs.des_h & 0xff000000);
@@ -723,7 +623,7 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
       BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit;
 
 #if BX_SUPPORT_ICACHE
-  BX_CPU_THIS_PTR iCache.fetchModeMask = createFetchModeMask(BX_CPU_THIS);
+  BX_CPU_THIS_PTR updateFetchModeMask();
 #endif
 
   // SS:
@@ -737,10 +637,6 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.dpl              = (cpu->ss.des_h >> 13) & 0x03;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.segment          = (cpu->ss.des_h >> 12) & 0x01;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.type             = (cpu->ss.des_h >> 8) & 0x0f;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.executable = (cpu->ss.des_h >> 11) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.c_ed   = (cpu->ss.des_h >> 10) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.r_w    = (cpu->ss.des_h >> 9) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.a      = (cpu->ss.des_h >> 8) & 0x01;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.base   = (cpu->ss.des_l >> 16);
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.base  |= (cpu->ss.des_h & 0xff) << 16;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.base  |= (cpu->ss.des_h & 0xff000000);
@@ -756,7 +652,6 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
     BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.limit_scaled =
       BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.limit;
 
-
   // DS:
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].selector.value = cpu->ds.sel;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].selector.index = cpu->ds.sel >> 3;
@@ -768,10 +663,6 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.dpl              = (cpu->ds.des_h >> 13) & 0x03;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.segment          = (cpu->ds.des_h >> 12) & 0x01;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.type             = (cpu->ds.des_h >> 8) & 0x0f;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.executable = (cpu->ds.des_h >> 11) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.c_ed   = (cpu->ds.des_h >> 10) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.r_w    = (cpu->ds.des_h >> 9) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.a      = (cpu->ds.des_h >> 8) & 0x01;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.base   = (cpu->ds.des_l >> 16);
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.base  |= (cpu->ds.des_h & 0xff) << 16;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.base  |= (cpu->ds.des_h & 0xff000000);
@@ -787,7 +678,6 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
     BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.limit_scaled =
       BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.limit;
 
-
   // ES:
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].selector.value = cpu->es.sel;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].selector.index = cpu->es.sel >> 3;
@@ -799,10 +689,6 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.dpl              = (cpu->es.des_h >> 13) & 0x03;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.segment          = (cpu->es.des_h >> 12) & 0x01;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.type             = (cpu->es.des_h >> 8) & 0x0f;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.executable = (cpu->es.des_h >> 11) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.c_ed   = (cpu->es.des_h >> 10) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.r_w    = (cpu->es.des_h >> 9) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.a      = (cpu->es.des_h >> 8) & 0x01;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.base   = (cpu->es.des_l >> 16);
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.base  |= (cpu->es.des_h & 0xff) << 16;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.base  |= (cpu->es.des_h & 0xff000000);
@@ -818,7 +704,6 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
     BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.limit_scaled =
       BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.limit;
 
-
   // FS:
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].selector.value = cpu->fs.sel;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].selector.index = cpu->fs.sel >> 3;
@@ -830,10 +715,6 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.dpl              = (cpu->fs.des_h >> 13) & 0x03;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.segment          = (cpu->fs.des_h >> 12) & 0x01;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.type             = (cpu->fs.des_h >> 8) & 0x0f;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.executable = (cpu->fs.des_h >> 11) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.c_ed   = (cpu->fs.des_h >> 10) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.r_w    = (cpu->fs.des_h >> 9) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.a      = (cpu->fs.des_h >> 8) & 0x01;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.base   = (cpu->fs.des_l >> 16);
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.base  |= (cpu->fs.des_h & 0xff) << 16;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.base  |= (cpu->fs.des_h & 0xff000000);
@@ -849,7 +730,6 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
     BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.limit_scaled =
       BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.limit;
 
-
   // GS:
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].selector.value = cpu->gs.sel;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].selector.index = cpu->gs.sel >> 3;
@@ -861,10 +741,6 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.dpl              = (cpu->gs.des_h >> 13) & 0x03;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.segment          = (cpu->gs.des_h >> 12) & 0x01;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.type             = (cpu->gs.des_h >> 8) & 0x0f;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.executable = (cpu->gs.des_h >> 11) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.c_ed   = (cpu->gs.des_h >> 10) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.r_w    = (cpu->gs.des_h >> 9) & 0x01;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.a      = (cpu->gs.des_h >> 8) & 0x01;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.base   = (cpu->gs.des_l >> 16);
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.base  |= (cpu->gs.des_h & 0xff) << 16;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.base  |= (cpu->gs.des_h & 0xff000000);
@@ -880,21 +756,31 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
     BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.limit_scaled =
       BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.limit;
 
-  // LDTR:
+  // LDTR
   BX_CPU_THIS_PTR ldtr.selector.value = cpu->ldtr.sel;
   BX_CPU_THIS_PTR ldtr.selector.index = cpu->ldtr.sel >> 3;
   BX_CPU_THIS_PTR ldtr.selector.ti    = (cpu->ldtr.sel >> 2) & 0x01;
   BX_CPU_THIS_PTR ldtr.selector.rpl   = cpu->ldtr.sel & 0x03;
 
-  BX_CPU_THIS_PTR ldtr.cache.valid            = cpu->ldtr.valid;
-  BX_CPU_THIS_PTR ldtr.cache.p                = (cpu->ldtr.des_h >> 15) & 0x01;
-  BX_CPU_THIS_PTR ldtr.cache.dpl              = (cpu->ldtr.des_h >> 13) & 0x03;
-  BX_CPU_THIS_PTR ldtr.cache.segment          = (cpu->ldtr.des_h >> 12) & 0x01;
-  BX_CPU_THIS_PTR ldtr.cache.type             = (cpu->ldtr.des_h >> 8) & 0x0f;
-  BX_CPU_THIS_PTR ldtr.cache.u.ldt.base       = (cpu->ldtr.des_l >> 16);
-  BX_CPU_THIS_PTR ldtr.cache.u.ldt.base      |= (cpu->ldtr.des_h & 0xff) << 16;
-  BX_CPU_THIS_PTR ldtr.cache.u.ldt.base      |= (cpu->ldtr.des_h & 0xff000000);
-  BX_CPU_THIS_PTR ldtr.cache.u.ldt.limit      = (cpu->ldtr.des_l & 0xffff);
+  BX_CPU_THIS_PTR ldtr.cache.valid        = cpu->ldtr.valid;
+  BX_CPU_THIS_PTR ldtr.cache.p            = (cpu->ldtr.des_h >> 15) & 0x01;
+  BX_CPU_THIS_PTR ldtr.cache.dpl          = (cpu->ldtr.des_h >> 13) & 0x03;
+  BX_CPU_THIS_PTR ldtr.cache.segment      = (cpu->ldtr.des_h >> 12) & 0x01;
+  BX_CPU_THIS_PTR ldtr.cache.type         = (cpu->ldtr.des_h >> 8) & 0x0f;
+  BX_CPU_THIS_PTR ldtr.cache.u.ldt.base   = (cpu->ldtr.des_l >> 16);
+  BX_CPU_THIS_PTR ldtr.cache.u.ldt.base  |= (cpu->ldtr.des_h & 0xff) << 16;
+  BX_CPU_THIS_PTR ldtr.cache.u.ldt.base  |= (cpu->ldtr.des_h & 0xff000000);
+  BX_CPU_THIS_PTR ldtr.cache.u.ldt.limit  = (cpu->ldtr.des_l & 0xffff);
+  BX_CPU_THIS_PTR ldtr.cache.u.ldt.limit |= (cpu->ldtr.des_h & 0x000f0000);
+  BX_CPU_THIS_PTR ldtr.cache.u.ldt.g      = (cpu->ldtr.des_h >> 23) & 0x01;
+  BX_CPU_THIS_PTR ldtr.cache.u.ldt.avl    = (cpu->ldtr.des_h >> 20) & 0x01;
+
+  if (BX_CPU_THIS_PTR ldtr.cache.u.ldt.g)
+    BX_CPU_THIS_PTR ldtr.cache.u.ldt.limit_scaled = 
+       (BX_CPU_THIS_PTR ldtr.cache.u.ldt.limit << 12) | 0x0fff;
+  else
+    BX_CPU_THIS_PTR ldtr.cache.u.ldt.limit_scaled = 
+       (BX_CPU_THIS_PTR ldtr.cache.u.ldt.limit);
 
   // TR
   type = (cpu->tr.des_h >> 8) & 0x0f;
@@ -904,32 +790,40 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
   BX_CPU_THIS_PTR tr.selector.ti    = (cpu->tr.sel >> 2) & 0x01;
   BX_CPU_THIS_PTR tr.selector.rpl   = cpu->tr.sel & 0x03;
 
-  BX_CPU_THIS_PTR tr.cache.valid            = cpu->tr.valid;
-  BX_CPU_THIS_PTR tr.cache.p                = (cpu->tr.des_h >> 15) & 0x01;
-  BX_CPU_THIS_PTR tr.cache.dpl              = (cpu->tr.des_h >> 13) & 0x03;
-  BX_CPU_THIS_PTR tr.cache.segment          = (cpu->tr.des_h >> 12) & 0x01;
-  BX_CPU_THIS_PTR tr.cache.type             = type;
-  if (type == 1) { // 286 TSS
-    BX_CPU_THIS_PTR tr.cache.u.tss286.base   = (cpu->tr.des_l >> 16);
-    BX_CPU_THIS_PTR tr.cache.u.tss286.base  |= (cpu->tr.des_h & 0xff) << 16;
-    BX_CPU_THIS_PTR tr.cache.u.tss286.limit  = (cpu->tr.des_l & 0xffff);
-    }
-  else { // type == 9, 386 TSS
-    BX_CPU_THIS_PTR tr.cache.u.tss386.base   = (cpu->tr.des_l >> 16);
-    BX_CPU_THIS_PTR tr.cache.u.tss386.base  |= (cpu->tr.des_h & 0xff) << 16;
-    BX_CPU_THIS_PTR tr.cache.u.tss386.base  |= (cpu->tr.des_h & 0xff000000);
-    BX_CPU_THIS_PTR tr.cache.u.tss386.limit  = (cpu->tr.des_l & 0xffff);
-    BX_CPU_THIS_PTR tr.cache.u.tss386.limit |= (cpu->tr.des_h & 0x000f0000);
-    BX_CPU_THIS_PTR tr.cache.u.tss386.g      = (cpu->tr.des_h >> 23) & 0x01;
-    BX_CPU_THIS_PTR tr.cache.u.tss386.avl    = (cpu->tr.des_h >> 20) & 0x01;
-    }
+  BX_CPU_THIS_PTR tr.cache.valid          = cpu->tr.valid;
+  BX_CPU_THIS_PTR tr.cache.p              = (cpu->tr.des_h >> 15) & 0x01;
+  BX_CPU_THIS_PTR tr.cache.dpl            = (cpu->tr.des_h >> 13) & 0x03;
+  BX_CPU_THIS_PTR tr.cache.segment        = (cpu->tr.des_h >> 12) & 0x01;
+  BX_CPU_THIS_PTR tr.cache.type           = type;
+  if (type == BX_SYS_SEGMENT_AVAIL_286_TSS) {
+    BX_CPU_THIS_PTR tr.cache.u.tss.base   = (cpu->tr.des_l >> 16);
+    BX_CPU_THIS_PTR tr.cache.u.tss.base  |= (cpu->tr.des_h & 0xff) << 16;
+    BX_CPU_THIS_PTR tr.cache.u.tss.limit  = (cpu->tr.des_l & 0xffff);
+    BX_CPU_THIS_PTR tr.cache.u.tss.g      = 0;
+    BX_CPU_THIS_PTR tr.cache.u.tss.avl    = 0;
+  }
+  else { // type == BX_SYS_SEGMENT_AVAIL_386_TSS
+    BX_CPU_THIS_PTR tr.cache.u.tss.base   = (cpu->tr.des_l >> 16);
+    BX_CPU_THIS_PTR tr.cache.u.tss.base  |= (cpu->tr.des_h & 0xff) << 16;
+    BX_CPU_THIS_PTR tr.cache.u.tss.base  |= (cpu->tr.des_h & 0xff000000);
+    BX_CPU_THIS_PTR tr.cache.u.tss.limit  = (cpu->tr.des_l & 0xffff);
+    BX_CPU_THIS_PTR tr.cache.u.tss.limit |= (cpu->tr.des_h & 0x000f0000);
+    BX_CPU_THIS_PTR tr.cache.u.tss.g      = (cpu->tr.des_h >> 23) & 0x01;
+    BX_CPU_THIS_PTR tr.cache.u.tss.avl    = (cpu->tr.des_h >> 20) & 0x01;
+  }
 
+  if (BX_CPU_THIS_PTR tr.cache.u.tss.g)
+    BX_CPU_THIS_PTR tr.cache.u.tss.limit_scaled = 
+       (BX_CPU_THIS_PTR tr.cache.u.tss.limit << 12) | 0x0fff;
+  else
+    BX_CPU_THIS_PTR tr.cache.u.tss.limit_scaled = 
+       (BX_CPU_THIS_PTR tr.cache.u.tss.limit);
 
-  // gdtr
+  // GDTR
   BX_CPU_THIS_PTR gdtr.base  = cpu->gdtr.base;
   BX_CPU_THIS_PTR gdtr.limit = cpu->gdtr.limit;
 
-  // idtr
+  // IDTR
   BX_CPU_THIS_PTR idtr.base  = cpu->idtr.base;
   BX_CPU_THIS_PTR idtr.limit = cpu->idtr.limit;
 
@@ -941,22 +835,14 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
   BX_CPU_THIS_PTR dr6 = cpu->dr6;
   BX_CPU_THIS_PTR dr7 = cpu->dr7;
 
-  // BX_CPU_THIS_PTR tr3 = cpu->tr3;
-  // BX_CPU_THIS_PTR tr4 = cpu->tr4;
-  // BX_CPU_THIS_PTR tr5 = cpu->tr5;
-  // BX_CPU_THIS_PTR tr6 = cpu->tr6;
-  // BX_CPU_THIS_PTR tr7 = cpu->tr7;
-
-
 #if BX_CPU_LEVEL >= 2
-  // cr0, cr1, cr2, cr3, cr4
+  // Control registers
   SetCR0(cpu->cr0);
-  BX_CPU_THIS_PTR cr1 = cpu->cr1;
   BX_CPU_THIS_PTR cr2 = cpu->cr2;
-  BX_CPU_THIS_PTR cr3 = cpu->cr3;
-#endif
+  CR3_change(cpu->cr3);
 #if BX_CPU_LEVEL >= 4
   BX_CPU_THIS_PTR cr4.setRegister(cpu->cr4);
+#endif
 #endif
 
   BX_CPU_THIS_PTR inhibit_mask = cpu->inhibit_mask;
@@ -964,7 +850,6 @@ bx_bool BX_CPU_C::dbg_set_cpu(bx_dbg_cpu_t *cpu)
   //
   // flush cached items, prefetch, paging, etc
   //
-  BX_CPU_THIS_PTR CR3_change(cpu->cr3);
   BX_CPU_THIS_PTR invalidate_prefetch_q();
   BX_CPU_THIS_PTR async_event = 1;
 
