@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: icache.h,v 1.11 2006/01/16 19:22:28 sshwarts Exp $
+// $Id: icache.h,v 1.17 2006/05/19 20:04:33 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -28,14 +28,15 @@
 #ifndef BX_ICACHE_H
 #define BX_ICACHE_H
 
+#include <assert.h>
+
 // bit31: 1=CS is 32/64-bit, 0=CS is 16-bit.
 // bit30: 1=Long Mode, 0=not Long Mode.
-// bit29: 1=iCache page, 0=Data.
-const Bit32u ICacheWriteStampInvalid = 0x1fffffff;
-const Bit32u ICacheWriteStampMax     = 0x1fffffff;
-const Bit32u ICacheWriteStampMask    = 0x1fffffff;
+// Combination bit31=1 & bit30=1 is invalid.
+const Bit32u ICacheWriteStampInvalid = 0xffffffff;
+const Bit32u ICacheWriteStampStart   = 0x3fffffff;
+const Bit32u ICacheWriteStampMask    = 0x3fffffff;
 const Bit32u ICacheFetchModeMask     = ~ICacheWriteStampMask;
-const Bit32u iCachePageDataMask      = 0x20000000;
 
 class bxPageWriteStampTable
 {
@@ -59,7 +60,7 @@ public:
     resetWriteStamps();
   }
 
-  BX_CPP_INLINE Bit32u getPageWriteStamp(Bit32u pAddr) const
+  BX_CPP_INLINE Bit32u getPageWriteStamp(bx_phy_address pAddr) const
   {
     if (pAddr < memSizeInBytes) 
        return pageWriteStampTable[pAddr>>12];
@@ -67,7 +68,7 @@ public:
        return ICacheWriteStampInvalid;
   }
 
-  BX_CPP_INLINE const Bit32u *getPageWriteStampPtr(Bit32u pAddr) const
+  BX_CPP_INLINE const Bit32u *getPageWriteStampPtr(bx_phy_address pAddr) const
   {
     if (pAddr < memSizeInBytes) 
        return &pageWriteStampTable[pAddr>>12];
@@ -75,26 +76,22 @@ public:
        return &ICacheWriteStampInvalid;
   }
 
-  BX_CPP_INLINE void setPageWriteStamp(Bit32u pAddr, Bit32u pageWriteStamp)
+  BX_CPP_INLINE void setPageWriteStamp(bx_phy_address pAddr, Bit32u pageWriteStamp)
   {
-    if (pAddr < memSizeInBytes) 
+    if (pAddr < memSizeInBytes)
        pageWriteStampTable[pAddr>>12] = pageWriteStamp;
   }
 
-  BX_CPP_INLINE void decWriteStamp(Bit32u a20Addr)
+  BX_CPP_INLINE void decWriteStamp(bx_phy_address pAddr)
   {
-    // Increment page write stamp, so iCache entries with older stamps
+    assert(pAddr < memSizeInBytes);
+    // Decrement page write stamp, so iCache entries with older stamps
     // are effectively invalidated.
-    Bit32u pageIndex = a20Addr >> 12;
-    Bit32u writeStamp = pageWriteStampTable[pageIndex];
-
-    if (writeStamp & iCachePageDataMask)
-    {
-      pageWriteStampTable[pageIndex]--;
-    }
+    pageWriteStampTable[pAddr >> 12]--;
   }
 
   BX_CPP_INLINE void resetWriteStamps(void);
+  BX_CPP_INLINE void purgeWriteStamps(void);
 };
 
 BX_CPP_INLINE void bxPageWriteStampTable::resetWriteStamps(void)
@@ -104,35 +101,33 @@ BX_CPP_INLINE void bxPageWriteStampTable::resetWriteStamps(void)
   }
 }
 
+BX_CPP_INLINE void bxPageWriteStampTable::purgeWriteStamps(void)
+{
+  for (Bit32u i=0; i<(memSizeInBytes>>12); i++) {
+    pageWriteStampTable[i] |= ICacheWriteStampMask;
+  }
+}
+
 extern bxPageWriteStampTable pageWriteStampTable;
 
 #define BxICacheEntries (32 * 1024)  // Must be a power of 2.
 
-class bxICacheEntry_c {
-  public:
-
-  Bit32u pAddr;       // Physical address of the instruction.
-  Bit32u writeStamp;  // Generation ID.  Each write to a physical page
-                      // decrements this value.
-  bxInstruction_c i;  // The instruction decode information.
+struct bxICacheEntry_c
+{
+  bx_phy_address pAddr; // Physical address of the instruction
+  Bit32u writeStamp;    // Generation ID. Each write to a physical page
+                        // decrements this value
+  bxInstruction_c i;    // The instruction decode information
 };
 
 class BOCHSAPI bxICache_c {
 public:
   bxICacheEntry_c entry[BxICacheEntries];
-  Bit32u fetchModeMask;
 
 public:
-  bxICache_c()
-  {
-    // Initially clear the iCache;
-    for (unsigned i=0; i<BxICacheEntries; i++) {
-      entry[i].writeStamp = ICacheWriteStampInvalid;
-    }
-    fetchModeMask = iCachePageDataMask; // CS is 16-bit, Long Mode disabled
-  }
+  bxICache_c() { flushICacheEntries(); }
 
-  BX_CPP_INLINE unsigned hash(Bit32u pAddr) const
+  BX_CPP_INLINE unsigned hash(bx_phy_address pAddr) const
   {
     // A pretty dumb hash function for now.
     return pAddr & (BxICacheEntries-1);
@@ -143,25 +138,27 @@ public:
 };
 
 BX_CPP_INLINE void bxICache_c::flushICacheEntries(void)
-{   bxICacheEntry_c* e = entry;
+{
+    bxICacheEntry_c* e = entry;
     for (unsigned i=0; i<BxICacheEntries; i++, e++) {
       e->writeStamp = ICacheWriteStampInvalid;
     }
 }
 
 BX_CPP_INLINE void bxICache_c::purgeICacheEntries(void)
-{ bxICacheEntry_c* e = entry;
+{
+  bxICacheEntry_c* e = entry;
     
   // Since the write stamps may overflow if we always simply decrese them,
   // this function has to be called often enough that we can reset them
   // (without invalidating the cache).
-  for (unsigned i=0;i<BxICacheEntries;i++, e++)
+  for (unsigned i=0;i<BxICacheEntries;i++,e++)
   {
     Bit32u pageWriteStamp = pageWriteStampTable.getPageWriteStamp(e->pAddr);
     if (e->writeStamp != pageWriteStamp)
       e->writeStamp = ICacheWriteStampInvalid;	// invalidate entry
     else
-      e->writeStamp |= 0x1fffffff;
+      e->writeStamp |= ICacheWriteStampMask;
   }
 }
 
