@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: lazy_flags.cc,v 1.30 2006/01/25 22:20:00 sshwarts Exp $
+// $Id: lazy_flags.cc,v 1.33 2006/08/03 21:03:21 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -28,8 +28,10 @@
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
+
 // This array defines a look-up table for the even parity-ness
 // of an 8bit quantity, for optimal assignment of the parity bit
 // in the EFLAGS register
@@ -56,7 +58,7 @@ bx_bool BX_CPU_C::get_CFLazy(void)
 {
   unsigned cf;
 
-  switch ( BX_CPU_THIS_PTR lf_flags_status & 0x00000f ) {
+  switch (BX_CPU_THIS_PTR lf_flags_status & 0x00000f) {
     case BX_LF_INDEX_OSZAPC:
       switch (BX_CPU_THIS_PTR oszapc.instr) {
         case BX_INSTR_ADD8:
@@ -307,7 +309,7 @@ bx_bool BX_CPU_C::get_AFLazy(void)
 {
   unsigned af;
 
-  switch ( (BX_CPU_THIS_PTR lf_flags_status>>8) & 0x00000f ) {
+  switch ((BX_CPU_THIS_PTR lf_flags_status>>8) & 0x00000f) {
     case BX_LF_INDEX_OSZAPC:
       switch (BX_CPU_THIS_PTR oszapc.instr) {
         case BX_INSTR_ADD8:
@@ -456,7 +458,7 @@ bx_bool BX_CPU_C::get_ZFLazy(void)
 {
   unsigned zf;
 
-  switch ( (BX_CPU_THIS_PTR lf_flags_status>>12) & 0x00000f ) {
+  switch ((BX_CPU_THIS_PTR lf_flags_status>>12) & 0x00000f) {
     case BX_LF_INDEX_OSZAPC:
       switch (BX_CPU_THIS_PTR oszapc.instr) {
         case BX_INSTR_LOGIC8:
@@ -581,7 +583,7 @@ bx_bool BX_CPU_C::get_SFLazy(void)
 {
   unsigned sf;
 
-  switch ( (BX_CPU_THIS_PTR lf_flags_status>>16) & 0x00000f ) {
+  switch ((BX_CPU_THIS_PTR lf_flags_status>>16) & 0x00000f) {
     case BX_LF_INDEX_OSZAPC:
       switch (BX_CPU_THIS_PTR oszapc.instr) {
         case BX_INSTR_LOGIC8:
@@ -698,77 +700,61 @@ bx_bool BX_CPU_C::get_SFLazy(void)
   }
 }
 
+#define GET_ADD_OVERFLOW(op1, op2, result, mask) \
+  (((~((op1) ^ (op2)) & ((op2) ^ (result))) & (mask)) != 0)
+
+#define GET_SUB_OVERFLOW(op1, op2, result, mask) \
+   (((((op1) ^ (op2)) & ((op1) ^ (result))) & (mask)) != 0)
+
 bx_bool BX_CPU_C::get_OFLazy(void)
 {
-  Bit8u op1_b7, op2_b7, result_b7;
-  Bit16u op1_b15, op2_b15, result_b15;
-  Bit32u op1_b31, op2_b31, result_b31;
-#if BX_SUPPORT_X86_64
-  Bit64u op1_b63, op2_b63, result_b63;
-#endif
   unsigned of;
 
-  switch ( (BX_CPU_THIS_PTR lf_flags_status>>20) & 0x00000f ) {
+  switch ((BX_CPU_THIS_PTR lf_flags_status>>20) & 0x00000f) {
     case BX_LF_INDEX_OSZAPC:
       switch (BX_CPU_THIS_PTR oszapc.instr) {
         case BX_INSTR_ADD8:
         case BX_INSTR_ADC8:
-          op1_b7 = BX_CPU_THIS_PTR oszapc.op1_8 & 0x80;
-          op2_b7 = BX_CPU_THIS_PTR oszapc.op2_8 & 0x80;
-          result_b7 = BX_CPU_THIS_PTR oszapc.result_8 & 0x80;
-          of = (op1_b7 == op2_b7) && (result_b7 ^ op2_b7);
+          of = GET_ADD_OVERFLOW(BX_CPU_THIS_PTR oszapc.op1_8, BX_CPU_THIS_PTR oszapc.op2_8,
+                                BX_CPU_THIS_PTR oszapc.result_8, 0x80);
           break;
         case BX_INSTR_ADD16:
         case BX_INSTR_ADC16:
-          op1_b15 = BX_CPU_THIS_PTR oszapc.op1_16 & 0x8000;
-          op2_b15 = BX_CPU_THIS_PTR oszapc.op2_16 & 0x8000;
-          result_b15 = BX_CPU_THIS_PTR oszapc.result_16 & 0x8000;
-          of = (op1_b15 == op2_b15) && (result_b15 ^ op2_b15);
+          of = GET_ADD_OVERFLOW(BX_CPU_THIS_PTR oszapc.op1_16, BX_CPU_THIS_PTR oszapc.op2_16,
+                                BX_CPU_THIS_PTR oszapc.result_16, 0x8000);
           break;
         case BX_INSTR_ADD32:
         case BX_INSTR_ADC32:
-          op1_b31 = BX_CPU_THIS_PTR oszapc.op1_32 & 0x80000000;
-          op2_b31 = BX_CPU_THIS_PTR oszapc.op2_32 & 0x80000000;
-          result_b31 = BX_CPU_THIS_PTR oszapc.result_32 & 0x80000000;
-          of = (op1_b31 == op2_b31) && (result_b31 ^ op2_b31);
+          of = GET_ADD_OVERFLOW(BX_CPU_THIS_PTR oszapc.op1_32, BX_CPU_THIS_PTR oszapc.op2_32,
+                                BX_CPU_THIS_PTR oszapc.result_32, 0x80000000);
           break;
 #if BX_SUPPORT_X86_64
         case BX_INSTR_ADD64:
         case BX_INSTR_ADC64:
-          op1_b63 = BX_CPU_THIS_PTR oszapc.op1_64 & BX_CONST64(0x8000000000000000);
-          op2_b63 = BX_CPU_THIS_PTR oszapc.op2_64 & BX_CONST64(0x8000000000000000);
-          result_b63 = BX_CPU_THIS_PTR oszapc.result_64 & BX_CONST64(0x8000000000000000);
-          of = (op1_b63 == op2_b63) && (result_b63 ^ op2_b63);
+          of = GET_ADD_OVERFLOW(BX_CPU_THIS_PTR oszapc.op1_64, BX_CPU_THIS_PTR oszapc.op2_64,
+                                BX_CPU_THIS_PTR oszapc.result_64, BX_CONST64(0x8000000000000000));
           break;
 #endif
         case BX_INSTR_SUB8:
         case BX_INSTR_SBB8:
-          op1_b7 = BX_CPU_THIS_PTR oszapc.op1_8 & 0x80;
-          op2_b7 = BX_CPU_THIS_PTR oszapc.op2_8 & 0x80;
-          result_b7 = BX_CPU_THIS_PTR oszapc.result_8 & 0x80;
-          of = (op1_b7 ^ op2_b7) && (op1_b7 ^ result_b7);
+          of = GET_SUB_OVERFLOW(BX_CPU_THIS_PTR oszapc.op1_8, BX_CPU_THIS_PTR oszapc.op2_8,
+                                BX_CPU_THIS_PTR oszapc.result_8, 0x80);
           break;
         case BX_INSTR_SUB16:
         case BX_INSTR_SBB16:
-          op1_b15 = BX_CPU_THIS_PTR oszapc.op1_16 & 0x8000;
-          op2_b15 = BX_CPU_THIS_PTR oszapc.op2_16 & 0x8000;
-          result_b15 = BX_CPU_THIS_PTR oszapc.result_16 & 0x8000;
-          of = (op1_b15 ^ op2_b15) && (op1_b15 ^ result_b15);
+          of = GET_SUB_OVERFLOW(BX_CPU_THIS_PTR oszapc.op1_16, BX_CPU_THIS_PTR oszapc.op2_16,
+                                BX_CPU_THIS_PTR oszapc.result_16, 0x8000);
           break;
         case BX_INSTR_SUB32:
         case BX_INSTR_SBB32:
-          op1_b31 = BX_CPU_THIS_PTR oszapc.op1_32 & 0x80000000;
-          op2_b31 = BX_CPU_THIS_PTR oszapc.op2_32 & 0x80000000;
-          result_b31 = BX_CPU_THIS_PTR oszapc.result_32 & 0x80000000;
-          of = (op1_b31 ^ op2_b31) && (op1_b31 ^ result_b31);
+          of = GET_SUB_OVERFLOW(BX_CPU_THIS_PTR oszapc.op1_32, BX_CPU_THIS_PTR oszapc.op2_32,
+                                BX_CPU_THIS_PTR oszapc.result_32, 0x80000000);
           break;
 #if BX_SUPPORT_X86_64
         case BX_INSTR_SUB64:
         case BX_INSTR_SBB64:
-          op1_b63 = BX_CPU_THIS_PTR oszapc.op1_64 & BX_CONST64(0x8000000000000000);
-          op2_b63 = BX_CPU_THIS_PTR oszapc.op2_64 & BX_CONST64(0x8000000000000000);
-          result_b63 = BX_CPU_THIS_PTR oszapc.result_64 & BX_CONST64(0x8000000000000000);
-          of = (op1_b63 ^ op2_b63) && (op1_b63 ^ result_b63);
+          of = GET_SUB_OVERFLOW(BX_CPU_THIS_PTR oszapc.op1_64, BX_CPU_THIS_PTR oszapc.op2_64,
+                                BX_CPU_THIS_PTR oszapc.result_64, BX_CONST64(0x8000000000000000));
           break;
 #endif
         case BX_INSTR_NEG8:
@@ -979,7 +965,7 @@ bx_bool BX_CPU_C::get_PFLazy(void)
 {
   unsigned pf;
 
-  switch ( (BX_CPU_THIS_PTR lf_flags_status>>4) & 0x00000f ) {
+  switch ((BX_CPU_THIS_PTR lf_flags_status>>4) & 0x00000f) {
     case BX_LF_INDEX_OSZAPC:
       switch (BX_CPU_THIS_PTR oszapc.instr) {
         case BX_INSTR_LOGIC8:
