@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: lazy_flags.h,v 1.19 2004/09/04 10:21:15 sshwarts Exp $
+// $Id: lazy_flags.h,v 1.23 2005/10/13 20:21:35 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -32,16 +32,21 @@
 #define BX_INSTR_ADD32          3
 #define BX_INSTR_ADD64          4
 
-#define BX_INSTR_SUB8           5
-#define BX_INSTR_SUB16          6
-#define BX_INSTR_SUB32          7
-#define BX_INSTR_SUB64          8
-
 // used only if CF = 1 when executing ADC instruction
-#define BX_INSTR_ADC8           9
-#define BX_INSTR_ADC16          10
-#define BX_INSTR_ADC32          11
-#define BX_INSTR_ADC64          12
+#define BX_INSTR_ADC8           5
+#define BX_INSTR_ADC16          6
+#define BX_INSTR_ADC32          7
+#define BX_INSTR_ADC64          8
+
+#define BX_INSTR_ADD_ADC8(cf)  (1 + ((cf)<<2))
+#define BX_INSTR_ADD_ADC16(cf) (2 + ((cf)<<2))
+#define BX_INSTR_ADD_ADC32(cf) (3 + ((cf)<<2))
+#define BX_INSTR_ADD_ADC64(cf) (4 + ((cf)<<2))
+
+#define BX_INSTR_SUB8           9
+#define BX_INSTR_SUB16          10
+#define BX_INSTR_SUB32          11
+#define BX_INSTR_SUB64          12
 
 // used only if CF = 1 when executing SBB instruction
 #define BX_INSTR_SBB8           13
@@ -49,6 +54,11 @@
 #define BX_INSTR_SBB32          15
 #define BX_INSTR_SBB64          16
 
+#define BX_INSTR_SUB_SBB8(cf)  (9  + ((cf)<<2))
+#define BX_INSTR_SUB_SBB16(cf) (10 + ((cf)<<2))
+#define BX_INSTR_SUB_SBB32(cf) (11 + ((cf)<<2))
+#define BX_INSTR_SUB_SBB64(cf) (12 + ((cf)<<2))
+
 #define BX_INSTR_INC8           17
 #define BX_INSTR_INC16          18
 #define BX_INSTR_INC32          19
@@ -99,6 +109,11 @@
 #define BX_INSTR_IMUL32         55
 #define BX_INSTR_IMUL64         56
 
+//      BX_INSTR_SHRD8          not exists, leave number for alignment
+#define BX_INSTR_SHRD16         58
+#define BX_INSTR_SHRD32         59
+#define BX_INSTR_SHRD64         60
+
 #define BX_INSTR_COMPARE8       BX_INSTR_SUB8
 #define BX_INSTR_COMPARE16      BX_INSTR_SUB16
 #define BX_INSTR_COMPARE32      BX_INSTR_SUB32
