@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logical16.cc,v 1.23 2005/05/20 20:06:50 sshwarts Exp $
+// $Id: logical16.cc,v 1.25 2006/03/26 18:58:01 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -27,6 +27,7 @@
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
@@ -57,7 +58,7 @@ void BX_CPU_C::XOR_EwGw(bxInstruction_c *i)
 #else
     result_16 = op1_16 ^ op2_16;
 #endif
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 
 #if !defined(BX_HostAsm_Xor16)
@@ -112,7 +113,7 @@ void BX_CPU_C::XOR_EwIw(bxInstruction_c *i)
   else {
     read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
     result_16 = op1_16 ^ op2_16;
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 
   SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
@@ -132,7 +133,7 @@ void BX_CPU_C::OR_EwIw(bxInstruction_c *i)
   else {
     read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
     result_16 = op1_16 | op2_16;
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 
   SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
@@ -150,7 +151,7 @@ void BX_CPU_C::NOT_Ew(bxInstruction_c *i)
   else {
     read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
     result_16 = ~op1_16;
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 }
 
@@ -168,7 +169,7 @@ void BX_CPU_C::OR_EwGw(bxInstruction_c *i)
   else {
     read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
     result_16 = op1_16 | op2_16;
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 
   SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
@@ -241,7 +242,7 @@ void BX_CPU_C::AND_EwGw(bxInstruction_c *i)
     result_16 = op1_16 & op2_16;
 #endif
 
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 
 #if !defined(BX_HostAsm_And16)
@@ -323,7 +324,7 @@ void BX_CPU_C::AND_EwIw(bxInstruction_c *i)
     result_16 = op1_16 & op2_16;
 #endif
 
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 
 #if !defined(BX_HostAsm_And16)
