@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: protect_ctrl.cc,v 1.44 2005/10/15 21:01:36 sshwarts Exp $
+// $Id: protect_ctrl.cc,v 1.49 2006/08/22 19:06:03 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -25,9 +25,9 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 #if BX_CPU_LEVEL >= 2
@@ -65,7 +65,7 @@ void BX_CPU_C::ARPL_EwGw(bxInstruction_c *i)
         }
       }
       else {
-        Write_RMW_virtual_word(op1_16);
+        write_RMW_virtual_word(op1_16);
       }
       set_ZF(1);
     }
@@ -130,7 +130,7 @@ void BX_CPU_C::LAR_GvEw(bxInstruction_c *i)
    */
 
   if (descriptor.segment) { /* normal segment */
-    if (descriptor.u.segment.executable && descriptor.u.segment.c_ed) {
+    if (IS_CODE_SEGMENT(descriptor.type) && IS_CODE_SEGMENT_CONFORMING(descriptor.type)) {
       /* ignore DPL for conforming segments */
     }
     else {
@@ -278,14 +278,13 @@ void BX_CPU_C::SLDT_Ew(bxInstruction_c *i)
     BX_INFO(("SLDT: not recognized in real or virtual-8086 mode"));
     UndefinedOpcode(i);
   }
+
+  Bit16u val16 = BX_CPU_THIS_PTR ldtr.selector.value;
+  if (i->modC0()) {
+    BX_WRITE_16BIT_REG(i->rm(), val16);
+  }
   else {
-    Bit16u val16 = BX_CPU_THIS_PTR ldtr.selector.value;
-    if (i->modC0()) {
-      BX_WRITE_16BIT_REG(i->rm(), val16);
-    }
-    else {
-      write_virtual_word(i->seg(), RMAddr(i), &val16);
-    }
+    write_virtual_word(i->seg(), RMAddr(i), &val16);
   }
 }
 
@@ -297,14 +296,13 @@ void BX_CPU_C::STR_Ew(bxInstruction_c *i)
     BX_INFO(("STR: not recognized in real or virtual-8086 mode"));
     UndefinedOpcode(i);
   }
+
+  Bit16u val16 = BX_CPU_THIS_PTR tr.selector.value;
+  if (i->modC0()) {
+    BX_WRITE_16BIT_REG(i->rm(), val16);
+  }
   else {
-    Bit16u val16 = BX_CPU_THIS_PTR tr.selector.value;
-    if (i->modC0()) {
-      BX_WRITE_16BIT_REG(i->rm(), val16);
-    }
-    else {
-      write_virtual_word(i->seg(), RMAddr(i), &val16);
-    }
+    write_virtual_word(i->seg(), RMAddr(i), &val16);
   }
 }
 
@@ -312,18 +310,21 @@ void BX_CPU_C::STR_Ew(bxInstruction_c *i)
 
 void BX_CPU_C::LLDT_Ew(bxInstruction_c *i)
 {
-  if (real_mode() || v8086_mode()) {
-    BX_INFO(("LLDT: not recognized in real or virtual-8086 mode"));
-    UndefinedOpcode(i);
-  }
-
-  invalidate_prefetch_q();
-
   /* protected mode */
   bx_descriptor_t  descriptor;
   bx_selector_t    selector;
   Bit16u raw_selector;
   Bit32u dword1, dword2;
+#if BX_SUPPORT_X86_64
+  Bit32u dword3;
+#endif
+
+  if (real_mode() || v8086_mode()) {
+    BX_INFO(("LTR: not recognized in real or virtual-8086 mode"));
+    UndefinedOpcode(i);
+  }
+
+  invalidate_prefetch_q();
 
   /* #GP(0) if the current privilege level is not 0 */
   if (CPL != 0) {
@@ -335,7 +336,7 @@ void BX_CPU_C::LLDT_Ew(bxInstruction_c *i)
     raw_selector = BX_READ_16BIT_REG(i->rm());
   }
   else {
-      read_virtual_word(i->seg(), RMAddr(i), &raw_selector);
+    read_virtual_word(i->seg(), RMAddr(i), &raw_selector);
   }
 
   /* if selector is NULL, invalidate and done */
@@ -354,19 +355,21 @@ void BX_CPU_C::LLDT_Ew(bxInstruction_c *i)
     exception(BX_GP_EXCEPTION, raw_selector & 0xfffc, 0);
   }
 
-  if ((selector.index*8 + 7) > BX_CPU_THIS_PTR gdtr.limit) {
-    BX_PANIC(("LLDT: GDT: index > limit"));
-    exception(BX_GP_EXCEPTION, raw_selector & 0xfffc, 0);
-    return;
-  }
-
-  access_linear(BX_CPU_THIS_PTR gdtr.base + selector.index*8,     4, 0,
-      BX_READ, &dword1);
-  access_linear(BX_CPU_THIS_PTR gdtr.base + selector.index*8 + 4, 4, 0,
-      BX_READ, &dword2);
-
+  /* fetch 2 dwords of descriptor; call handles out of limits checks */
+  fetch_raw_descriptor(&selector, &dword1, &dword2, BX_GP_EXCEPTION);
   parse_descriptor(dword1, dword2, &descriptor);
 
+#if BX_SUPPORT_X86_64
+  if (BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64) {
+    // set upper 32 bits of tss base
+    access_linear(BX_CPU_THIS_PTR gdtr.base + selector.index*8 + 8, 4, 0, BX_READ, &dword3);
+    descriptor.u.ldt.base |= ((Bit64u)dword3 << 32);
+    BX_INFO(("64 bit LDT base = 0x%08x%08x",
+       (Bit32u)(descriptor.u.ldt.base >> 32),
+       (Bit32u) descriptor.u.ldt.base));
+  }
+#endif
+
   /* if selector doesn't point to an LDT descriptor #GP(selector) */
   if (descriptor.valid == 0 || descriptor.segment ||
          descriptor.type != BX_SYS_SEGMENT_LDT) 
@@ -388,14 +391,6 @@ void BX_CPU_C::LLDT_Ew(bxInstruction_c *i)
 
 void BX_CPU_C::LTR_Ew(bxInstruction_c *i)
 {
-  if (real_mode() || v8086_mode()) {
-    BX_INFO(("LTR: not recognized in real or virtual-8086 mode"));
-    UndefinedOpcode(i);
-  }
-
-  // protected mode
-  invalidate_prefetch_q();
-
   bx_descriptor_t descriptor;
   bx_selector_t selector;
   Bit16u raw_selector;
@@ -404,6 +399,13 @@ void BX_CPU_C::LTR_Ew(bxInstruction_c *i)
   Bit32u dword3;
 #endif
 
+  if (real_mode() || v8086_mode()) {
+    BX_INFO(("LTR: not recognized in real or virtual-8086 mode"));
+    UndefinedOpcode(i);
+  }
+
+  invalidate_prefetch_q();
+
   /* #GP(0) if the current privilege level is not 0 */
   if (CPL != 0) {
     BX_ERROR(("LTR: The current priveledge level is not 0"));
@@ -439,10 +441,10 @@ void BX_CPU_C::LTR_Ew(bxInstruction_c *i)
   if (BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64) {
     // set upper 32 bits of tss base
     access_linear(BX_CPU_THIS_PTR gdtr.base + selector.index*8 + 8, 4, 0, BX_READ, &dword3);
-    descriptor.u.tss386.base |= ((Bit64u)dword3 << 32);
+    descriptor.u.tss.base |= ((Bit64u)dword3 << 32);
     BX_INFO(("64 bit tss base = 0x%08x%08x",
-       (Bit32u)(descriptor.u.tss386.base >> 32),
-       (Bit32u) descriptor.u.tss386.base));
+       (Bit32u)(descriptor.u.tss.base >> 32),
+       (Bit32u) descriptor.u.tss.base));
   }
 #endif
 
@@ -461,16 +463,6 @@ void BX_CPU_C::LTR_Ew(bxInstruction_c *i)
     exception(BX_NP_EXCEPTION, raw_selector & 0xfffc, 0);
   }
 
-/*
-  // the real hardware CPU allow loading of tss with limit < minimum
-  if (descriptor.type==1 && descriptor.u.tss286.limit<43) {
-    BX_PANIC(("LTR:286TSS: loading tr.limit < 43"));
-  }
-  else if (descriptor.type==9 && descriptor.u.tss386.limit_scaled<103) {
-    BX_PANIC(("LTR:386TSS: loading tr.limit < 103"));
-  }
-*/
-
   BX_CPU_THIS_PTR tr.selector = selector;
   BX_CPU_THIS_PTR tr.cache    = descriptor;
   BX_CPU_THIS_PTR tr.cache.valid = 1;
@@ -541,22 +533,24 @@ void BX_CPU_C::VERR_Ew(bxInstruction_c *i)
   }
 
   /* normal data/code segment */
-  if (descriptor.u.segment.executable) { /* code segment */
+  if (IS_CODE_SEGMENT(descriptor.type)) { /* code segment */
     /* ignore DPL for readable conforming segments */
-    if (descriptor.u.segment.c_ed && descriptor.u.segment.r_w) {
+    if (IS_CODE_SEGMENT_CONFORMING(descriptor.type) &&
+        IS_CODE_SEGMENT_READABLE(descriptor.type))
+    {
       BX_DEBUG(("VERR: conforming code, OK"));
       assert_ZF(); /* accessible */
       return;
     }
-    if (descriptor.u.segment.r_w==0) {
+    if (!IS_CODE_SEGMENT_READABLE(descriptor.type)) {
       BX_DEBUG(("VERR: code not readable"));
-      clear_ZF (); /* inaccessible */
+      clear_ZF();  /* inaccessible */
       return;
     }
     /* readable, non-conforming code segment */
     if ((descriptor.dpl<CPL) || (descriptor.dpl<selector.rpl)) {
       BX_DEBUG(("VERR: non-conforming code not withing priv level"));
-      clear_ZF (); /* inaccessible */
+      clear_ZF();  /* inaccessible */
       return;
     }
 
@@ -615,7 +609,7 @@ void BX_CPU_C::VERW_Ew(bxInstruction_c *i)
   parse_descriptor(dword1, dword2, &descriptor);
 
   /* rule out system segments & code segments */
-  if (descriptor.segment==0 || descriptor.u.segment.executable) {
+  if (descriptor.segment==0 || IS_CODE_SEGMENT(descriptor.type)) {
     BX_DEBUG(("VERW: system seg or code"));
     clear_ZF();
     return;
@@ -628,7 +622,7 @@ void BX_CPU_C::VERW_Ew(bxInstruction_c *i)
   }
 
   /* data segment */
-  if (descriptor.u.segment.r_w) { /* writable */
+  if (IS_DATA_SEGMENT_WRITEABLE(descriptor.type)) { /* writable */
     if ((descriptor.dpl<CPL) || (descriptor.dpl<selector.rpl)) {
       BX_DEBUG(("VERW: writable data seg not within priv level"));
       clear_ZF(); /* not accessible */
@@ -648,8 +642,7 @@ void BX_CPU_C::SGDT_Ms(bxInstruction_c *i)
 {
   /* op1 is a register or memory reference */
   if (i->modC0()) {
-    /* undefined opcode exception */
-    BX_INFO(("SGDT_Ms: use of register is undefined opcode."));
+    BX_INFO(("SGDT_Ms: use of register is undefined opcode"));
     UndefinedOpcode(i);
   }
 
@@ -687,8 +680,7 @@ void BX_CPU_C::SIDT_Ms(bxInstruction_c *i)
 
   /* op1 is a register or memory reference */
   if (i->modC0()) {
-    /* undefined opcode exception */
-    BX_INFO(("SIDT: use of register is undefined opcode."));
+    BX_INFO(("SIDT: use of register is undefined opcode"));
     UndefinedOpcode(i);
   }
 
