@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////
-// $Id: ret_far.cc,v 1.4 2005/08/03 21:01:02 sshwarts Exp $
+// $Id: ret_far.cc,v 1.7 2006/06/12 16:58:27 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -28,6 +28,7 @@
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 #if BX_SUPPORT_X86_64==0
@@ -66,7 +67,7 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
   }
 
 #if BX_SUPPORT_X86_64
-  if ( i->os64L() ) {
+  if (i->os64L()) {
     /* operand size=64: 2nd qword on stack must be within stack limits,
      *   else #SS(0); */
     if (!can_pop(16))
@@ -84,10 +85,10 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
   } 
   else
 #endif
-  if ( i->os32L() ) {
+  if (i->os32L()) {
     /* operand size=32: 2nd dword on stack must be within stack limits,
      *   else #SS(0); */
-    if (!can_pop(8))
+    if (! can_pop(8))
     {
       BX_ERROR(("return_protected: 2rd dword not in stack limits"));
       exception(BX_SS_EXCEPTION, 0, 0);
@@ -105,7 +106,7 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
   else {
     /* operand size=16: second word on stack must be within stack limits,
      *   else #SS(0); */
-    if ( !can_pop(4) )
+    if (! can_pop(4))
     {
       BX_ERROR(("return_protected: 2nd word not in stack limits"));
       exception(BX_SS_EXCEPTION, 0, 0);
@@ -122,8 +123,8 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
   }
 
   // selector must be non-null else #GP(0)
-  if ( (raw_cs_selector & 0xfffc) == 0 ) {
-    BX_INFO(("return_protected: CS selector null"));
+  if ((raw_cs_selector & 0xfffc) == 0) {
+    BX_ERROR(("return_protected: CS selector null"));
     exception(BX_GP_EXCEPTION, 0, 0);
   }
 
@@ -186,7 +187,7 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
 #if BX_SUPPORT_X86_64
     if (i->os64L()) {
       /* top 32+immediate bytes on stack must be within stack limits, else #SS(0) */
-      if ( !can_pop(32 + pop_bytes) ) {
+      if (! can_pop(32 + pop_bytes)) {
         BX_ERROR(("return_protected: 32 bytes not within stack limits"));
         exception(BX_SS_EXCEPTION, 0, 0);
       }
@@ -200,7 +201,7 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
 #endif
     if (i->os32L()) {
       /* top 16+immediate bytes on stack must be within stack limits, else #SS(0) */
-      if ( !can_pop(16 + pop_bytes) ) {
+      if (! can_pop(16 + pop_bytes)) {
         BX_ERROR(("return_protected: 16 bytes not within stack limits"));
         exception(BX_SS_EXCEPTION, 0, 0);
       }
@@ -214,7 +215,7 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
     }
     else {
       /* top 8+immediate bytes on stack must be within stack limits, else #SS(0) */
-      if ( !can_pop(8 + pop_bytes) ) {
+      if (! can_pop(8 + pop_bytes)) {
         BX_ERROR(("return_protected: 8 bytes not within stack limits"));
         exception(BX_SS_EXCEPTION, 0, 0);
       }
@@ -258,10 +259,10 @@ BX_CPU_C::return_protected(bxInstruction_c *i, Bit16u pop_bytes)
     /* descriptor AR byte must indicate a writable data segment,
      * else #GP(selector) */
     if (ss_descriptor.valid==0 || ss_descriptor.segment==0 ||
-        ss_descriptor.u.segment.executable ||
-        ss_descriptor.u.segment.r_w==0)
+         IS_CODE_SEGMENT(ss_descriptor.type) ||
+        !IS_DATA_SEGMENT_WRITEABLE(ss_descriptor.type))
     {
-      BX_PANIC(("return_protected: SS.AR byte not writable data"));
+      BX_ERROR(("return_protected: SS.AR byte not writable data"));
       exception(BX_GP_EXCEPTION, raw_ss_selector & 0xfffc, 0);
     }
 
