@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: shift16.cc,v 1.28 2005/10/13 20:21:35 sshwarts Exp $
+// $Id: shift16.cc,v 1.32 2006/03/26 18:58:01 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -27,6 +27,7 @@
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
@@ -74,7 +75,7 @@ void BX_CPU_C::SHLD_EwGw(bxInstruction_c *i)
     BX_WRITE_16BIT_REG(i->rm(), result_16);
   }
   else {
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 
   /* set eflags:
@@ -126,7 +127,7 @@ void BX_CPU_C::SHRD_EwGw(bxInstruction_c *i)
     BX_WRITE_16BIT_REG(i->rm(), result_16);
   }
   else {
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 
   /* set eflags:
@@ -160,6 +161,7 @@ void BX_CPU_C::ROL_Ew(bxInstruction_c *i)
     if ( count & 0x10 ) {
       unsigned bit0 = op1_16 & 1;
       set_CF(bit0);
+      set_OF(bit0 ^ (op1_16 >> 15));
     }
     return;
   }
@@ -172,7 +174,7 @@ void BX_CPU_C::ROL_Ew(bxInstruction_c *i)
     BX_WRITE_16BIT_REG(i->rm(), result_16);
   }
   else {
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 
   /* set eflags:
@@ -207,8 +209,10 @@ void BX_CPU_C::ROR_Ew(bxInstruction_c *i)
 
   if ( (count & 0x0f) == 0 ) {
     if ( count & 0x10 ) {
-      unsigned MSB = op1_16 >> 15;
-      set_CF(MSB);
+      unsigned bit14 = (op1_16 >> 14) & 1;
+      unsigned bit15 = (op1_16 >> 15);
+      set_CF(bit15);
+      set_OF(bit15 ^ bit14);
     }
     return;
   }
@@ -221,7 +225,7 @@ void BX_CPU_C::ROR_Ew(bxInstruction_c *i)
     BX_WRITE_16BIT_REG(i->rm(), result_16);
   }
   else {
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
   
   /* set eflags:
@@ -275,7 +279,7 @@ void BX_CPU_C::RCL_Ew(bxInstruction_c *i)
     BX_WRITE_16BIT_REG(i->rm(), result_16);
   }
   else {
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 
   /* set eflags:
@@ -321,7 +325,7 @@ void BX_CPU_C::RCR_Ew(bxInstruction_c *i)
     BX_WRITE_16BIT_REG(i->rm(), result_16);
   }
   else {
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 
   /* set eflags:
@@ -364,7 +368,7 @@ void BX_CPU_C::SHL_Ew(bxInstruction_c *i)
     BX_WRITE_16BIT_REG(i->rm(), result_16);
   }
   else {
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 
   SET_FLAGS_OSZAPC_16(op1_16, count, result_16, BX_INSTR_SHL16);
@@ -409,7 +413,7 @@ void BX_CPU_C::SHR_Ew(bxInstruction_c *i)
     BX_WRITE_16BIT_REG(i->rm(), result_16);
   }
   else {
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 }
 
@@ -460,7 +464,7 @@ void BX_CPU_C::SAR_Ew(bxInstruction_c *i)
     BX_WRITE_16BIT_REG(i->rm(), result_16);
   }
   else {
-    Write_RMW_virtual_word(result_16);
+    write_RMW_virtual_word(result_16);
   }
 
   SET_FLAGS_OSZAPC_16(op1_16, count, result_16, BX_INSTR_SAR16);
