@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: shift32.cc,v 1.29 2005/10/13 20:21:35 sshwarts Exp $
+// $Id: shift32.cc,v 1.31 2006/03/26 18:58:01 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -27,6 +27,7 @@
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
@@ -62,7 +63,7 @@ void BX_CPU_C::SHLD_EdGd(bxInstruction_c *i)
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
   }
   else {
-    Write_RMW_virtual_dword(result_32);
+    write_RMW_virtual_dword(result_32);
   }
 
   /* set eflags:
@@ -101,7 +102,7 @@ void BX_CPU_C::SHRD_EdGd(bxInstruction_c *i)
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
   }
   else {
-    Write_RMW_virtual_dword(result_32);
+    write_RMW_virtual_dword(result_32);
   }
 
   /* set eflags:
@@ -140,7 +141,7 @@ void BX_CPU_C::ROL_Ed(bxInstruction_c *i)
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
   }
   else {
-    Write_RMW_virtual_dword(result_32);
+    write_RMW_virtual_dword(result_32);
   }
 
   /* set eflags:
@@ -182,7 +183,7 @@ void BX_CPU_C::ROR_Ed(bxInstruction_c *i)
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
   }
   else {
-    Write_RMW_virtual_dword(result_32);
+    write_RMW_virtual_dword(result_32);
   }
 
   /* set eflags:
@@ -231,7 +232,7 @@ void BX_CPU_C::RCL_Ed(bxInstruction_c *i)
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
   }
   else {
-    Write_RMW_virtual_dword(result_32);
+    write_RMW_virtual_dword(result_32);
   }
 
   /* set eflags:
@@ -279,7 +280,7 @@ void BX_CPU_C::RCR_Ed(bxInstruction_c *i)
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
   }
   else {
-    Write_RMW_virtual_dword(result_32);
+    write_RMW_virtual_dword(result_32);
   }
 
   /* set eflags:
@@ -320,7 +321,7 @@ void BX_CPU_C::SHL_Ed(bxInstruction_c *i)
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
   }
   else {
-    Write_RMW_virtual_dword(result_32);
+    write_RMW_virtual_dword(result_32);
   }
 
   SET_FLAGS_OSZAPC_32(op1_32, count, result_32, BX_INSTR_SHL32);
@@ -363,7 +364,7 @@ void BX_CPU_C::SHR_Ed(bxInstruction_c *i)
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
   }
   else {
-    Write_RMW_virtual_dword(result_32);
+    write_RMW_virtual_dword(result_32);
   }
 }
 
@@ -403,7 +404,7 @@ void BX_CPU_C::SAR_Ed(bxInstruction_c *i)
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
   }
   else {
-    Write_RMW_virtual_dword(result_32);
+    write_RMW_virtual_dword(result_32);
   }
 
   SET_FLAGS_OSZAPC_32(op1_32, count, result_32, BX_INSTR_SAR32);
