@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: shift8.cc,v 1.22 2005/10/13 19:28:10 sshwarts Exp $
+// $Id: shift8.cc,v 1.24 2006/03/26 18:58:01 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -27,6 +27,7 @@
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
@@ -68,7 +69,7 @@ void BX_CPU_C::ROL_Eb(bxInstruction_c *i)
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
   }
   else {
-    Write_RMW_virtual_byte(result_8);
+    write_RMW_virtual_byte(result_8);
   }
 
   /* set eflags:
@@ -119,7 +120,7 @@ void BX_CPU_C::ROR_Eb(bxInstruction_c *i)
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
   }
   else {
-    Write_RMW_virtual_byte(result_8);
+    write_RMW_virtual_byte(result_8);
   }
 
   /* set eflags:
@@ -170,7 +171,7 @@ void BX_CPU_C::RCL_Eb(bxInstruction_c *i)
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
   }
   else {
-    Write_RMW_virtual_byte(result_8);
+    write_RMW_virtual_byte(result_8);
   }
 
   /* set eflags:
@@ -215,7 +216,7 @@ void BX_CPU_C::RCR_Eb(bxInstruction_c *i)
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
   }
   else {
-    Write_RMW_virtual_byte(result_8);
+    write_RMW_virtual_byte(result_8);
   }
 
   /* set eflags:
@@ -258,7 +259,7 @@ void BX_CPU_C::SHL_Eb(bxInstruction_c *i)
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
   }
   else {
-    Write_RMW_virtual_byte(result_8);
+    write_RMW_virtual_byte(result_8);
   }
 
   SET_FLAGS_OSZAPC_8(op1_8, count, result_8, BX_INSTR_SHL8);
@@ -297,7 +298,7 @@ void BX_CPU_C::SHR_Eb(bxInstruction_c *i)
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
   }
   else {
-    Write_RMW_virtual_byte(result_8);
+    write_RMW_virtual_byte(result_8);
   }
 
   SET_FLAGS_OSZAPC_8(op1_8, count, result_8, BX_INSTR_SHR8);
@@ -350,7 +351,7 @@ void BX_CPU_C::SAR_Eb(bxInstruction_c *i)
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), result_8);
   }
   else {
-    Write_RMW_virtual_byte(result_8);
+    write_RMW_virtual_byte(result_8);
   }
 
   SET_FLAGS_OSZAPC_8(op1_8, count, result_8, BX_INSTR_SAR8);
