@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: sse_move.cc,v 1.47 2006/01/24 19:03:54 sshwarts Exp $
+// $Id: sse_move.cc,v 1.52 2006/04/27 06:09:56 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003 Stanislav Shwartsman
@@ -21,9 +21,9 @@
 //
 
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
@@ -127,31 +127,53 @@ void BX_CPU_C::FXSAVE(bxInstruction_c *i)
 
   /* x87 FPU Opcode (16 bits) */
   /* The lower 11 bits contain the FPU opcode, upper 5 bits are reserved */
-  xmm.xmm16u(3) = 0;  /* still not implemented */
+  xmm.xmm16u(3) = BX_CPU_THIS_PTR the_i387.foo;
 
   /* 
-   * x87 FPU IP Offset (32 bits)
+   * x87 FPU IP Offset (32/64 bits)
    * The contents of this field differ depending on the current 
-   * addressing mode (16/32 bit) when the FXSAVE instruction was executed:
-   *   + 32-bit mode-32-bit IP offset
-   *   + 16-bit mode-low 16 bits are IP offset; high 16 bits are reserved.
-   *
-   * x87 CS FPU IP Selector (16 bits)
+   * addressing mode (16/32/64 bit) when the FXSAVE instruction was executed:
+   *   + 64-bit mode - 64-bit IP offset
+   *   + 32-bit mode - 32-bit IP offset
+   *   + 16-bit mode - low 16 bits are IP offset; high 16 bits are reserved.
+   * x87 CS FPU IP Selector
+   *   + 16 bit, in 16/32 bit mode only
    */
-  xmm.xmm64u(1) = 0;  /* still not implemented */
+#if BX_SUPPORT_X86_64
+  if (i->os64L()) /* 64 bit operand size mode */
+  {
+    xmm.xmm64u(1) = BX_CPU_THIS_PTR the_i387.fip;
+  }
+  else
+#endif
+  {
+    xmm.xmm32u(2) = (BX_CPU_THIS_PTR the_i387.fip) & 0xffffffff;
+    xmm.xmm32u(3) = (BX_CPU_THIS_PTR the_i387.fcs);
+  }
  
   writeVirtualDQwordAligned(i->seg(), RMAddr(i), (Bit8u *) &xmm);
 
   /* 
-   * x87 FPU Instruction Operand (Data) Pointer Offset (32 bits)
+   * x87 FPU Instruction Operand (Data) Pointer Offset (32/64 bits)
    * The contents of this field differ depending on the current 
    * addressing mode (16/32 bit) when the FXSAVE instruction was executed:
-   *   + 32-bit mode-32-bit offset
-   *   + 16-bit mode-low 16 bits are offset; high 16 bits are reserved.
-   *
-   * x87 DS FPU Instruction Operand (Data) Pointer Selector (16 bits)
+   *   + 64-bit mode - 64-bit offset
+   *   + 32-bit mode - 32-bit offset
+   *   + 16-bit mode - low 16 bits are offset; high 16 bits are reserved.
+   * x87 DS FPU Instruction Operand (Data) Pointer Selector 
+   *   + 16 bit, in 16/32 bit mode only
    */
-  xmm.xmm64u(0) = 0;  /* still not implemented */
+#if BX_SUPPORT_X86_64
+  if (i->os64L()) /* 64 bit operand size mode */
+  {
+    xmm.xmm64u(0) = BX_CPU_THIS_PTR the_i387.fdp;
+  }
+  else
+#endif
+  {
+    xmm.xmm32u(0) = (BX_CPU_THIS_PTR the_i387.fdp) & 0xffffffff;
+    xmm.xmm32u(1) = (BX_CPU_THIS_PTR the_i387.fds);
+  }
 
 #if BX_SUPPORT_SSE >= 1
   xmm.xmm32u(2) = BX_MXCSR_REGISTER;
@@ -214,19 +236,45 @@ void BX_CPU_C::FXRSTOR(bxInstruction_c *i)
   BX_CPU_THIS_PTR the_i387.swd = xmm.xmm16u(1);
   BX_CPU_THIS_PTR the_i387.tos = (xmm.xmm16u(1) >> 11) & 0x07;
 
-  /* FOO/FPU IP restore still not implemented */
+  /* Restore x87 FPU Opcode */
+  /* The lower 11 bits contain the FPU opcode, upper 5 bits are reserved */
+  BX_CPU_THIS_PTR the_i387.foo = xmm.xmm16u(3) & 0x7FF;
+
+  /* Restore x87 FPU IP */
+#if BX_SUPPORT_X86_64
+  if (i->os64L()) /* 64 bit operand size mode */
+  {
+    BX_CPU_THIS_PTR the_i387.fip = xmm.xmm64u(1);
+  }
+  else
+#endif
+  {
+    BX_CPU_THIS_PTR the_i387.fip = xmm.xmm32u(2);
+    BX_CPU_THIS_PTR the_i387.fcs = xmm.xmm16u(5);
+  }
 
   Bit32u twd = 0, tag_byte = xmm.xmm16u(2);
 
-  /* FPU DP restore still not implemented */
+  /* Restore x87 FPU DP */
+  readVirtualDQwordAligned(i->seg(), RMAddr(i) + 16, (Bit8u *) &xmm);
+
+#if BX_SUPPORT_X86_64
+  if (i->os64L()) /* 64 bit operand size mode */
+  {
+    BX_CPU_THIS_PTR the_i387.fdp = xmm.xmm64u(0);
+  }
+  else
+#endif
+  {
+    BX_CPU_THIS_PTR the_i387.fdp = xmm.xmm32u(0);
+    BX_CPU_THIS_PTR the_i387.fds = xmm.xmm16u(2);
+  }
 
 #if BX_SUPPORT_SSE >= 1
   /* If the OSFXSR bit in CR4 is not set, the FXRSTOR instruction does
      not restore the states of the XMM and MXCSR registers. */
   if(BX_CPU_THIS_PTR cr4.get_OSFXSR())
   {
-    readVirtualDQwordAligned(i->seg(), RMAddr(i) + 16, (Bit8u *) &xmm);
-
     Bit32u new_mxcsr = xmm.xmm32u(2);
     if(new_mxcsr & ~MXCSR_MASK)
        exception(BX_GP_EXCEPTION, 0, 0);
@@ -238,8 +286,7 @@ void BX_CPU_C::FXRSTOR(bxInstruction_c *i)
   /* load i387 register file */
   for(index=0; index < 8; index++)
   {
-    read_virtual_tword(i->seg(), 
-           RMAddr(i)+index*16+32, &(BX_FPU_REG(index)));
+    read_virtual_tword(i->seg(), RMAddr(i)+index*16+32, &(BX_FPU_REG(index)));
   }
 
   /*                                 FTW
@@ -303,7 +350,7 @@ void BX_CPU_C::FXRSTOR(bxInstruction_c *i)
     /* load XMM register file */
     for(index=0; index < BX_XMM_REGISTERS; index++)
     {
-      readVirtualDQwordAligned(i->seg(), 
+      readVirtualDQwordAligned(i->seg(),
            RMAddr(i)+index*16+160, (Bit8u *) &(BX_CPU_THIS_PTR xmm[index]));
     }
   }
@@ -572,7 +619,7 @@ void BX_CPU_C::MOVLPS_VpsMq(bxInstruction_c *i)
 /* F2 0F 12 */
 void BX_CPU_C::MOVDDUP_VpdWq(bxInstruction_c *i)
 {
-#if BX_SUPPORT_PNI
+#if BX_SUPPORT_SSE >= 3
   BX_CPU_THIS_PTR prepareSSE();
   Bit64u val64;
   BxPackedXmmRegister op;
@@ -600,7 +647,7 @@ void BX_CPU_C::MOVDDUP_VpdWq(bxInstruction_c *i)
 /* F3 0F 12 */
 void BX_CPU_C::MOVSLDUP_VpsWps(bxInstruction_c *i)
 {
-#if BX_SUPPORT_PNI
+#if BX_SUPPORT_SSE >= 3
   BX_CPU_THIS_PTR prepareSSE();
   BxPackedXmmRegister op, result;
 
@@ -629,7 +676,7 @@ void BX_CPU_C::MOVSLDUP_VpsWps(bxInstruction_c *i)
 /* F3 0F 16 */
 void BX_CPU_C::MOVSHDUP_VpsWps(bxInstruction_c *i)
 {
-#if BX_SUPPORT_PNI
+#if BX_SUPPORT_SSE >= 3
   BX_CPU_THIS_PTR prepareSSE();
   BxPackedXmmRegister op, result;
 
@@ -725,7 +772,7 @@ void BX_CPU_C::MOVHPS_MqVps(bxInstruction_c *i)
 /* F2 0F F0 */
 void BX_CPU_C::LDDQU_VdqMdq(bxInstruction_c *i)
 {
-#if BX_SUPPORT_PNI
+#if BX_SUPPORT_SSE >= 3
   BX_CPU_THIS_PTR prepareSSE();
 
   /* source must be memory reference */
@@ -913,7 +960,7 @@ void BX_CPU_C::MOVD_EdVd(bxInstruction_c *i)
       write_virtual_dword(i->seg(), RMAddr(i), &op2);
     }
   }
-
+#else
   BX_INFO(("MOVD_EdVd: required SSE2, use --enable-sse option"));
   UndefinedOpcode(i);
 #endif
