@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: sse_pfp.cc,v 1.23 2005/11/26 21:36:51 sshwarts Exp $
+// $Id: sse_pfp.cc,v 1.27 2006/04/05 17:31:33 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003 Stanislav Shwartsman
@@ -22,6 +22,7 @@
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 #if BX_SUPPORT_SSE
@@ -2448,7 +2449,7 @@ void BX_CPU_C::MAXSS_VssWss(bxInstruction_c *i)
  */
 void BX_CPU_C::HADDPD_VpdWpd(bxInstruction_c *i)
 {
-#if BX_SUPPORT_PNI
+#if BX_SUPPORT_SSE >= 3
   BX_CPU_THIS_PTR prepareSSE();
 
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->nnn()), op2, result;
@@ -2482,7 +2483,7 @@ void BX_CPU_C::HADDPD_VpdWpd(bxInstruction_c *i)
   BX_WRITE_XMM_REG(i->nnn(), result);
 
 #else
-  BX_INFO(("HADDPD_VpdWpd: required PNI, use --enable-pni option"));
+  BX_INFO(("HADDPD_VpdWpd: required SSE3, use --enable-sse option"));
   UndefinedOpcode(i);
 #endif
 }
@@ -2494,7 +2495,7 @@ void BX_CPU_C::HADDPD_VpdWpd(bxInstruction_c *i)
  */
 void BX_CPU_C::HADDPS_VpsWps(bxInstruction_c *i)
 {
-#if BX_SUPPORT_PNI
+#if BX_SUPPORT_SSE >= 3
   BX_CPU_THIS_PTR prepareSSE();
 
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->nnn()), op2, result;
@@ -2536,7 +2537,7 @@ void BX_CPU_C::HADDPS_VpsWps(bxInstruction_c *i)
   BX_WRITE_XMM_REG(i->nnn(), result);
 
 #else
-  BX_INFO(("HADDPS_VpsWps: required PNI, use --enable-pni option"));
+  BX_INFO(("HADDPS_VpsWps: required SSE3, use --enable-sse option"));
   UndefinedOpcode(i);
 #endif
 }
@@ -2548,7 +2549,7 @@ void BX_CPU_C::HADDPS_VpsWps(bxInstruction_c *i)
  */
 void BX_CPU_C::HSUBPD_VpdWpd(bxInstruction_c *i)
 {
-#if BX_SUPPORT_PNI
+#if BX_SUPPORT_SSE >= 3
   BX_CPU_THIS_PTR prepareSSE();
 
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->nnn()), op2, result;
@@ -2582,7 +2583,7 @@ void BX_CPU_C::HSUBPD_VpdWpd(bxInstruction_c *i)
   BX_WRITE_XMM_REG(i->nnn(), result);
 
 #else
-  BX_INFO(("HSUBPD_VpdWpd: required PNI, use --enable-pni option"));
+  BX_INFO(("HSUBPD_VpdWpd: required SSE3, use --enable-sse option"));
   UndefinedOpcode(i);
 #endif
 }
@@ -2594,7 +2595,7 @@ void BX_CPU_C::HSUBPD_VpdWpd(bxInstruction_c *i)
  */
 void BX_CPU_C::HSUBPS_VpsWps(bxInstruction_c *i)
 {
-#if BX_SUPPORT_PNI
+#if BX_SUPPORT_SSE >= 3
   BX_CPU_THIS_PTR prepareSSE();
 
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->nnn()), op2, result;
@@ -2636,7 +2637,7 @@ void BX_CPU_C::HSUBPS_VpsWps(bxInstruction_c *i)
   BX_WRITE_XMM_REG(i->nnn(), result);
 
 #else
-  BX_INFO(("HSUBPS_VpsWps: required PNI, use --enable-pni option"));
+  BX_INFO(("HSUBPS_VpsWps: required SSE3, use --enable-sse option"));
   UndefinedOpcode(i);
 #endif
 }
@@ -2666,6 +2667,12 @@ void BX_CPU_C::CMPPS_VpsWpsIb(bxInstruction_c *i)
   mxcsr_to_softfloat_status_word(status, MXCSR);
   int ib = i->Ib();
 
+  /* mask used bits, ignore reserved */
+  if (ib > 7) {
+    BX_ERROR(("CMPPS_VpsWpsIb: unrecognized predicate %u", i->Ib()));
+  }
+  ib &= 7; 
+
   if (MXCSR.get_DAZ()) {
     op1.xmm32u(0) = handleDAZ(op1.xmm32u(0));
     op1.xmm32u(1) = handleDAZ(op1.xmm32u(1));
@@ -2689,7 +2696,7 @@ void BX_CPU_C::CMPPS_VpsWpsIb(bxInstruction_c *i)
     result.xmm32u(3) = 
         compare32[ib](op1.xmm32u(3), op2.xmm32u(3), status) ? 0xFFFFFFFF : 0;
   }
-  else if(ib < 8)
+  else
   {
     ib -= 4;
 
@@ -2702,9 +2709,6 @@ void BX_CPU_C::CMPPS_VpsWpsIb(bxInstruction_c *i)
     result.xmm32u(3) = 
         compare32[ib](op1.xmm32u(3), op2.xmm32u(3), status) ? 0 : 0xFFFFFFFF;
   }
-  else {
-    BX_PANIC(("CMPPS_VpsWpsIb: unrecognized predicate %u", ib));
-  }
 
   BX_CPU_THIS_PTR check_exceptionsSSE(status.float_exception_flags);
   BX_WRITE_XMM_REG(i->nnn(), result);
@@ -2740,6 +2744,12 @@ void BX_CPU_C::CMPPD_VpdWpdIb(bxInstruction_c *i)
   mxcsr_to_softfloat_status_word(status, MXCSR);
   int ib = i->Ib();
 
+  /* mask used bits, ignore reserved */
+  if (ib > 7) {
+    BX_ERROR(("CMPPD_VpdWpdIb: unrecognized predicate %u", i->Ib()));
+  }
+  ib &= 7; 
+
   if (MXCSR.get_DAZ()) 
   {
     op1.xmm64u(0) = handleDAZ(op1.xmm64u(0));
@@ -2755,7 +2765,7 @@ void BX_CPU_C::CMPPD_VpdWpdIb(bxInstruction_c *i)
     result.xmm64u(1) = compare64[ib](op1.xmm64u(1), op2.xmm64u(1), status) ? 
        BX_CONST64(0xFFFFFFFFFFFFFFFF) : 0;
   }
-  else if(ib < 8)
+  else
   {
     ib -= 4;
 
@@ -2764,9 +2774,6 @@ void BX_CPU_C::CMPPD_VpdWpdIb(bxInstruction_c *i)
     result.xmm64u(1) = compare64[ib](op1.xmm64u(1), op2.xmm64u(1), status) ? 
        0 : BX_CONST64(0xFFFFFFFFFFFFFFFF);
   }
-  else {
-    BX_PANIC(("CMPPS_VpdWpdIb: unrecognized predicate %u", ib));
-  }
 
   BX_CPU_THIS_PTR check_exceptionsSSE(status.float_exception_flags);
   BX_WRITE_XMM_REG(i->nnn(), result);
@@ -2802,6 +2809,12 @@ void BX_CPU_C::CMPSD_VsdWsdIb(bxInstruction_c *i)
   mxcsr_to_softfloat_status_word(status_word, MXCSR);
   int ib = i->Ib();
 
+  /* mask used bits, ignore reserved */
+  if (ib > 7) {
+    BX_ERROR(("CMPSD_VsdWsdIb: unrecognized predicate %u", i->Ib()));
+  }
+  ib &= 7; 
+
   if (MXCSR.get_DAZ()) 
   {
     op1 = handleDAZ(op1);
@@ -2814,19 +2827,16 @@ void BX_CPU_C::CMPSD_VsdWsdIb(bxInstruction_c *i)
      } else {
         result = 0;
      }
-  } else if(ib < 8) {
+  } else {
      if(compare64[ib-4](op1, op2, status_word)) {
         result = 0;
      } else {
         result = BX_CONST64(0xFFFFFFFFFFFFFFFF);
      }
-  } else {
-     BX_PANIC(("CMPPS_VsdWsdIb: unrecognized predicate %u", ib));
   }
 
   BX_CPU_THIS_PTR check_exceptionsSSE(status_word.float_exception_flags);
   BX_WRITE_XMM_REG_LO_QWORD(i->nnn(), result);
-
 #else
   BX_INFO(("CMPSD_VsdWsdIb: required SSE2, use --enable-sse option"));
   UndefinedOpcode(i);
@@ -2858,6 +2868,12 @@ void BX_CPU_C::CMPSS_VssWssIb(bxInstruction_c *i)
   mxcsr_to_softfloat_status_word(status_word, MXCSR);
   int ib = i->Ib();
 
+  /* mask used bits, ignore reserved */
+  if (ib > 7) {
+    BX_ERROR(("CMPSS_VssWssIb: unrecognized predicate %u", i->Ib()));
+  }
+  ib &= 7; 
+
   if (MXCSR.get_DAZ()) 
   {
     op1 = handleDAZ(op1);
@@ -2870,19 +2886,16 @@ void BX_CPU_C::CMPSS_VssWssIb(bxInstruction_c *i)
      } else {
         result = 0;
      }
-  } else if(ib < 8) {
+  } else {
      if(compare32[ib-4](op1, op2, status_word)) {
         result = 0;
      } else {
         result = 0xFFFFFFFF;
      }
-  } else {
-     BX_PANIC(("CMPPS_VssWssIb: unrecognized predicate %u", ib));
   }
 
   BX_CPU_THIS_PTR check_exceptionsSSE(status_word.float_exception_flags);
   BX_WRITE_XMM_REG_LO_DWORD(i->nnn(), result);
-
 #else
   BX_INFO(("CMPSS_VssWssIb: required SSE, use --enable-sse option"));
   UndefinedOpcode(i);
@@ -2896,7 +2909,7 @@ void BX_CPU_C::CMPSS_VssWssIb(bxInstruction_c *i)
  */
 void BX_CPU_C::ADDSUBPD_VpdWpd(bxInstruction_c *i)
 {
-#if BX_SUPPORT_PNI
+#if BX_SUPPORT_SSE >= 3
   BX_CPU_THIS_PTR prepareSSE();
 
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->nnn()), op2, result;
@@ -2930,7 +2943,7 @@ void BX_CPU_C::ADDSUBPD_VpdWpd(bxInstruction_c *i)
   BX_WRITE_XMM_REG(i->nnn(), result);
 
 #else
-  BX_INFO(("ADDSUBPD_VpdWpd: required PNI, use --enable-pni option"));
+  BX_INFO(("ADDSUBPD_VpdWpd: required SSE3, use --enable-sse option"));
   UndefinedOpcode(i);
 #endif
 }
@@ -2942,7 +2955,7 @@ void BX_CPU_C::ADDSUBPD_VpdWpd(bxInstruction_c *i)
  */
 void BX_CPU_C::ADDSUBPS_VpsWps(bxInstruction_c *i)
 {
-#if BX_SUPPORT_PNI
+#if BX_SUPPORT_SSE >= 3
   BX_CPU_THIS_PTR prepareSSE();
 
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->nnn()), op2, result;
@@ -2984,7 +2997,7 @@ void BX_CPU_C::ADDSUBPS_VpsWps(bxInstruction_c *i)
   BX_WRITE_XMM_REG(i->nnn(), result);
 
 #else
-  BX_INFO(("ADDSUBPS_VpsWps: required PNI, use --enable-pni option"));
+  BX_INFO(("ADDSUBPS_VpsWps: required SSE3, use --enable-sse option"));
   UndefinedOpcode(i);
 #endif
 }
