@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: string.cc,v 1.31 2005/08/01 21:40:17 sshwarts Exp $
+// $Id: string.cc,v 1.35 2006/05/24 20:57:37 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -28,6 +28,7 @@
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
@@ -67,7 +68,7 @@ Bit32u BX_CPU_C::FastRepMOVSB(bxInstruction_c *i, unsigned srcSeg, bx_address sr
   // we need the A20 address.
   paddrSrc = A20ADDR(paddrSrc);
   Bit8u *hostAddrSrc = BX_CPU_THIS_PTR mem->getHostMemAddr(BX_CPU_THIS,
-            paddrSrc, BX_READ);
+            paddrSrc, BX_READ, DATA_ACCESS);
 
   if (! hostAddrSrc) return 0;
 
@@ -84,12 +85,12 @@ Bit32u BX_CPU_C::FastRepMOVSB(bxInstruction_c *i, unsigned srcSeg, bx_address sr
   // we need the A20 address.
   paddrDst = A20ADDR(paddrDst);
   Bit8u *hostAddrDst = BX_CPU_THIS_PTR mem->getHostMemAddr(BX_CPU_THIS,
-            paddrDst, BX_WRITE);
+            paddrDst, BX_WRITE, DATA_ACCESS);
 
   if (! hostAddrDst) return 0;
 
   // See how many bytes can fit in the rest of this page.
-  if (BX_CPU_THIS_PTR get_DF ()) {
+  if (BX_CPU_THIS_PTR get_DF()) {
     // Counting downward.
     bytesFitSrc = 1 + (paddrSrc & 0xfff);
     bytesFitDst = 1 + (paddrDst & 0xfff);
@@ -142,7 +143,7 @@ Bit32u BX_CPU_C::FastRepMOVSB(bxInstruction_c *i, unsigned srcSeg, bx_address sr
       // Now make sure transfer will fit within the constraints of the
       // segment boundaries, 0..limit for non expand-down.  We know
       // count >= 1 here.
-      if (BX_CPU_THIS_PTR get_DF ()) {
+      if (BX_CPU_THIS_PTR get_DF()) {
         Bit32u minOffset = (count-1);
         if ( srcOff < minOffset )
           return 0;
@@ -201,7 +202,7 @@ Bit32u BX_CPU_C::FastRepMOVSW(bxInstruction_c *i, unsigned srcSeg, bx_address sr
   // we need the A20 address.
   paddrSrc = A20ADDR(paddrSrc);
   Bit8u *hostAddrSrc = BX_CPU_THIS_PTR mem->getHostMemAddr(BX_CPU_THIS,
-            paddrSrc, BX_READ);
+            paddrSrc, BX_READ, DATA_ACCESS);
 
   if (! hostAddrSrc) return 0;
 
@@ -218,12 +219,12 @@ Bit32u BX_CPU_C::FastRepMOVSW(bxInstruction_c *i, unsigned srcSeg, bx_address sr
   // we need the A20 address.
   paddrDst = A20ADDR(paddrDst);
   Bit8u *hostAddrDst = BX_CPU_THIS_PTR mem->getHostMemAddr(BX_CPU_THIS,
-            paddrDst, BX_WRITE);
+            paddrDst, BX_WRITE, DATA_ACCESS);
 
   if (! hostAddrDst) return 0;
 
   // See how many words can fit in the rest of this page.
-  if (BX_CPU_THIS_PTR get_DF ()) {
+  if (BX_CPU_THIS_PTR get_DF()) {
     // Counting downward.
     // Note: 1st word must not cross page boundary.
     if ( ((paddrSrc & 0xfff) > 0xffe) || ((paddrDst & 0xfff) > 0xffe) )
@@ -279,7 +280,7 @@ Bit32u BX_CPU_C::FastRepMOVSW(bxInstruction_c *i, unsigned srcSeg, bx_address sr
       // Now make sure transfer will fit within the constraints of the
       // segment boundaries, 0..limit for non expand-down.  We know
       // count >= 1 here.
-      if (BX_CPU_THIS_PTR get_DF ()) {
+      if (BX_CPU_THIS_PTR get_DF()) {
         // Counting downward.
         Bit32u minOffset = (count-1) << 1;
         if ( srcOff < minOffset )
@@ -339,7 +340,7 @@ Bit32u BX_CPU_C::FastRepMOVSD(bxInstruction_c *i, unsigned srcSeg, bx_address sr
   // we need the A20 address.
   paddrSrc = A20ADDR(paddrSrc);
   Bit8u *hostAddrSrc = BX_CPU_THIS_PTR mem->getHostMemAddr(BX_CPU_THIS,
-            paddrSrc, BX_READ);
+            paddrSrc, BX_READ, DATA_ACCESS);
 
   if (! hostAddrSrc) return 0;
 
@@ -356,12 +357,12 @@ Bit32u BX_CPU_C::FastRepMOVSD(bxInstruction_c *i, unsigned srcSeg, bx_address sr
   // we need the A20 address.
   paddrDst = A20ADDR(paddrDst);
   Bit8u *hostAddrDst = BX_CPU_THIS_PTR mem->getHostMemAddr(BX_CPU_THIS,
-            paddrDst, BX_WRITE);
+            paddrDst, BX_WRITE, DATA_ACCESS);
 
   if (! hostAddrDst) return 0;
 
   // See how many dwords can fit in the rest of this page.
-  if (BX_CPU_THIS_PTR get_DF ()) {
+  if (BX_CPU_THIS_PTR get_DF()) {
     // Counting downward.
     // Note: 1st dword must not cross page boundary.
     if ( ((paddrSrc & 0xfff) > 0xffc) || ((paddrDst & 0xfff) > 0xffc) )
@@ -417,7 +418,7 @@ Bit32u BX_CPU_C::FastRepMOVSD(bxInstruction_c *i, unsigned srcSeg, bx_address sr
       // Now make sure transfer will fit within the constraints of the
       // segment boundaries, 0..limit for non expand-down.  We know
       // count >= 1 here.
-      if (BX_CPU_THIS_PTR get_DF ()) {
+      if (BX_CPU_THIS_PTR get_DF()) {
         // Counting downward.
         Bit32u minOffset = (count-1) << 2;
         if ( srcOff < minOffset )
@@ -471,12 +472,12 @@ Bit32u BX_CPU_C::FastRepSTOSB(bxInstruction_c *i, unsigned dstSeg, bx_address ds
   // we need the A20 address.
   paddrDst = A20ADDR(paddrDst);
   Bit8u *hostAddrDst = BX_CPU_THIS_PTR mem->getHostMemAddr(BX_CPU_THIS,
-            paddrDst, BX_WRITE);
+            paddrDst, BX_WRITE, DATA_ACCESS);
 
   if (! hostAddrDst) return 0;
 
   // See how many bytes can fit in the rest of this page.
-  if (BX_CPU_THIS_PTR get_DF ()) {
+  if (BX_CPU_THIS_PTR get_DF()) {
     // Counting downward.
     bytesFitDst = 1 + (paddrDst & 0xfff);
     pointerDelta = (signed int) -1;
@@ -518,7 +519,7 @@ Bit32u BX_CPU_C::FastRepSTOSB(bxInstruction_c *i, unsigned dstSeg, bx_address ds
       // Now make sure transfer will fit within the constraints of the
       // segment boundaries, 0..limit for non expand-down.  We know
       // count >= 1 here.
-      if (BX_CPU_THIS_PTR get_DF ()) {
+      if (BX_CPU_THIS_PTR get_DF()) {
         Bit32u minOffset = (count-1);
         if ( dstOff < minOffset )
           return 0;
@@ -565,12 +566,12 @@ Bit32u BX_CPU_C::FastRepSTOSW(bxInstruction_c *i, unsigned dstSeg, bx_address ds
   // we need the A20 address.
   paddrDst = A20ADDR(paddrDst);
   Bit8u *hostAddrDst = BX_CPU_THIS_PTR mem->getHostMemAddr(BX_CPU_THIS,
-            paddrDst, BX_WRITE);
+            paddrDst, BX_WRITE, DATA_ACCESS);
 
   if (! hostAddrDst) return 0;
 
   // See how many words can fit in the rest of this page.
-  if (BX_CPU_THIS_PTR get_DF ()) {
+  if (BX_CPU_THIS_PTR get_DF()) {
     // Counting downward.
     // Note: 1st word must not cross page boundary.
     if ((paddrDst & 0xfff) > 0xffe) return 0;
@@ -614,7 +615,7 @@ Bit32u BX_CPU_C::FastRepSTOSW(bxInstruction_c *i, unsigned dstSeg, bx_address ds
       // Now make sure transfer will fit within the constraints of the
       // segment boundaries, 0..limit for non expand-down.  We know
       // count >= 1 here.
-      if (BX_CPU_THIS_PTR get_DF ()) {
+      if (BX_CPU_THIS_PTR get_DF()) {
         // Counting downward.
         Bit32u minOffset = (count-1) << 1;
         if ( dstOff < minOffset )
@@ -662,12 +663,12 @@ Bit32u BX_CPU_C::FastRepSTOSD(bxInstruction_c *i, unsigned dstSeg, bx_address ds
   // we need the A20 address.
   paddrDst = A20ADDR(paddrDst);
   Bit8u *hostAddrDst = BX_CPU_THIS_PTR mem->getHostMemAddr(BX_CPU_THIS,
-            paddrDst, BX_WRITE);
+            paddrDst, BX_WRITE, DATA_ACCESS);
 
   if (! hostAddrDst) return 0;
 
   // See how many dwords can fit in the rest of this page.
-  if (BX_CPU_THIS_PTR get_DF ()) {
+  if (BX_CPU_THIS_PTR get_DF()) {
     // Counting downward.
     // Note: 1st dword must not cross page boundary.
     if ((paddrDst & 0xfff) > 0xffc) return 0;    
@@ -711,7 +712,7 @@ Bit32u BX_CPU_C::FastRepSTOSD(bxInstruction_c *i, unsigned dstSeg, bx_address ds
       // Now make sure transfer will fit within the constraints of the
       // segment boundaries, 0..limit for non expand-down.  We know
       // count >= 1 here.
-      if (BX_CPU_THIS_PTR get_DF ()) {
+      if (BX_CPU_THIS_PTR get_DF()) {
         // Counting downward.
         Bit32u minOffset = (count-1) << 2;
         if ( dstOff < minOffset )
@@ -744,27 +745,17 @@ Bit32u BX_CPU_C::FastRepSTOSD(bxInstruction_c *i, unsigned dstSeg, bx_address ds
  */
 void BX_CPU_C::MOVSB_XbYb(bxInstruction_c *i)
 {
-  unsigned seg;
   Bit8u temp8;
 
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    seg = i->seg();
-  }
-  else {
-    seg = BX_SEG_REG_DS;
-  }
-
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
-    Bit64u rsi, rdi;
+    Bit64u rsi = RSI;
+    Bit64u rdi = RDI;
 
-    rsi = RSI;
-    rdi = RDI;
-
-    read_virtual_byte(seg, rsi, &temp8);
+    read_virtual_byte(i->seg(), rsi, &temp8);
     write_virtual_byte(BX_SEG_REG_ES, rdi, &temp8);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       /* decrement RSI, RDI */
       rsi--;
       rdi--;
@@ -782,15 +773,13 @@ void BX_CPU_C::MOVSB_XbYb(bxInstruction_c *i)
 #endif  // #if BX_SUPPORT_X86_64
   if (i->as32L())
   {
-    Bit32u esi, edi;
+    Bit32u esi = ESI;
+    Bit32u edi = EDI;
 
-    esi = ESI;
-    edi = EDI;
-
-    read_virtual_byte(seg, esi, &temp8);
+    read_virtual_byte(i->seg(), esi, &temp8);
     write_virtual_byte(BX_SEG_REG_ES, edi, &temp8);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       /* decrement ESI, EDI */
       esi--;
       edi--;
@@ -820,7 +809,7 @@ void BX_CPU_C::MOVSB_XbYb(bxInstruction_c *i)
     {
       Bit32u byteCount = CX;
       BX_ASSERT(byteCount > 0);
-      byteCount = FastRepMOVSB(i, seg, si, BX_SEG_REG_ES, di, byteCount);
+      byteCount = FastRepMOVSB(i, i->seg(), si, BX_SEG_REG_ES, di, byteCount);
       if (byteCount)
       {
         // Decrement the ticks count by the number of iterations, minus
@@ -840,7 +829,7 @@ void BX_CPU_C::MOVSB_XbYb(bxInstruction_c *i)
 #endif  // (BX_DEBUGGER == 0)
 #endif  // BX_SupportRepeatSpeedups
 
-    read_virtual_byte(seg, si, &temp8);
+    read_virtual_byte(i->seg(), si, &temp8);
     write_virtual_byte(BX_SEG_REG_ES, di, &temp8);
 
 #if BX_SupportRepeatSpeedups
@@ -849,7 +838,7 @@ doIncr16:
 #endif  // (BX_DEBUGGER == 0)
 #endif  // BX_SupportRepeatSpeedups
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       /* decrement SI, DI */
       si -= incr;
       di -= incr;
@@ -869,25 +858,16 @@ doIncr16:
 void BX_CPU_C::MOVSW_XwYw(bxInstruction_c *i)
 {
   Bit16u temp16;
-  unsigned seg;
-
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    seg = i->seg();
-  }
-  else {
-    seg = BX_SEG_REG_DS;
-  }
 
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
-
     Bit64u rsi = RSI;
     Bit64u rdi = RDI;
 
-    read_virtual_word(seg, rsi, &temp16);
+    read_virtual_word(i->seg(), rsi, &temp16);
     write_virtual_word(BX_SEG_REG_ES, rdi, &temp16);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rsi -= 2;
       rdi -= 2;
     }
@@ -906,10 +886,10 @@ void BX_CPU_C::MOVSW_XwYw(bxInstruction_c *i)
     Bit32u esi = ESI;
     Bit32u edi = EDI;
 
-    read_virtual_word(seg, esi, &temp16);
+    read_virtual_word(i->seg(), esi, &temp16);
     write_virtual_word(BX_SEG_REG_ES, edi, &temp16);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       esi -= 2;
       edi -= 2;
     }
@@ -938,7 +918,7 @@ void BX_CPU_C::MOVSW_XwYw(bxInstruction_c *i)
     {
       Bit32u wordCount = CX;
       BX_ASSERT(wordCount > 0);
-      wordCount = FastRepMOVSW(i, seg, si, BX_SEG_REG_ES, di, wordCount);
+      wordCount = FastRepMOVSW(i, i->seg(), si, BX_SEG_REG_ES, di, wordCount);
       if (wordCount)
       {
         // Decrement the ticks count by the number of iterations, minus
@@ -958,7 +938,7 @@ void BX_CPU_C::MOVSW_XwYw(bxInstruction_c *i)
 #endif  // (BX_DEBUGGER == 0)
 #endif  // BX_SupportRepeatSpeedups
 
-    read_virtual_word(seg, si, &temp16);
+    read_virtual_word(i->seg(), si, &temp16);
     write_virtual_word(BX_SEG_REG_ES, di, &temp16);
 
 #if BX_SupportRepeatSpeedups
@@ -967,7 +947,7 @@ doIncr16:
 #endif
 #endif
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       /* decrement SI, DI */
       si -= incr;
       di -= incr;
@@ -987,24 +967,16 @@ doIncr16:
 void BX_CPU_C::MOVSD_XdYd(bxInstruction_c *i)
 {
   Bit32u temp32;
-  unsigned seg;
-
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    seg = i->seg();
-  }
-  else {
-    seg = BX_SEG_REG_DS;
-  }
 
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
     Bit64u rsi = RSI;
     Bit64u rdi = RDI;
 
-    read_virtual_dword(seg, rsi, &temp32);
+    read_virtual_dword(i->seg(), rsi, &temp32);
     write_virtual_dword(BX_SEG_REG_ES, rdi, &temp32);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rsi -= 4;
       rdi -= 4;
     }
@@ -1034,7 +1006,7 @@ void BX_CPU_C::MOVSD_XdYd(bxInstruction_c *i)
     {
       Bit32u dwordCount = ECX;
       BX_ASSERT(dwordCount > 0);
-      dwordCount = FastRepMOVSD(i, seg, esi, BX_SEG_REG_ES, edi, dwordCount);
+      dwordCount = FastRepMOVSD(i, i->seg(), esi, BX_SEG_REG_ES, edi, dwordCount);
       if (dwordCount)
       {
         // Decrement the ticks count by the number of iterations, minus
@@ -1054,7 +1026,7 @@ void BX_CPU_C::MOVSD_XdYd(bxInstruction_c *i)
 #endif  // (BX_DEBUGGER == 0)
 #endif  // BX_SupportRepeatSpeedups
 
-    read_virtual_dword(seg, esi, &temp32);
+    read_virtual_dword(i->seg(), esi, &temp32);
     write_virtual_dword(BX_SEG_REG_ES, edi, &temp32);
 
 #if BX_SupportRepeatSpeedups
@@ -1063,7 +1035,7 @@ doIncr32:
 #endif
 #endif
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       esi -= incr;
       edi -= incr;
     }
@@ -1081,10 +1053,10 @@ doIncr32:
     Bit16u si = SI;
     Bit16u di = DI;
 
-    read_virtual_dword(seg, si, &temp32);
+    read_virtual_dword(i->seg(), si, &temp32);
     write_virtual_dword(BX_SEG_REG_ES, di, &temp32);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       si -= 4;
       di -= 4;
     }
@@ -1104,23 +1076,15 @@ doIncr32:
 void BX_CPU_C::MOVSQ_XqYq(bxInstruction_c *i)
 {
   Bit64u temp64;
-  unsigned seg;
-
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    seg = i->seg();
-  }
-  else {
-    seg = BX_SEG_REG_DS;
-  }
 
   if (i->as64L()) {
     Bit64u rsi = RSI;
     Bit64u rdi = RDI;
 
-    read_virtual_qword(seg, rsi, &temp64);
+    read_virtual_qword(i->seg(), rsi, &temp64);
     write_virtual_qword(BX_SEG_REG_ES, rdi, &temp64);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rsi -= 8;
       rdi -= 8;
     }
@@ -1137,10 +1101,10 @@ void BX_CPU_C::MOVSQ_XqYq(bxInstruction_c *i)
     Bit32u esi = ESI;
     Bit32u edi = EDI;
 
-    read_virtual_qword(seg, esi, &temp64);
+    read_virtual_qword(i->seg(), esi, &temp64);
     write_virtual_qword(BX_SEG_REG_ES, edi, &temp64);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       esi -= 8;
       edi -= 8;
     }
@@ -1159,29 +1123,21 @@ void BX_CPU_C::MOVSQ_XqYq(bxInstruction_c *i)
 
 void BX_CPU_C::CMPSB_XbYb(bxInstruction_c *i)
 {
-  unsigned seg;
   Bit8u op1_8, op2_8, diff_8;
 
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    seg = i->seg();
-  }
-  else {
-    seg = BX_SEG_REG_DS;
-  }
-
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
     Bit64u rsi = RSI;
     Bit64u rdi = RDI;
 
-    read_virtual_byte(seg, rsi, &op1_8);
+    read_virtual_byte(i->seg(), rsi, &op1_8);
     read_virtual_byte(BX_SEG_REG_ES, rdi, &op2_8);
 
     diff_8 = op1_8 - op2_8;
 
     SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_COMPARE8);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rsi--;
       rdi--;
     }
@@ -1199,14 +1155,14 @@ void BX_CPU_C::CMPSB_XbYb(bxInstruction_c *i)
     Bit32u esi = ESI;
     Bit32u edi = EDI;
 
-    read_virtual_byte(seg, esi, &op1_8);
+    read_virtual_byte(i->seg(), esi, &op1_8);
     read_virtual_byte(BX_SEG_REG_ES, edi, &op2_8);
 
     diff_8 = op1_8 - op2_8;
 
     SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_COMPARE8);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       esi--;
       edi--;
     }
@@ -1224,14 +1180,14 @@ void BX_CPU_C::CMPSB_XbYb(bxInstruction_c *i)
     Bit16u si = SI;
     Bit16u di = DI;
 
-    read_virtual_byte(seg, si, &op1_8);
+    read_virtual_byte(i->seg(), si, &op1_8);
     read_virtual_byte(BX_SEG_REG_ES, di, &op2_8);
 
     diff_8 = op1_8 - op2_8;
 
     SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_COMPARE8);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       si--;
       di--;
     }
@@ -1249,30 +1205,20 @@ void BX_CPU_C::CMPSB_XbYb(bxInstruction_c *i)
 void BX_CPU_C::CMPSW_XwYw(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, diff_16;
-  unsigned seg;
-
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    seg = i->seg();
-  }
-  else {
-    seg = BX_SEG_REG_DS;
-  }
 
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
-    Bit64u rsi, rdi;
+    Bit64u rsi = RSI;
+    Bit64u rdi = RDI;
 
-    rsi = RSI;
-    rdi = RDI;
-
-    read_virtual_word(seg, rsi, &op1_16);
+    read_virtual_word(i->seg(), rsi, &op1_16);
     read_virtual_word(BX_SEG_REG_ES, rdi, &op2_16);
 
     diff_16 = op1_16 - op2_16;
 
     SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_INSTR_COMPARE16);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rsi -= 2;
       rdi -= 2;
     }
@@ -1290,14 +1236,14 @@ void BX_CPU_C::CMPSW_XwYw(bxInstruction_c *i)
     Bit32u esi = ESI;
     Bit32u edi = EDI;
 
-    read_virtual_word(seg, esi, &op1_16);
+    read_virtual_word(i->seg(), esi, &op1_16);
     read_virtual_word(BX_SEG_REG_ES, edi, &op2_16);
 
     diff_16 = op1_16 - op2_16;
 
     SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_INSTR_COMPARE16);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       esi -= 2;
       edi -= 2;
     }
@@ -1315,13 +1261,13 @@ void BX_CPU_C::CMPSW_XwYw(bxInstruction_c *i)
     Bit16u si = SI;
     Bit16u di = DI;
 
-    read_virtual_word(seg, si, &op1_16);
+    read_virtual_word(i->seg(), si, &op1_16);
     read_virtual_word(BX_SEG_REG_ES, di, &op2_16);
 
     diff_16 = op1_16 - op2_16;
     SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_INSTR_COMPARE16);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       si -= 2;
       di -= 2;
     }
@@ -1339,14 +1285,6 @@ void BX_CPU_C::CMPSW_XwYw(bxInstruction_c *i)
 void BX_CPU_C::CMPSD_XdYd(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, diff_32;
-  unsigned seg;
-
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    seg = i->seg();
-  }
-  else {
-    seg = BX_SEG_REG_DS;
-  }
 
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
@@ -1355,14 +1293,14 @@ void BX_CPU_C::CMPSD_XdYd(bxInstruction_c *i)
 
     Bit32u op1_32, op2_32, diff_32;
 
-    read_virtual_dword(seg, rsi, &op1_32);
+    read_virtual_dword(i->seg(), rsi, &op1_32);
     read_virtual_dword(BX_SEG_REG_ES, rdi, &op2_32);
 
     diff_32 = op1_32 - op2_32;
 
     SET_FLAGS_OSZAPC_32(op1_32, op2_32, diff_32, BX_INSTR_COMPARE32);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rsi -= 4;
       rdi -= 4;
     }
@@ -1380,14 +1318,14 @@ void BX_CPU_C::CMPSD_XdYd(bxInstruction_c *i)
     Bit32u esi = ESI;
     Bit32u edi = EDI;
 
-    read_virtual_dword(seg, esi, &op1_32);
+    read_virtual_dword(i->seg(), esi, &op1_32);
     read_virtual_dword(BX_SEG_REG_ES, edi, &op2_32);
 
     diff_32 = op1_32 - op2_32;
 
     SET_FLAGS_OSZAPC_32(op1_32, op2_32, diff_32, BX_INSTR_COMPARE32);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       esi -= 4;
       edi -= 4;
     }
@@ -1405,14 +1343,14 @@ void BX_CPU_C::CMPSD_XdYd(bxInstruction_c *i)
     Bit16u si = SI;
     Bit16u di = DI;
 
-    read_virtual_dword(seg, si, &op1_32);
+    read_virtual_dword(i->seg(), si, &op1_32);
     read_virtual_dword(BX_SEG_REG_ES, di, &op2_32);
 
     diff_32 = op1_32 - op2_32;
 
     SET_FLAGS_OSZAPC_32(op1_32, op2_32, diff_32, BX_INSTR_COMPARE32);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       si -= 4;
       di -= 4;
     }
@@ -1432,27 +1370,19 @@ void BX_CPU_C::CMPSD_XdYd(bxInstruction_c *i)
 void BX_CPU_C::CMPSQ_XqYq(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, diff_64;
-  unsigned seg;
-
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    seg = i->seg();
-  }
-  else {
-    seg = BX_SEG_REG_DS;
-  }
 
   if (i->as64L()) {
     Bit64u rsi = RSI;
     Bit64u rdi = RDI;
 
-    read_virtual_qword(seg, rsi, &op1_64);
+    read_virtual_qword(i->seg(), rsi, &op1_64);
     read_virtual_qword(BX_SEG_REG_ES, rdi, &op2_64);
 
     diff_64 = op1_64 - op2_64;
 
     SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_COMPARE64);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rsi -= 8;
       rdi -= 8;
     }
@@ -1469,14 +1399,14 @@ void BX_CPU_C::CMPSQ_XqYq(bxInstruction_c *i)
     Bit32u esi = ESI;
     Bit32u edi = EDI;
 
-    read_virtual_qword(seg, esi, &op1_64);
+    read_virtual_qword(i->seg(), esi, &op1_64);
     read_virtual_qword(BX_SEG_REG_ES, edi, &op2_64);
 
     diff_64 = op1_64 - op2_64;
 
     SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_COMPARE64);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       esi -= 8;
       edi -= 8;
     }
@@ -1495,9 +1425,7 @@ void BX_CPU_C::CMPSQ_XqYq(bxInstruction_c *i)
 
 void BX_CPU_C::SCASB_ALXb(bxInstruction_c *i)
 {
-  Bit8u op1_8, op2_8, diff_8;
-
-  op1_8 = AL;
+  Bit8u op1_8 = AL, op2_8, diff_8;
 
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
@@ -1509,7 +1437,7 @@ void BX_CPU_C::SCASB_ALXb(bxInstruction_c *i)
 
     SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_COMPARE8);
  
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rdi--;
     }
     else {
@@ -1529,7 +1457,7 @@ void BX_CPU_C::SCASB_ALXb(bxInstruction_c *i)
 
     SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_COMPARE8);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       edi--;
     }
     else {
@@ -1549,7 +1477,7 @@ void BX_CPU_C::SCASB_ALXb(bxInstruction_c *i)
 
     SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_COMPARE8);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       di--;
     }
     else {
@@ -1563,9 +1491,7 @@ void BX_CPU_C::SCASB_ALXb(bxInstruction_c *i)
 /* 16 bit opsize mode */
 void BX_CPU_C::SCASW_AXXw(bxInstruction_c *i)
 {
-  Bit16u op1_16, op2_16, diff_16;
-
-  op1_16 = AX;
+  Bit16u op1_16 = AX, op2_16, diff_16;
 
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
@@ -1576,7 +1502,7 @@ void BX_CPU_C::SCASW_AXXw(bxInstruction_c *i)
 
     SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_INSTR_COMPARE16);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rdi -= 2;
     }
     else {
@@ -1595,7 +1521,7 @@ void BX_CPU_C::SCASW_AXXw(bxInstruction_c *i)
 
     SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_INSTR_COMPARE16);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       edi -= 2;
     }
     else {
@@ -1614,7 +1540,7 @@ void BX_CPU_C::SCASW_AXXw(bxInstruction_c *i)
 
     SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_INSTR_COMPARE16);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       di -= 2;
     }
     else {
@@ -1628,9 +1554,7 @@ void BX_CPU_C::SCASW_AXXw(bxInstruction_c *i)
 /* 32 bit opsize mode */
 void BX_CPU_C::SCASD_EAXXd(bxInstruction_c *i)
 {
-  Bit32u op1_32, op2_32, diff_32;
-
-  op1_32 = EAX;
+  Bit32u op1_32 = EAX, op2_32, diff_32;
 
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
@@ -1641,7 +1565,7 @@ void BX_CPU_C::SCASD_EAXXd(bxInstruction_c *i)
 
     SET_FLAGS_OSZAPC_32(op1_32, op2_32, diff_32, BX_INSTR_COMPARE32);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rdi -= 4;
     }
     else {
@@ -1660,7 +1584,7 @@ void BX_CPU_C::SCASD_EAXXd(bxInstruction_c *i)
 
     SET_FLAGS_OSZAPC_32(op1_32, op2_32, diff_32, BX_INSTR_COMPARE32);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       edi -= 4;
     }
     else {
@@ -1679,7 +1603,7 @@ void BX_CPU_C::SCASD_EAXXd(bxInstruction_c *i)
 
     SET_FLAGS_OSZAPC_32(op1_32, op2_32, diff_32, BX_INSTR_COMPARE32);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       di -= 4;
     }
     else {
@@ -1695,9 +1619,7 @@ void BX_CPU_C::SCASD_EAXXd(bxInstruction_c *i)
 /* 64 bit opsize mode */
 void BX_CPU_C::SCASQ_RAXXq(bxInstruction_c *i)
 {
-  Bit64u op1_64, op2_64, diff_64;
-
-  op1_64 = RAX;
+  Bit64u op1_64 = RAX, op2_64, diff_64;
 
   if (i->as64L()) {
     Bit64u rdi = RDI;
@@ -1707,7 +1629,7 @@ void BX_CPU_C::SCASQ_RAXXq(bxInstruction_c *i)
 
     SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_COMPARE64);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rdi -= 8;
     }
     else {
@@ -1725,7 +1647,7 @@ void BX_CPU_C::SCASQ_RAXXq(bxInstruction_c *i)
 
     SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_COMPARE64);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       edi -= 8;
     }
     else {
@@ -1749,7 +1671,7 @@ void BX_CPU_C::STOSB_YbAL(bxInstruction_c *i)
 
     write_virtual_byte(BX_SEG_REG_ES, rdi, &al);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rdi--;
     }
     else {
@@ -1818,7 +1740,7 @@ doIncr16:
 #endif
 #endif
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       edi -= incr;
     }
     else {
@@ -1844,7 +1766,7 @@ void BX_CPU_C::STOSW_YwAX(bxInstruction_c *i)
 
     write_virtual_word(BX_SEG_REG_ES, rdi, &ax);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rdi -= 2;
     }
     else {
@@ -1861,7 +1783,7 @@ void BX_CPU_C::STOSW_YwAX(bxInstruction_c *i)
 
     write_virtual_word(BX_SEG_REG_ES, edi, &ax);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       edi -= 2;
     }
     else {
@@ -1877,7 +1799,7 @@ void BX_CPU_C::STOSW_YwAX(bxInstruction_c *i)
 
     write_virtual_word(BX_SEG_REG_ES, di, &ax);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       di -= 2;
     }
     else {
@@ -1899,7 +1821,7 @@ void BX_CPU_C::STOSD_YdEAX(bxInstruction_c *i)
 
     write_virtual_dword(BX_SEG_REG_ES, rdi, &eax);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rdi -= 4;
     }
     else {
@@ -1916,7 +1838,7 @@ void BX_CPU_C::STOSD_YdEAX(bxInstruction_c *i)
 
     write_virtual_dword(BX_SEG_REG_ES, edi, &eax);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       edi -= 4;
     }
     else {
@@ -1932,7 +1854,7 @@ void BX_CPU_C::STOSD_YdEAX(bxInstruction_c *i)
 
     write_virtual_dword(BX_SEG_REG_ES, di, &eax);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       di -= 4;
     }
     else {
@@ -1955,7 +1877,7 @@ void BX_CPU_C::STOSQ_YqRAX(bxInstruction_c *i)
 
     write_virtual_qword(BX_SEG_REG_ES, rdi, &rax);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rdi -= 8;
     }
     else {
@@ -1970,7 +1892,7 @@ void BX_CPU_C::STOSQ_YqRAX(bxInstruction_c *i)
 
     write_virtual_qword(BX_SEG_REG_ES, edi, &rax);
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       edi -= 8;
     }
     else {
@@ -1986,24 +1908,16 @@ void BX_CPU_C::STOSQ_YqRAX(bxInstruction_c *i)
 
 void BX_CPU_C::LODSB_ALXb(bxInstruction_c *i)
 {
-  unsigned seg;
   Bit8u al;
 
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    seg = i->seg();
-  }
-  else {
-    seg = BX_SEG_REG_DS;
-  }
-
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
     Bit64u rsi = RSI;
 
-    read_virtual_byte(seg, rsi, &al);
+    read_virtual_byte(i->seg(), rsi, &al);
 
     AL = al;
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rsi--;
     }
     else {
@@ -2018,10 +1932,10 @@ void BX_CPU_C::LODSB_ALXb(bxInstruction_c *i)
   {
     Bit32u esi = ESI;
 
-    read_virtual_byte(seg, esi, &al);
+    read_virtual_byte(i->seg(), esi, &al);
 
     AL = al;
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       esi--;
     }
     else {
@@ -2035,10 +1949,10 @@ void BX_CPU_C::LODSB_ALXb(bxInstruction_c *i)
   { /* 16bit address mode */
     Bit16u si = SI;
 
-    read_virtual_byte(seg, si, &al);
+    read_virtual_byte(i->seg(), si, &al);
 
     AL = al;
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       si--;
     }
     else {
@@ -2052,24 +1966,16 @@ void BX_CPU_C::LODSB_ALXb(bxInstruction_c *i)
 /* 16 bit opsize mode */
 void BX_CPU_C::LODSW_AXXw(bxInstruction_c *i)
 {
-  unsigned seg;
   Bit16u ax;
 
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    seg = i->seg();
-  }
-  else {
-    seg = BX_SEG_REG_DS;
-  }
-
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
     Bit64u rsi = RSI;
 
-    read_virtual_word(seg, rsi, &ax);
+    read_virtual_word(i->seg(), rsi, &ax);
     AX = ax;
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rsi -= 2;
     }
     else {
@@ -2084,10 +1990,10 @@ void BX_CPU_C::LODSW_AXXw(bxInstruction_c *i)
   {
     Bit32u esi = ESI;
 
-    read_virtual_word(seg, esi, &ax);
+    read_virtual_word(i->seg(), esi, &ax);
     AX = ax;
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       esi -= 2;
     }
     else {
@@ -2101,10 +2007,10 @@ void BX_CPU_C::LODSW_AXXw(bxInstruction_c *i)
   { /* 16bit address mode */
     Bit16u si = SI;
 
-    read_virtual_word(seg, si, &ax);
+    read_virtual_word(i->seg(), si, &ax);
     AX = ax;
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       si -= 2;
     }
     else {
@@ -2118,24 +2024,16 @@ void BX_CPU_C::LODSW_AXXw(bxInstruction_c *i)
 /* 32 bit opsize mode */
 void BX_CPU_C::LODSD_EAXXd(bxInstruction_c *i)
 {
-  unsigned seg;
   Bit32u eax;
 
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    seg = i->seg();
-  }
-  else {
-    seg = BX_SEG_REG_DS;
-  }
-
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
     Bit64u rsi = RSI;
 
-    read_virtual_dword(seg, rsi, &eax);
+    read_virtual_dword(i->seg(), rsi, &eax);
     RAX = eax;
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rsi -= 4;
     }
     else {
@@ -2150,10 +2048,10 @@ void BX_CPU_C::LODSD_EAXXd(bxInstruction_c *i)
   {
     Bit32u esi = ESI;
 
-    read_virtual_dword(seg, esi, &eax);
+    read_virtual_dword(i->seg(), esi, &eax);
     RAX = eax;
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       esi -= 4;
     }
     else {
@@ -2167,10 +2065,10 @@ void BX_CPU_C::LODSD_EAXXd(bxInstruction_c *i)
   { /* 16bit address mode */
     Bit16u si = SI;
 
-    read_virtual_dword(seg, si, &eax);
+    read_virtual_dword(i->seg(), si, &eax);
     RAX = eax;
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       si -= 4;
     }
     else {
@@ -2186,23 +2084,15 @@ void BX_CPU_C::LODSD_EAXXd(bxInstruction_c *i)
 /* 64 bit opsize mode */
 void BX_CPU_C::LODSQ_RAXXq(bxInstruction_c *i)
 {
-  unsigned seg;
   Bit64u rax;
 
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    seg = i->seg();
-  }
-  else {
-    seg = BX_SEG_REG_DS;
-  }
-
   if (i->as64L()) {
     Bit64u rsi = RSI;
 
-    read_virtual_qword(seg, rsi, &rax);
+    read_virtual_qword(i->seg(), rsi, &rax);
     RAX = rax;
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       rsi -= 8;
     }
     else {
@@ -2215,10 +2105,10 @@ void BX_CPU_C::LODSQ_RAXXq(bxInstruction_c *i)
   {
     Bit32u esi = ESI;
 
-    read_virtual_qword(seg, esi, &rax);
+    read_virtual_qword(i->seg(), esi, &rax);
     RAX = rax;
 
-    if (BX_CPU_THIS_PTR get_DF ()) {
+    if (BX_CPU_THIS_PTR get_DF()) {
       esi -= 8;
     }
     else {
