@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: macintosh.cc,v 1.21.2.2 2004/02/08 18:08:31 danielg4 Exp $
+// $Id: macintosh.cc,v 1.26 2006/02/21 21:35:08 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -37,6 +37,7 @@
 // BOCHS INCLUDES
 #include <MacTypes.h>
 #include "bochs.h"
+#include "iodev.h"
 
 // decide whether to enable this file or not
 #if BX_WITH_MACOS
@@ -251,7 +252,7 @@ void CreateTile(void)
         unsigned  long p_f;
         long      theRowBytes = ((((long) ( vga_bpp==24?32:(((vga_bpp+1)>>1)<<1) ) * ((long) (srcTileRect.right-srcTileRect.left)) + 31) >> 5) << 2);
         
-      //  if (bx_options.Oprivate_colormap->get ())
+      //  if (SIM->get_param_bool(BXPN_PRIVATE_COLORMAP)->get())
       //  {
                 GetGWorld(&savePort, &saveDevice);
     switch(vga_bpp)
@@ -924,7 +925,7 @@ void bx_macintosh_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
 //                      RGBForeColor(&fgColor);
 //                      RGBBackColor(&bgColor);
                         
-                        if (bx_options.Oprivate_colormap->get ())
+                        if (SIM->get_param_bool(BXPN_PRIVATE_COLORMAP)->get())
                         {
                                 PmForeColor(new_text[i+1] & 0x0F);
                                 PmBackColor((new_text[i+1] & 0xF0) >> 4);
@@ -990,7 +991,7 @@ bx_bool bx_macintosh_gui_c::palette_change(unsigned index, unsigned red, unsigne
         CGrafPtr  savePort;
   GrafPtr   oldPort;
   
-/*  if (gOffWorld != NULL) //(bx_options.Oprivate_colormap->get ())
+/*  if (gOffWorld != NULL) //(SIM->get_param_bool(BXPN_PRIVATE_COLORMAP)->get())
   {
     GetGWorld(&savePort, &saveDevice);
 
@@ -1012,9 +1013,9 @@ bx_bool bx_macintosh_gui_c::palette_change(unsigned index, unsigned red, unsigne
         
     SetPort(oldPort);
   }
-/*  if (gOffWorld != NULL) //(bx_options.Oprivate_colormap->get ())
+/*  if (gOffWorld != NULL) //(SIM->get_param_bool(BXPN_PRIVATE_COLORMAP)->get())
     SetGWorld(savePort, saveDevice);*/
-  if (bx_options.Oprivate_colormap->get ())
+  if (SIM->get_param_bool(BXPN_PRIVATE_COLORMAP)->get())
   {
         
                 thePal = NewPalette(index, gCTable, pmTolerant, 0x5000);
@@ -1136,6 +1137,10 @@ void bx_macintosh_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheig
                 width = x;
                 height = y;
         }
+
+  host_xres = x;
+  host_yres = y;
+  host_bpp = bpp;
 }
 
 
