@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: x.cc,v 1.100 2006/01/23 18:34:47 vruppert Exp $
+// $Id: x.cc,v 1.103 2006/04/14 13:27:17 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -424,7 +424,7 @@ bx_x_gui_c::specific_init(int argc, char **argv, unsigned tilewidth, unsigned ti
   default_depth  = DefaultDepth(bx_x_display, bx_x_screen_num);
   default_visual = DefaultVisual(bx_x_display, bx_x_screen_num);
 
-  if (!bx_options.Oprivate_colormap->get ()) {
+  if (!SIM->get_param_bool(BXPN_PRIVATE_COLORMAP)->get()) {
     default_cmap = DefaultColormap(bx_x_display, bx_x_screen_num);
     // try to use default colormap.  If not enough colors are available,
     // then switch to private colormap despite the user setting.  There
@@ -432,7 +432,7 @@ bx_x_gui_c::specific_init(int argc, char **argv, unsigned tilewidth, unsigned ti
     // draws everything in black on black.
     if (!test_alloc_colors (default_cmap, 16)) {
       BX_ERROR (("I can't even allocate 16 colors!  Switching to a private colormap"));
-      bx_options.Oprivate_colormap->set (1);
+      SIM->get_param_bool(BXPN_PRIVATE_COLORMAP)->set(1);
     }
     col_vals[0]  = BlackPixel(bx_x_display, bx_x_screen_num);
     col_vals[15] = WhitePixel(bx_x_display, bx_x_screen_num);
@@ -442,7 +442,7 @@ bx_x_gui_c::specific_init(int argc, char **argv, unsigned tilewidth, unsigned ti
     }
   }
 
-  if (bx_options.Oprivate_colormap->get ()) {
+  if (SIM->get_param_bool(BXPN_PRIVATE_COLORMAP)->get()) {
     default_cmap = XCreateColormap(bx_x_display, DefaultRootWindow(bx_x_display),
                                    default_visual, AllocNone);
     if (XAllocColorCells(bx_x_display, default_cmap, False,
@@ -542,6 +542,8 @@ bx_x_gui_c::specific_init(int argc, char **argv, unsigned tilewidth, unsigned ti
   XSetWMProperties(bx_x_display, win, &windowName, &iconName,
       argv, argc, &size_hints, &wm_hints,
       &class_hints);
+  XFree(windowName.value);
+  XFree(iconName.value);
   }
 
   /* Select event types wanted */
@@ -643,9 +645,9 @@ bx_x_gui_c::specific_init(int argc, char **argv, unsigned tilewidth, unsigned ti
   SIM->set_notify_callback (x11_notify_callback, NULL);
 
   // loads keymap for x11
-  if(bx_options.keyboard.OuseMapping->get()) {
+  if (SIM->get_param_bool(BXPN_KBD_USEMAPPING)->get()) {
     bx_keymap.loadKeymap(convertStringToXKeysym);
-    }
+  }
 
   new_gfx_api = 1;
   dialog_caps |= BX_GUI_DLG_USER;
@@ -1002,7 +1004,7 @@ xkeypress(KeySym keysym, int press_release)
   }
 
   /* Old (no mapping) behavior */
-  if(!bx_options.keyboard.OuseMapping->get()){
+  if (!SIM->get_param_bool(BXPN_KBD_USEMAPPING)->get()) {
 
     // this depends on the fact that the X11 keysyms which
     // correspond to the ascii characters space .. tilde
@@ -1609,7 +1611,7 @@ bx_x_gui_c::palette_change(unsigned index, unsigned red, unsigned green, unsigne
   color.green = green << 8;
   color.blue  = blue << 8;
 
-  if (bx_options.Oprivate_colormap->get ()) {
+  if (SIM->get_param_bool(BXPN_PRIVATE_COLORMAP)->get()) {
     color.pixel = index;
     XStoreColor(bx_x_display, default_cmap, &color);
     return(0); // no screen update needed
@@ -1798,7 +1800,7 @@ bx_x_gui_c::exit(void)
 
 static void warp_cursor (int dx, int dy)
 {
-      if (bx_options.Omouse_enabled->get () &&
+      if (SIM->get_param_bool(BXPN_MOUSE_ENABLED)->get() &&
 	  (warp_dx || warp_dy || dx || dy)
 	  ) {
 	    warp_dx = dx;
