@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: instrument.cc,v 1.16 2006/01/17 18:17:01 sshwarts Exp $
+// $Id: instrument.cc,v 1.18 2006/06/17 12:09:55 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -28,6 +28,7 @@
 #include <assert.h>
 
 #include "bochs.h"
+#include "cpu/cpu.h"
 
 // maximum size of an instruction
 #define MAX_OPCODE_SIZE 16
@@ -214,8 +215,7 @@ void bx_instr_hwinterrupt(unsigned cpu, unsigned vector, Bit16u cs, bx_address e
 void bx_instr_mem_data(unsigned cpu, bx_address lin, unsigned size, unsigned rw)
 {
   unsigned index;
-  Bit32u phy;
-  bx_bool page_valid;
+  bx_phy_address phy;
 
   if(!active || !instruction[cpu].valid) return;
 
@@ -224,7 +224,7 @@ void bx_instr_mem_data(unsigned cpu, bx_address lin, unsigned size, unsigned rw)
     return;
   }
 
-  BX_CPU(cpu)->dbg_xlate_linear2phy(lin, &phy, &page_valid);
+  bx_bool page_valid = BX_CPU(cpu)->dbg_xlate_linear2phy(lin, &phy);
   phy = A20ADDR(phy);
 
   // If linear translation doesn't exist, a paging exception will occur.
