@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: eth_fbsd.cc,v 1.26 2002/11/20 19:06:22 bdenney Exp $
+// $Id: eth_fbsd.cc,v 1.32 2005/12/10 18:37:35 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -53,8 +53,12 @@
 // is used to know when we are exporting symbols and when we are importing.
 #define BX_PLUGGABLE
  
-#include "bochs.h"
-#if BX_NE2K_SUPPORT && defined(ETH_FBSD)
+#define NO_DEVICE_INCLUDES
+#include "iodev.h"
+
+#if BX_NETWORKING && defined(ETH_FBSD)
+
+#include "eth.h"
 
 #define LOG_THIS bx_devices.pluginNE2kDevice->
 
@@ -69,8 +73,6 @@ extern "C" {
 
 #define BX_BPF_POLL  1000    // Poll for a frame every 250 usecs
 
-#define BX_BPF_BUFSIZ 2048   // enough for an ether frame + bpf hdr
-
 #define BX_BPF_INSNSIZ  8    // number of bpf insns
 
 // template filter for a unicast mac address and all 
@@ -102,7 +104,7 @@ public:
   bx_fbsd_pktmover_c(const char *netif, 
 		     const char *macaddr,
 		     eth_rx_handler_t rxh,
-		     void *rxarg);
+		     void *rxarg, char *script);
   void sendpkt(void *buf, unsigned io_len);
 
 private:
@@ -127,8 +129,8 @@ protected:
   eth_pktmover_c *allocate(const char *netif, 
 			   const char *macaddr,
 			   eth_rx_handler_t rxh,
-			   void *rxarg) {
-    return (new bx_fbsd_pktmover_c(netif, macaddr, rxh, rxarg));
+			   void *rxarg, char *script) {
+    return (new bx_fbsd_pktmover_c(netif, macaddr, rxh, rxarg, script));
   }
 } bx_fbsd_match;
 
@@ -145,7 +147,8 @@ protected:
 bx_fbsd_pktmover_c::bx_fbsd_pktmover_c(const char *netif, 
 				       const char *macaddr,
 				       eth_rx_handler_t rxh,
-				       void *rxarg)
+				       void *rxarg,
+				       char *script)
 {
   char device[sizeof "/dev/bpf000"];
   int tmpfd;
@@ -184,7 +187,7 @@ bx_fbsd_pktmover_c::bx_fbsd_pktmover_c(const char *netif,
   }
 
   // Set buffer size
-  v = BX_BPF_BUFSIZ;
+  v = BX_PACKET_BUFSIZE;
   if (ioctl(this->bpf_fd, BIOCSBLEN, (caddr_t)&v) < 0) {
     BX_PANIC(("eth_freebsd: could not set buffer size: %s", strerror(errno)));
     close(this->bpf_fd);
@@ -325,7 +328,7 @@ void
 bx_fbsd_pktmover_c::rx_timer(void)
 {
   int nbytes = 0;
-    unsigned char rxbuf[BX_BPF_BUFSIZ]; 
+    unsigned char rxbuf[BX_PACKET_BUFSIZE]; 
   struct bpf_hdr *bhdr;
     struct bpf_stat bstat;
     static struct bpf_stat previous_bstat;
@@ -381,5 +384,5 @@ bx_fbsd_pktmover_c::rx_timer(void)
   }  
 }
 
-#endif /* if BX_NE2K_SUPPORT && defined(ETH_FBSD) */
+#endif /* if BX_NETWORKING && defined(ETH_FBSD) */
 
