@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: parallel.cc,v 1.26 2004/06/19 15:20:13 sshwarts Exp $
+// $Id: parallel.cc,v 1.30 2006/05/29 22:33:38 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -39,8 +39,7 @@
 
 bx_parallel_c *theParallelDevice = NULL;
 
-  int
-libparallel_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
+int libparallel_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
 {
   theParallelDevice = new bx_parallel_c ();
   bx_devices.pluginParallelDevice = theParallelDevice;
@@ -48,12 +47,11 @@ libparallel_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char
   return(0); // Success
 }
 
-  void
-libparallel_LTX_plugin_fini(void)
+void libparallel_LTX_plugin_fini(void)
 {
 }
 
-bx_parallel_c::bx_parallel_c(void)
+bx_parallel_c::bx_parallel_c()
 {
   put("PAR");
   settype(PARLOG);
@@ -62,7 +60,7 @@ bx_parallel_c::bx_parallel_c(void)
   }
 }
 
-bx_parallel_c::~bx_parallel_c(void)
+bx_parallel_c::~bx_parallel_c()
 {
   for (int i=0; i<BX_PARPORT_MAXDEV; i++) {
     if (s[i].output != NULL)
@@ -70,17 +68,19 @@ bx_parallel_c::~bx_parallel_c(void)
   }
 }
 
-  void
-bx_parallel_c::init(void)
+void bx_parallel_c::init(void)
 {
   Bit16u ports[BX_PARPORT_MAXDEV] = {0x0378, 0x0278};
   Bit8u irqs[BX_PARPORT_MAXDEV] = {7, 5};
-  char name[16];
+  char name[16], pname[20];
+  bx_list_c *base;
 
-  BX_DEBUG(("Init $Id: parallel.cc,v 1.26 2004/06/19 15:20:13 sshwarts Exp $"));
+  BX_DEBUG(("Init $Id: parallel.cc,v 1.30 2006/05/29 22:33:38 sshwarts Exp $"));
 
   for (unsigned i=0; i<BX_N_PARALLEL_PORTS; i++) {
-    if (bx_options.par[i].Oenabled->get ()) {
+    sprintf(pname, "ports.parallel.%d", i+1);
+    base = (bx_list_c*) SIM->get_param(pname);
+    if (SIM->get_param_bool("enabled", base)->get()) {
       sprintf(name, "Parallel Port %d", i + 1);
       /* parallel interrupt and i/o ports */
       BX_PAR_THIS s[i].IRQ = irqs[i];
@@ -106,55 +106,80 @@ bx_parallel_c::init(void)
 
       BX_PAR_THIS s[i].initmode = 0;
       /* output file */
-      if (strlen(bx_options.par[i].Ooutfile->getptr ()) > 0) {
-        s[i].output = fopen(bx_options.par[i].Ooutfile->getptr (), "wb");
+      char *outfile = SIM->get_param_string("outfile", base)->getptr();
+      if (strlen(outfile) > 0) {
+        s[i].output = fopen(outfile, "wb");
         if (!s[i].output)
-          BX_PANIC (("Could not open '%s' to write parport%d output",
-                     bx_options.par[i].Ooutfile->getptr (), i+1));
+          BX_PANIC(("Could not open '%s' to write parport%d output",
+                    outfile, i+1));
       }
     }
   }
 }
 
-  void
-bx_parallel_c::reset(unsigned type)
+void bx_parallel_c::reset(unsigned type)
 {
 }
 
-  void
-bx_parallel_c::virtual_printer(Bit8u port)
+#if BX_SUPPORT_SAVE_RESTORE
+void bx_parallel_c::register_state(void)
+{
+  unsigned i;
+  char name[4], pname[20];
+  bx_list_c *base, *port;
+
+  bx_list_c *list = new bx_list_c(SIM->get_sr_root(), "parallel", "Parallel Port State");
+  for (i=0; i<BX_N_PARALLEL_PORTS; i++) {
+    sprintf(pname, "ports.parallel.%d", i+1);
+    base = (bx_list_c*) SIM->get_param(pname);
+    if (SIM->get_param_bool("enabled", base)->get()) {
+      sprintf(name, "%d", i);
+      port = new bx_list_c(list, name, 11);
+      new bx_shadow_num_c(port, "data", &BX_PAR_THIS s[i].data, BASE_HEX);
+      new bx_shadow_bool_c(port, "slct", &BX_PAR_THIS s[i].STATUS.slct);
+      new bx_shadow_bool_c(port, "ack", &BX_PAR_THIS s[i].STATUS.ack);
+      new bx_shadow_bool_c(port, "busy", &BX_PAR_THIS s[i].STATUS.busy);
+      new bx_shadow_bool_c(port, "strobe", &BX_PAR_THIS s[i].CONTROL.strobe);
+      new bx_shadow_bool_c(port, "autofeed", &BX_PAR_THIS s[i].CONTROL.autofeed);
+      new bx_shadow_bool_c(port, "init", &BX_PAR_THIS s[i].CONTROL.init);
+      new bx_shadow_bool_c(port, "slct_in", &BX_PAR_THIS s[i].CONTROL.slct_in);
+      new bx_shadow_bool_c(port, "irq", &BX_PAR_THIS s[i].CONTROL.irq);
+      new bx_shadow_bool_c(port, "input", &BX_PAR_THIS s[i].CONTROL.input);
+      new bx_shadow_bool_c(port, "initmode", &BX_PAR_THIS s[i].initmode);
+    }
+  }
+}
+#endif
+
+void bx_parallel_c::virtual_printer(Bit8u port)
 {
   if (BX_PAR_THIS s[port].STATUS.slct) {
     if (BX_PAR_THIS s[port].output != NULL) {
       fputc(BX_PAR_THIS s[port].data, BX_PAR_THIS s[port].output);
       fflush (BX_PAR_THIS s[port].output);
-      }
+    }
     if (BX_PAR_THIS s[port].CONTROL.irq == 1) {
       DEV_pic_raise_irq(BX_PAR_THIS s[port].IRQ);
-      }
+    }
     BX_PAR_THIS s[port].STATUS.ack = 0;
     BX_PAR_THIS s[port].STATUS.busy = 1;
-    }
+  }
   else {
     BX_ERROR(("data is valid, but printer is offline"));
-    }
+  }
 }
 
-  // static IO port read callback handler
-  // redirects to non-static class handler to avoid virtual functions
+// static IO port read callback handler
+// redirects to non-static class handler to avoid virtual functions
 
-  Bit32u
-bx_parallel_c::read_handler(void *this_ptr, Bit32u address, unsigned io_len)
+Bit32u bx_parallel_c::read_handler(void *this_ptr, Bit32u address, unsigned io_len)
 {
 #if !BX_USE_PAR_SMF
   bx_parallel_c *class_ptr = (bx_parallel_c *) this_ptr;
-
-  return( class_ptr->read(address, io_len) );
+  return class_ptr->read(address, io_len);
 }
 
-
-  Bit32u
-bx_parallel_c::read(Bit32u address, unsigned io_len)
+Bit32u bx_parallel_c::read(Bit32u address, unsigned io_len)
 {
 #else
   UNUSED(this_ptr);
@@ -220,12 +245,10 @@ bx_parallel_c::read(Bit32u address, unsigned io_len)
   return(0);
 }
 
+// static IO port write callback handler
+// redirects to non-static class handler to avoid virtual functions
 
-  // static IO port write callback handler
-  // redirects to non-static class handler to avoid virtual functions
-
-  void
-bx_parallel_c::write_handler(void *this_ptr, Bit32u address, Bit32u value, unsigned io_len)
+void bx_parallel_c::write_handler(void *this_ptr, Bit32u address, Bit32u value, unsigned io_len)
 {
 #if !BX_USE_PAR_SMF
   bx_parallel_c *class_ptr = (bx_parallel_c *) this_ptr;
@@ -233,8 +256,7 @@ bx_parallel_c::write_handler(void *this_ptr, Bit32u address, Bit32u value, unsig
   class_ptr->write(address, value, io_len);
 }
 
-  void
-bx_parallel_c::write(Bit32u address, Bit32u value, unsigned io_len)
+void bx_parallel_c::write(Bit32u address, Bit32u value, unsigned io_len)
 {
 #else
   UNUSED(this_ptr);
