@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pci_ide.h,v 1.7 2005/10/29 12:35:01 vruppert Exp $
+// $Id: pci_ide.h,v 1.11 2006/05/27 15:54:48 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2004  MandrakeSoft S.A.
@@ -24,6 +24,8 @@
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
+#ifndef BX_IODEV_PCIIDE_H
+#define BX_IODEV_PCIIDE_H
 
 #if BX_USE_PIDE_SMF
 #  define BX_PIDE_SMF  static
@@ -35,16 +37,27 @@
 #  define BX_PIDE_THIS_PTR this
 #endif
 
-
 class bx_pci_ide_c : public bx_pci_ide_stub_c {
-
 public:
-  bx_pci_ide_c(void);
-  ~bx_pci_ide_c(void);
-  virtual void   init(void);
-  virtual void   reset(unsigned type);
+  bx_pci_ide_c();
+  virtual ~bx_pci_ide_c();
+  virtual void init(void);
+  virtual void reset(unsigned type);
   virtual bx_bool bmdma_present(void);
-  virtual void   bmdma_set_irq(Bit8u channel);
+  virtual void bmdma_set_irq(Bit8u channel);
+#if BX_SUPPORT_SAVE_RESTORE
+  virtual void register_state(void);
+  virtual void after_restore_state(void);
+  static  Bit64s param_save_handler(void *devptr, bx_param_c *param, Bit64s val);
+  static  Bit64s param_restore_handler(void *devptr, bx_param_c *param, Bit64s val);
+#if !BX_USE_PIDE_SMF
+  Bit64s param_save(bx_param_c *param, Bit64s val);
+  Bit64s param_restore(bx_param_c *param, Bit64s val);
+#endif
+#endif
+
+  virtual Bit32u pci_read_handler(Bit8u address, unsigned io_len);
+  virtual void   pci_write_handler(Bit8u address, Bit32u value, unsigned io_len);
 
   static void timer_handler(void *);
   BX_PIDE_SMF void timer(void);
@@ -69,12 +82,10 @@ private:
 
   static Bit32u read_handler(void *this_ptr, Bit32u address, unsigned io_len);
   static void   write_handler(void *this_ptr, Bit32u address, Bit32u value, unsigned io_len);
-  static Bit32u pci_read_handler(void *this_ptr, Bit8u address, unsigned io_len);
-  static void   pci_write_handler(void *this_ptr, Bit8u address, Bit32u value, unsigned io_len);
 #if !BX_USE_PIDE_SMF
   Bit32u read(Bit32u address, unsigned io_len);
   void   write(Bit32u address, Bit32u value, unsigned io_len);
-  Bit32u pci_read(Bit8u address, unsigned io_len);
-  void   pci_write(Bit8u address, Bit32u value, unsigned io_len);
 #endif
-  };
+};
+
+#endif
