@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pcipnic.cc,v 1.15 2005/11/15 17:19:28 vruppert Exp $
+// $Id: pcipnic.cc,v 1.21 2006/05/29 22:33:38 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2003  Fen Systems Ltd.
@@ -22,6 +22,7 @@
 // Define BX_PLUGGABLE in files that can be compiled into plugins.  For
 // platforms that require a special tag on exported symbols, BX_PLUGGABLE 
 // is used to know when we are exporting symbols and when we are importing.
+
 #define BX_PLUGGABLE
 
 #include "iodev.h"
@@ -44,40 +45,29 @@ libpcipnic_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *
   return 0; // Success
 }
 
-  void
-libpcipnic_LTX_plugin_fini(void)
-{
-}
+void libpcipnic_LTX_plugin_fini(void) {}
 
-
-bx_pcipnic_c::bx_pcipnic_c(void)
+bx_pcipnic_c::bx_pcipnic_c()
 {
   put("PNIC");
   settype(PCIPNICLOG);
 }
 
-bx_pcipnic_c::~bx_pcipnic_c(void)
+bx_pcipnic_c::~bx_pcipnic_c()
 {
   // nothing for now
-  BX_DEBUG(("Exit."));
 }
 
-
-  void
-bx_pcipnic_c::init(void)
+void bx_pcipnic_c::init(void)
 {
-  // called once when bochs initializes
+  bx_list_c *base;
 
-  if (!bx_options.pnic.Oenabled->get()) return;
-
-  memcpy ( BX_PNIC_THIS s.macaddr, bx_options.pnic.Omacaddr->getptr(),
-	   sizeof( BX_PNIC_THIS s.macaddr ) );
+  // Read in values from config interface
+  base = (bx_list_c*) SIM->get_param(BXPN_PNIC);
+  memcpy(BX_PNIC_THIS s.macaddr, SIM->get_param_string("macaddr", base)->getptr(), 6);
 
   BX_PNIC_THIS s.devfunc = 0x00;
-  DEV_register_pci_handlers(this,
-                            pci_read_handler,
-                            pci_write_handler,
-                            &BX_PNIC_THIS s.devfunc, BX_PLUGIN_PCIPNIC,
+  DEV_register_pci_handlers(this, &BX_PNIC_THIS s.devfunc, BX_PLUGIN_PCIPNIC,
                             "Experimental PCI Pseudo NIC");
 
   for (unsigned i=0; i<256; i++) {
@@ -86,13 +76,13 @@ bx_pcipnic_c::init(void)
 
   // This code ripped wholesale from ne2k.cc:
   // Attach to the simulated ethernet dev
-  char *ethmod = bx_options.pnic.Oethmod->get_choice(bx_options.pnic.Oethmod->get());
+  char *ethmod = SIM->get_param_enum("ethmod", base)->get_selected();
   BX_PNIC_THIS ethdev = eth_locator_c::create(ethmod,
-                                              bx_options.pnic.Oethdev->getptr (),
-                                              (const char *) bx_options.pnic.Omacaddr->getptr (),
+                                              SIM->get_param_string("ethdev", base)->getptr(),
+                                              (const char *) SIM->get_param_string("macaddr", base)->getptr(),
                                               rx_handler, 
                                               this,
-                                              bx_options.pnic.Oscript->getptr ());
+                                              SIM->get_param_string("script", base)->getptr());
 
   if (BX_PNIC_THIS ethdev == NULL) {
     BX_PANIC(("could not find eth module %s", ethmod));
@@ -100,7 +90,7 @@ bx_pcipnic_c::init(void)
     BX_INFO(("could not find eth module %s - using null instead", ethmod));
 
     BX_PNIC_THIS ethdev = eth_locator_c::create("null", NULL,
-                                                (const char *) bx_options.pnic.Omacaddr->getptr (),
+                                                (const char *) SIM->get_param_string("macaddr", base)->getptr(),
                                                 rx_handler, 
                                                 this, "");
     if (BX_PNIC_THIS ethdev == NULL)
@@ -112,8 +102,7 @@ bx_pcipnic_c::init(void)
   BX_INFO(("PCI Pseudo NIC initialized - I/O base and IRQ assigned by PCI BIOS"));
 }
 
-void
-bx_pcipnic_c::reset(unsigned type)
+void bx_pcipnic_c::reset(unsigned type)
 {
   unsigned i;
 
@@ -159,28 +148,61 @@ bx_pcipnic_c::reset(unsigned type)
   set_irq_level(0);
 }
 
-  void
-bx_pcipnic_c::set_irq_level(bx_bool level)
+#if BX_SUPPORT_SAVE_RESTORE
+void bx_pcipnic_c::register_state(void)
+{
+  unsigned i;
+  char name[6];
+
+  bx_list_c *list = new bx_list_c(SIM->get_sr_root(), "pcipnic", "PCI Pseudo NIC State", 11);
+  new bx_shadow_num_c(list, "irqEnabled", &BX_PNIC_THIS s.irqEnabled);
+  new bx_shadow_num_c(list, "rCmd", &BX_PNIC_THIS s.rCmd);
+  new bx_shadow_num_c(list, "rStatus", &BX_PNIC_THIS s.rStatus);
+  new bx_shadow_num_c(list, "rLength", &BX_PNIC_THIS s.rLength);
+  new bx_shadow_num_c(list, "rDataCursor", &BX_PNIC_THIS s.rDataCursor);
+  new bx_shadow_num_c(list, "recvIndex", &BX_PNIC_THIS s.recvIndex);
+  new bx_shadow_num_c(list, "recvQueueLength", &BX_PNIC_THIS s.recvQueueLength);
+  bx_list_c *recvRL = new bx_list_c(list, "recvRingLength", PNIC_RECV_RINGS);
+  for (i=0; i<PNIC_RECV_RINGS; i++) {
+    sprintf(name, "%d", i);
+    new bx_shadow_num_c(recvRL, name, &BX_PNIC_THIS s.recvRingLength[i]);
+  }
+  new bx_shadow_data_c(list, "rData", BX_PNIC_THIS s.rData, PNIC_DATA_SIZE);
+  new bx_shadow_data_c(list, "recvRing", (Bit8u*)BX_PNIC_THIS s.recvRing, PNIC_RECV_RINGS*PNIC_DATA_SIZE);
+  bx_list_c *pci_conf = new bx_list_c(list, "pci_conf", 256);
+  for (i=0; i<256; i++) {
+    sprintf(name, "0x%02x", i);
+    new bx_shadow_num_c(pci_conf, name, &BX_PNIC_THIS s.pci_conf[i], BASE_HEX);
+  }
+}
+
+void bx_pcipnic_c::after_restore_state(void)
+{
+  if (DEV_pci_set_base_io(BX_PNIC_THIS_PTR, read_handler, write_handler,
+                          &BX_PNIC_THIS s.base_ioaddr,
+                          &BX_PNIC_THIS s.pci_conf[0x10],
+                          16, &pnic_iomask[0], "PNIC")) {
+    BX_INFO(("new base address: 0x%04x", BX_PNIC_THIS s.base_ioaddr));
+  }
+}
+#endif
+
+void bx_pcipnic_c::set_irq_level(bx_bool level)
 {
   DEV_pci_set_irq(BX_PNIC_THIS s.devfunc, BX_PNIC_THIS s.pci_conf[0x3d], level);
 }
 
+// static IO port read callback handler
+// redirects to non-static class handler to avoid virtual functions
 
-  // static IO port read callback handler
-  // redirects to non-static class handler to avoid virtual functions
-
-  Bit32u
-bx_pcipnic_c::read_handler(void *this_ptr, Bit32u address, unsigned io_len)
+Bit32u bx_pcipnic_c::read_handler(void *this_ptr, Bit32u address, unsigned io_len)
 {
 #if !BX_USE_PCIPNIC_SMF
   bx_pcipnic_c *class_ptr = (bx_pcipnic_c *) this_ptr;
-
-  return( class_ptr->read(address, io_len) );
+  return class_ptr->read(address, io_len);
 }
 
-
-  Bit32u
-bx_pcipnic_c::read(Bit32u address, unsigned io_len)
+Bit32u bx_pcipnic_c::read(Bit32u address, unsigned io_len)
 {
 #else
   UNUSED(this_ptr);
@@ -219,12 +241,10 @@ bx_pcipnic_c::read(Bit32u address, unsigned io_len)
   return(val);
 }
 
+// static IO port write callback handler
+// redirects to non-static class handler to avoid virtual functions
 
-  // static IO port write callback handler
-  // redirects to non-static class handler to avoid virtual functions
-
-  void
-bx_pcipnic_c::write_handler(void *this_ptr, Bit32u address, Bit32u value, unsigned io_len)
+void bx_pcipnic_c::write_handler(void *this_ptr, Bit32u address, Bit32u value, unsigned io_len)
 {
 #if !BX_USE_PCIPNIC_SMF
   bx_pcipnic_c *class_ptr = (bx_pcipnic_c *) this_ptr;
@@ -232,8 +252,7 @@ bx_pcipnic_c::write_handler(void *this_ptr, Bit32u address, Bit32u value, unsign
   class_ptr->write(address, value, io_len);
 }
 
-  void
-bx_pcipnic_c::write(Bit32u address, Bit32u value, unsigned io_len)
+void bx_pcipnic_c::write(Bit32u address, Bit32u value, unsigned io_len)
 {
 #else
   UNUSED(this_ptr);
@@ -284,26 +303,9 @@ void bx_pcipnic_c::pnic_timer(void)
 
 }
   
-// static pci configuration space read callback handler
-// redirects to non-static class handler to avoid virtual functions
-
-Bit32u
-bx_pcipnic_c::pci_read_handler(void *this_ptr, Bit8u address, unsigned io_len)
+// pci configuration space read callback handler
+Bit32u bx_pcipnic_c::pci_read_handler(Bit8u address, unsigned io_len)
 {
-#if !BX_USE_PCIPNIC_SMF
-  bx_pcipnic_c *class_ptr = (bx_pcipnic_c *) this_ptr;
-  
-  return class_ptr->pci_read(address, io_len);
-}
-
-
-Bit32u
-bx_pcipnic_c::pci_read(Bit8u address, unsigned io_len)
-{
-#else
-  UNUSED(this_ptr);
-#endif // !BX_USE_PCIPNIC_SMF
-
   Bit32u value = 0;
 
   if (io_len > 4 || io_len == 0) {
@@ -359,25 +361,9 @@ bx_pcipnic_c::pci_read(Bit8u address, unsigned io_len)
 }
 
 
-// static pci configuration space write callback handler
-// redirects to non-static class handler to avoid virtual functions
-
-void
-bx_pcipnic_c::pci_write_handler(void *this_ptr, Bit8u address, Bit32u value, unsigned io_len)
+// pci configuration space write callback handler
+void bx_pcipnic_c::pci_write_handler(Bit8u address, Bit32u value, unsigned io_len)
 {
-#if !BX_USE_PCIPNIC_SMF
-  bx_pcipnic_c *class_ptr = (bx_pcipnic_c *) this_ptr;
-  
-  class_ptr->pci_write(address, value, io_len);
-}
-
-void
-bx_pcipnic_c::pci_write(Bit8u address, Bit32u value, unsigned io_len)
-{
-#else
-  UNUSED(this_ptr);
-#endif // !BX_USE_PCIPNIC_SMF
-  
   Bit8u value8, oldval;
   bx_bool baseaddr_change = 0;
 
@@ -436,8 +422,7 @@ bx_pcipnic_c::pci_write(Bit8u address, Bit32u value, unsigned io_len)
 /*
  * Execute a hardware command.
  */
-void
-bx_pcipnic_c::exec_command(void)
+void bx_pcipnic_c::exec_command(void)
 {
   Bit16u command = BX_PNIC_THIS s.rCmd;
   Bit16u ilength = BX_PNIC_THIS s.rLength;
@@ -451,7 +436,6 @@ bx_pcipnic_c::exec_command(void)
 	      ilength, BX_PNIC_THIS s.rDataCursor ));
   
   switch ( command ) {
-
   case PNIC_CMD_NOOP :
     status = PNIC_STATUS_OK;
     break;
@@ -478,7 +462,10 @@ bx_pcipnic_c::exec_command(void)
     break;
 
   case PNIC_CMD_XMIT :
-    BX_PNIC_THIS ethdev->sendpkt ( data, ilength );
+    BX_PNIC_THIS ethdev->sendpkt(data, ilength);
+    if (BX_PNIC_THIS s.irqEnabled) {
+      set_irq_level(1);
+    }
     status = PNIC_STATUS_OK;
     break;
 
@@ -540,12 +527,10 @@ bx_pcipnic_c::exec_command(void)
 /*
  * Callback from the eth system driver when a frame has arrived
  */
-void
-bx_pcipnic_c::rx_handler(void *arg, const void *buf, unsigned len)
+void bx_pcipnic_c::rx_handler(void *arg, const void *buf, unsigned len)
 {
     // BX_DEBUG(("rx_handler with length %d", len));
   bx_pcipnic_c *class_ptr = (bx_pcipnic_c *) arg;
-  
   class_ptr->rx_frame(buf, len);
 }
 
@@ -556,8 +541,7 @@ bx_pcipnic_c::rx_handler(void *arg, const void *buf, unsigned len)
  * rx ring has enough room, it is copied into it and
  * the receive process is updated
  */
-void
-bx_pcipnic_c::rx_frame(const void *buf, unsigned io_len)
+void bx_pcipnic_c::rx_frame(const void *buf, unsigned io_len)
 {
   // Check packet length
   if ( io_len > PNIC_DATA_SIZE ) {
