@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pit82c54.cc,v 1.26 2006/01/08 20:39:08 vruppert Exp $
+// $Id: pit82c54.cc,v 1.28 2006/05/29 22:33:38 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 /*
@@ -178,13 +178,51 @@ void pit_82C54::init (void) {
   seen_problems=0;
 }
 
-pit_82C54::pit_82C54 (void) {
+pit_82C54::pit_82C54(void)
+{
   init();
 }
 
-void pit_82C54::reset (unsigned type) {
+void pit_82C54::reset(unsigned type)
+{
 }
 
+#if BX_SUPPORT_SAVE_RESTORE
+void pit_82C54::register_state(bx_param_c *parent)
+{
+  unsigned i;
+  char name[4];
+  bx_list_c *tim;
+
+  for (i=0; i<3; i++) {
+    sprintf(name, "%d", i);
+    tim = new bx_list_c(parent, name, 22);
+    new bx_shadow_bool_c(tim, "GATE", &counter[i].GATE);
+    new bx_shadow_bool_c(tim, "OUTpin", &counter[i].OUTpin);
+    new bx_shadow_num_c(tim, "count", &counter[i].count);
+    new bx_shadow_num_c(tim, "outlatch", &counter[i].outlatch);
+    new bx_shadow_num_c(tim, "inlatch", &counter[i].inlatch);
+    new bx_shadow_num_c(tim, "status_latch", &counter[i].status_latch);
+    new bx_shadow_num_c(tim, "rw_mode", &counter[i].rw_mode);
+    new bx_shadow_num_c(tim, "mode", &counter[i].mode);
+    new bx_shadow_bool_c(tim, "bcd_mode", &counter[i].bcd_mode);
+    new bx_shadow_bool_c(tim, "null_count", &counter[i].null_count);
+    new bx_shadow_bool_c(tim, "count_LSB_latched", &counter[i].count_LSB_latched);
+    new bx_shadow_bool_c(tim, "count_MSB_latched", &counter[i].count_MSB_latched);
+    new bx_shadow_bool_c(tim, "status_latched", &counter[i].status_latched);
+    new bx_shadow_num_c(tim, "count_binary", &counter[i].count_binary);
+    new bx_shadow_bool_c(tim, "triggerGATE", &counter[i].triggerGATE);
+    new bx_shadow_num_c(tim, "write_state", (Bit8u*)&counter[i].write_state);
+    new bx_shadow_num_c(tim, "read_state", (Bit8u*)&counter[i].read_state);
+    new bx_shadow_bool_c(tim, "count_written", &counter[i].count_written);
+    new bx_shadow_bool_c(tim, "first_pass", &counter[i].first_pass);
+    new bx_shadow_bool_c(tim, "state_bit_1", &counter[i].state_bit_1);
+    new bx_shadow_bool_c(tim, "state_bit_2", &counter[i].state_bit_2);
+    new bx_shadow_num_c(tim, "next_change_time", &counter[i].next_change_time);
+  }
+}
+#endif
+
 void  BX_CPP_AttrRegparmN(2)
 pit_82C54::decrement_multiple(counter_type & thisctr, Bit32u cycles) {
   while(cycles>0) {
