@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: load32bitOShack.cc,v 1.18 2006/01/25 22:19:57 sshwarts Exp $
+// $Id: load32bitOShack.cc,v 1.21 2006/03/27 18:02:06 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -25,64 +25,57 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
 
-
-
 #include "bochs.h"
+#include "cpu/cpu.h"
 #include "iodev/iodev.h"
 #define LOG_THIS genlog->
 
 
-
 static void bx_load_linux_hack(void);
 static void bx_load_null_kernel_hack(void);
 static Bit32u bx_load_kernel_image(char *path, Bit32u paddr);
 
-  void
-bx_load32bitOSimagehack(void)
+void bx_load32bitOSimagehack(void)
 {
-  if ( bx_options.load32bitOSImage.Oiolog 
-       && (bx_options.load32bitOSImage.Oiolog->getptr ()[0] != '\0')
-      ) {
+  if (SIM->get_param_string(BXPN_LOAD32BITOS_IOLOG) &&
+     (SIM->get_param_string(BXPN_LOAD32BITOS_IOLOG)->getptr()[0] != '\0'))
+  {
 
-  // Replay IO from log to initialize IO devices to
-  // a reasonable state needed for the OS.  This is done
-  // in lieu of running the 16-bit BIOS to init things,
-  // since we want to test straight 32bit stuff for
-  // freemware.
+    // Replay IO from log to initialize IO devices to
+    // a reasonable state needed for the OS.  This is done
+    // in lieu of running the 16-bit BIOS to init things,
+    // since we want to test straight 32bit stuff for
+    // freemware.
 
-  FILE *fp;
-
-  fp = fopen(bx_options.load32bitOSImage.Oiolog->getptr (), "r");
-
-  if (fp == NULL) {
-    BX_PANIC(("could not open IO init file."));
-    }
-
-  while (1) {
-    unsigned len, op, port, val;
-    int ret;
-    ret = fscanf(fp, "%u %u %x %x\n",
-      &len, &op, &port, &val);
-    if (ret != 4) {
+    FILE *fp = fopen(SIM->get_param_string(BXPN_LOAD32BITOS_IOLOG)->getptr(), "r");
+    if (fp == NULL) {
       BX_PANIC(("could not open IO init file."));
-      }
-    if (op == 0) {
-      // read
-      (void) bx_devices.inp(port, len);
-      }
-    else if (op == 1) {
-      // write
-      bx_devices.outp(port, val, len);
-      }
-    else {
-      BX_PANIC(("bad IO op in init filen"));
-      }
-    if (feof(fp)) break;
     }
-  }//if iolog file to load
+
+    while (1) {
+      unsigned len, op, port, val;
+      int ret;
+      ret = fscanf(fp, "%u %u %x %x\n", &len, &op, &port, &val);
+      if (ret != 4) {
+        BX_PANIC(("could not open IO init file."));
+      }
+      if (op == 0) {
+        // read
+        bx_devices.inp(port, len);
+      }
+      else if (op == 1) {
+        // write
+        bx_devices.outp(port, val, len);
+      }
+      else {
+        BX_PANIC(("bad IO op in init filen"));
+      }
+      if (feof(fp)) break;
+    }
+  } //if iolog file to load
 
   // Invoke proper hack depending on which OS image we're loading
-  switch (bx_options.load32bitOSImage.OwhichOS->get ()) {
+  switch (SIM->get_param_enum(BXPN_LOAD32BITOS_WHICH)->get()) {
     case Load32bitOSLinux:
       bx_load_linux_hack();
       break;
@@ -99,6 +92,7 @@ struct gdt_entry
   Bit32u low;
   Bit32u high;
 };
+
 struct linux_setup_params
 {
    /* 0x000 */ Bit8u   orig_x;
@@ -138,8 +132,7 @@ struct linux_setup_params
    /* 0x800 */ Bit8u   commandline[2048];
 };
 
-  static void
-bx_load_linux_setup_params( Bit32u initrd_start, Bit32u initrd_size )
+static void bx_load_linux_setup_params(Bit32u initrd_start, Bit32u initrd_size)
 {
   BX_MEM_C *mem = BX_MEM(0);
   struct linux_setup_params *params =
@@ -176,8 +169,7 @@ bx_load_linux_setup_params( Bit32u initrd_start, Bit32u initrd_size )
   params->gdt[3].low  = 0x0000ffff;
 }
 
-  void
-bx_load_linux_hack(void)
+void bx_load_linux_hack(void)
 {
   Bit32u initrd_start = 0, initrd_size = 0;
 
@@ -185,10 +177,10 @@ bx_load_linux_hack(void)
   // Set CPU and memory features which are assumed at this point.
 
   // Load Linux kernel image
-  bx_load_kernel_image( bx_options.load32bitOSImage.Opath->getptr (), 0x100000 );
+  bx_load_kernel_image(SIM->get_param_string(BXPN_LOAD32BITOS_PATH)->getptr(), 0x100000);
 
   // Load initial ramdisk image if requested
-  char * tmpPtr = bx_options.load32bitOSImage.Oinitrd->getptr ();
+  char * tmpPtr = SIM->get_param_string(BXPN_LOAD32BITOS_INITRD)->getptr();
   if ( tmpPtr && tmpPtr[0] ) /* The initial value is "" and not NULL */
   {
     initrd_start = 0x00800000;  /* FIXME: load at top of memory */
@@ -235,13 +227,12 @@ bx_load_linux_hack(void)
   BX_CPU(0)->jump_protected( NULL, 0x10, 0x00100000 );
 }
 
-  void
-bx_load_null_kernel_hack(void)
+void bx_load_null_kernel_hack(void)
 {
   // The RESET function will have been called first.
   // Set CPU and memory features which are assumed at this point.
 
-  bx_load_kernel_image(bx_options.load32bitOSImage.Opath->getptr (), 0x100000);
+  bx_load_kernel_image(SIM->get_param_string(BXPN_LOAD32BITOS_PATH)->getptr(), 0x100000);
 
   // EIP deltas
   BX_CPU(0)->prev_eip =
@@ -254,6 +245,10 @@ bx_load_null_kernel_hack(void)
   BX_CPU(0)->sregs[BX_SEG_REG_CS].cache.u.segment.g   = 1; // page gran
   BX_CPU(0)->sregs[BX_SEG_REG_CS].cache.u.segment.d_b = 1; // 32bit
 
+#if BX_SUPPORT_ICACHE
+  BX_CPU(0)->updateFetchModeMask();
+#endif
+
   // DS deltas
   BX_CPU(0)->sregs[BX_SEG_REG_DS].cache.u.segment.base = 0x00000000;
   BX_CPU(0)->sregs[BX_SEG_REG_DS].cache.u.segment.limit = 0xFFFFF;
@@ -282,12 +277,12 @@ bx_load_kernel_image(char *path, Bit32u paddr)
   if (fd < 0) {
     BX_INFO(( "load_kernel_image: couldn't open image file '%s'.", path ));
     BX_EXIT(1);
-    }
+  }
   ret = fstat(fd, &stat_buf);
   if (ret) {
     BX_INFO(( "load_kernel_image: couldn't stat image file '%s'.", path ));
     BX_EXIT(1);
-    }
+  }
 
   size = (unsigned long)stat_buf.st_size;
   page_size = ((Bit32u)size + 0xfff) & ~0xfff;
@@ -296,7 +291,7 @@ bx_load_kernel_image(char *path, Bit32u paddr)
   if ( (paddr + size) > mem->len ) {
     BX_INFO(( "load_kernel_image: address range > physical memsize!" ));
     BX_EXIT(1);
-    }
+  }
 
   offset = 0;
   while (size > 0) {
@@ -304,10 +299,10 @@ bx_load_kernel_image(char *path, Bit32u paddr)
     if (ret <= 0) {
       BX_INFO(( "load_kernel_image: read failed on image" ));
       BX_EXIT(1);
-      }
+    }
     size -= ret;
     offset += ret;
-    }
+  }
   close(fd);
   BX_INFO(("load_kernel_image: '%s', size=%u read into memory at %08x",
        path, (unsigned) stat_buf.st_size, (unsigned) paddr));
