@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: main.cc,v 1.307 2006/01/22 12:31:15 vruppert Exp $
+// $Id: main.cc,v 1.339 2006/07/21 18:26:53 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -25,6 +25,7 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
 #include "bochs.h"
+#include "cpu/cpu.h"
 #include "iodev/iodev.h"
 
 #ifdef HAVE_LOCALE_H
@@ -185,6 +186,67 @@ static void carbonFatalDialog(const char *error, const char *exposition)
 }
 #endif
 
+void print_tree(bx_param_c *node, int level)
+{
+  int i;
+  char tmpstr[BX_PATHNAME_LEN], tmpbyte[4];
+
+  for (i=0; i<level; i++)
+    printf ("  ");
+  if (node == NULL) {
+      printf("NULL pointer\n");
+      return;
+  }
+  switch (node->get_type()) {
+    case BXT_PARAM_NUM:
+      if (((bx_param_num_c*)node)->get_base() == BASE_DEC) {
+        printf("%s = " FMT_LL "d (number)\n", node->get_name(), ((bx_param_num_c*)node)->get64());
+      } else {
+        printf("%s = 0x" FMT_LL "x (hex number)\n", node->get_name(), ((bx_param_num_c*)node)->get64());
+      }
+      break;
+    case BXT_PARAM_BOOL:
+      printf("%s = %s (boolean)\n", node->get_name(), ((bx_param_bool_c*)node)->get()?"true":"false");
+      break;
+    case BXT_PARAM_ENUM:
+      printf("%s = '%s' (enum)\n", node->get_name(), ((bx_param_enum_c*)node)->get_selected());
+      break;
+    case BXT_PARAM_STRING:
+      if (((bx_param_string_c*)node)->get_options()->get() & bx_param_string_c::RAW_BYTES) {
+        tmpstr[0] = 0;
+        for (i = 0; i < ((bx_param_string_c*)node)->get_maxsize(); i++) {
+          if (i > 0) {
+            tmpbyte[0] = ((bx_param_string_c*)node)->get_separator();
+            tmpbyte[1] = 0;
+            strcat(tmpstr, tmpbyte);
+          }
+          sprintf(tmpbyte, "%02x", (Bit8u)((bx_param_string_c*)node)->getptr()[i]);
+          strcat(tmpstr, tmpbyte);
+        }
+        printf("%s = '%s' (raw byte string)\n", node->get_name(), tmpstr);
+      } else {
+        printf("%s = '%s' (string)\n", node->get_name(), ((bx_param_string_c*)node)->getptr());
+      }
+      break;
+    case BXT_LIST:
+      {
+	printf("%s = \n", node->get_name());
+	bx_list_c *list = (bx_list_c*)node;
+	for (i=0; i < list->get_size(); i++) {
+	   print_tree(list->get(i), level+1);
+	}
+	break;
+      }
+#if BX_SUPPORT_SAVE_RESTORE
+    case BXT_PARAM_DATA:
+      printf("%s = 'size=%d' (binary data)\n", node->get_name(), ((bx_shadow_data_c*)node)->get_size());
+      break;
+#endif
+    default:
+      printf("%s (unknown parameter type)\n", node->get_name());
+  }
+}
+
 int bxmain () {
 #ifdef HAVE_LOCALE_H
   // Initialize locale (for isprint() and other functions)
@@ -199,11 +261,11 @@ int bxmain () {
       return 0;
     // read a param to decide which config interface to start.
     // If one exists, start it.  If not, just begin.
-    bx_param_enum_c *ci_param = SIM->get_param_enum (BXP_SEL_CONFIG_INTERFACE);
-    char *ci_name = ci_param->get_choice (ci_param->get());
+    bx_param_enum_c *ci_param = SIM->get_param_enum(BXPN_SEL_CONFIG_INTERFACE);
+    char *ci_name = ci_param->get_selected();
     if (!strcmp(ci_name, "textconfig")) {
 #if BX_USE_TEXTCONFIG
-      init_text_config_interface ();   // in textconfig.h
+      init_text_config_interface();   // in textconfig.h
 #else
       BX_PANIC(("configuration interface 'textconfig' not present"));
 #endif
@@ -214,27 +276,28 @@ int bxmain () {
     }
 #endif
     else {
-      BX_PANIC (("unsupported configuration interface '%s'", ci_name));
+      BX_PANIC(("unsupported configuration interface '%s'", ci_name));
     }
-    int status = SIM->configuration_interface (ci_name, CI_START);
+    ci_param->set_enabled(0);
+    int status = SIM->configuration_interface(ci_name, CI_START);
     if (status == CI_ERR_NO_TEXT_CONSOLE)
-      BX_PANIC (("Bochs needed the text console, but it was not usable"));
+      BX_PANIC(("Bochs needed the text console, but it was not usable"));
     // user quit the config interface, so just quit
   } else {
     // quit via longjmp
   }
-  SIM->set_quit_context (NULL);
+  SIM->set_quit_context(NULL);
 #if defined(WIN32)
   if (!bx_user_quit) {
     // ask user to press ENTER before exiting, so that they can read messages
     // before the console window is closed. This isn't necessary after pressing
     // the power button.
-    fprintf (stderr, "\nBochs is exiting. Press ENTER when you're ready to close this window.\n");
+    fprintf(stderr, "\nBochs is exiting. Press ENTER when you're ready to close this window.\n");
     char buf[16];
-    fgets (buf, sizeof(buf), stdin);
+    fgets(buf, sizeof(buf), stdin);
   }
 #endif
-  return SIM->get_exit_code ();
+  return SIM->get_exit_code();
 }
 
 #if defined(__WXMSW__)
@@ -336,7 +399,7 @@ int split_string_into_argv (
 // NOTE: It could probably be written so that it can safely be called for all
 // win32 builds.  Right now it appears to have absolutely no error checking:
 // for example if AllocConsole returns false we should probably return early.
-void RedirectIOToConsole ()
+void RedirectIOToConsole()
 {
   int hConHandle;
   long lStdHandle;
@@ -346,13 +409,13 @@ void RedirectIOToConsole ()
   // redirect unbuffered STDOUT to the console
   lStdHandle = (long)GetStdHandle(STD_OUTPUT_HANDLE);
   hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
-  fp = _fdopen( hConHandle, "w" );
+  fp = _fdopen(hConHandle, "w");
   *stdout = *fp;
   setvbuf( stdout, NULL, _IONBF, 0 );
   // redirect unbuffered STDIN to the console
   lStdHandle = (long)GetStdHandle(STD_INPUT_HANDLE);
   hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
-  fp = _fdopen( hConHandle, "r" );
+  fp = _fdopen(hConHandle, "r");
   *stdin = *fp;
   setvbuf( stdin, NULL, _IONBF, 0 );
   // redirect unbuffered STDERR to the console
@@ -360,7 +423,7 @@ void RedirectIOToConsole ()
   hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
   fp = _fdopen( hConHandle, "w" );
   *stderr = *fp;
-  setvbuf( stderr, NULL, _IONBF, 0 );
+  setvbuf(stderr, NULL, _IONBF, 0);
 }
 #endif  /* if defined(__WXMSW__) || (BX_WITH_SDL && defined(WIN32)) */
 
@@ -380,8 +443,8 @@ int WINAPI WinMain(
   RedirectIOToConsole ();
   int max_argv = 20;
   bx_startup_flags.argv = (char**) malloc (max_argv * sizeof (char*));
-  split_string_into_argv (m_lpCmdLine, &bx_startup_flags.argc, bx_startup_flags.argv, max_argv);
-  return bxmain ();
+  split_string_into_argv(m_lpCmdLine, &bx_startup_flags.argc, bx_startup_flags.argv, max_argv);
+  return bxmain();
 }
 #endif
 
@@ -399,17 +462,20 @@ int main (int argc, char *argv[])
 #if defined(WIN32)
   SetConsoleTitle("Bochs for Windows - Console");
 #endif
-  return bxmain ();
+  return bxmain();
 }
 #endif
 
-void print_usage ()
+void print_usage()
 {
   fprintf(stderr, 
     "Usage: bochs [flags] [bochsrc options]\n\n"
     "  -n               no configuration file\n"
     "  -f configfile    specify configuration file\n"
     "  -q               quick start (skip configuration interface)\n"
+#if BX_SUPPORT_SAVE_RESTORE
+    "  -r path          restore the Bochs state from path\n"
+#endif
     "  --help           display this help and exit\n\n"
     "For information on Bochs configuration file arguments, see the\n"
 #if (!defined(WIN32)) && !BX_WITH_MACOS
@@ -433,13 +499,13 @@ int bx_init_main (int argc, char *argv[])
   SAFE_GET_GENLOG();  // never freed
 
   // initalization must be done early because some destructors expect
-  // the bx_options to exist by the time they are called.
-  bx_init_bx_dbg ();
-  bx_init_options ();
+  // the bochs config options to exist by the time they are called.
+  bx_init_bx_dbg();
+  bx_init_options();
 
-  bx_print_header ();
+  bx_print_header();
 
-  SIM->get_param_enum(BXP_BOCHS_START)->set (BX_RUN_START);
+  SIM->get_param_enum(BXPN_BOCHS_START)->set(BX_RUN_START);
 
   // interpret the args that start with -, like -q, -f, etc.
   int arg = 1, load_rcfile=1;
@@ -457,17 +523,27 @@ int bx_init_main (int argc, char *argv[])
       load_rcfile = 0;
     }
     else if (!strcmp ("-q", argv[arg])) {
-      SIM->get_param_enum(BXP_BOCHS_START)->set (BX_QUICK_START);
+      SIM->get_param_enum(BXPN_BOCHS_START)->set(BX_QUICK_START);
     }
     else if (!strcmp ("-f", argv[arg])) {
       if (++arg >= argc) BX_PANIC(("-f must be followed by a filename"));
       else bochsrc_filename = argv[arg];
     }
     else if (!strcmp ("-qf", argv[arg])) {
-      SIM->get_param_enum(BXP_BOCHS_START)->set (BX_QUICK_START);
+      SIM->get_param_enum(BXPN_BOCHS_START)->set(BX_QUICK_START);
       if (++arg >= argc) BX_PANIC(("-qf must be followed by a filename"));
       else bochsrc_filename = argv[arg];
     }
+#if BX_SUPPORT_SAVE_RESTORE
+    else if (!strcmp ("-r", argv[arg])) {
+      if (++arg >= argc) BX_PANIC(("-r must be followed by a path"));
+      else {
+        SIM->get_param_enum(BXPN_BOCHS_START)->set(BX_QUICK_START);
+        SIM->get_param_bool(BXPN_RESTORE_FLAG)->set(1);
+        SIM->get_param_string(BXPN_RESTORE_PATH)->set(argv[arg]);
+      }
+    }
+#endif
 #if BX_WITH_CARBON
     else if (!strncmp ("-psn", argv[arg], 4)) {
       // "-psn" is passed if we are launched by double-clicking
@@ -478,7 +554,7 @@ int bx_init_main (int argc, char *argv[])
       arg = argc; // ignore all other args.
       setupWorkingDirectory (argv[0]);
       // there is no stdin/stdout so disable the text-based config interface.
-      SIM->get_param_enum(BXP_BOCHS_START)->set (BX_QUICK_START);
+      SIM->get_param_enum(BXPN_BOCHS_START)->set(BX_QUICK_START);
       char cwd[MAXPATHLEN];
       getwd (cwd);
       BX_INFO (("Now my working directory is %s", cwd));
@@ -543,7 +619,7 @@ int bx_init_main (int argc, char *argv[])
     if(!isatty(STDIN_FILENO))
     {
       // there is no stdin/stdout so disable the text-based config interface.
-      SIM->get_param_enum(BXP_BOCHS_START)->set (BX_QUICK_START);
+      SIM->get_param_enum(BXPN_BOCHS_START)->set(BX_QUICK_START);
     }
     BX_INFO (("fixing default lib location ..."));
     // locate the lib directory within the application bundle.
@@ -586,6 +662,12 @@ int bx_init_main (int argc, char *argv[])
 
   int norcfile = 1;
 
+#if BX_SUPPORT_SAVE_RESTORE
+  if (SIM->get_param_bool(BXPN_RESTORE_FLAG)->get()) {
+    load_rcfile = 0;
+    norcfile = 0;
+  }
+#endif
   if (load_rcfile) {
     /* parse configuration file and command line arguments */
 #ifdef WIN32
@@ -601,28 +683,38 @@ int bx_init_main (int argc, char *argv[])
 #endif
     if (bochsrc_filename == NULL) bochsrc_filename = bx_find_bochsrc ();
     if (bochsrc_filename)
-      norcfile = bx_read_configuration (bochsrc_filename);
+      norcfile = bx_read_configuration(bochsrc_filename);
   }
 
   if (norcfile) {
     // No configuration was loaded, so the current settings are unusable.
     // Switch off quick start so that we will drop into the configuration
     // interface.
-    if (SIM->get_param_enum(BXP_BOCHS_START)->get() == BX_QUICK_START) {
-      if (!SIM->test_for_text_console ())
+    if (SIM->get_param_enum(BXPN_BOCHS_START)->get() == BX_QUICK_START) {
+      if (!SIM->test_for_text_console())
         BX_PANIC(("Unable to start Bochs without a bochsrc.txt and without a text console"));
       else 
-        BX_ERROR (("Switching off quick start, because no configuration file was found."));
+        BX_ERROR(("Switching off quick start, because no configuration file was found."));
     }
-    SIM->get_param_enum(BXP_BOCHS_START)->set (BX_LOAD_START);
+    SIM->get_param_enum(BXPN_BOCHS_START)->set(BX_LOAD_START);
   }
 
-  // parse the rest of the command line.  This is done after reading the
-  // configuration file so that the command line arguments can override
-  // the settings from the file.
-  if (bx_parse_cmdline (arg, argc, argv)) {
-    BX_PANIC(("There were errors while parsing the command line"));
-    return -1;
+#if BX_SUPPORT_SAVE_RESTORE
+  if (SIM->get_param_bool(BXPN_RESTORE_FLAG)->get()) {
+    if (arg < argc) {
+      BX_ERROR(("WARNING: bochsrc options are ignored in restore mode!"));
+    }
+  }
+  else
+#endif
+  {
+    // parse the rest of the command line.  This is done after reading the
+    // configuration file so that the command line arguments can override
+    // the settings from the file.
+    if (bx_parse_cmdline(arg, argc, argv)) {
+      BX_PANIC(("There were errors while parsing the command line"));
+      return -1;
+    }
   }
   // initialize plugin system. This must happen before we attempt to
   // load any modules.
@@ -630,7 +722,7 @@ int bx_init_main (int argc, char *argv[])
   return 0;
 }
 
-bx_bool load_and_init_display_lib ()
+bx_bool load_and_init_display_lib()
 {
   if (bx_gui != NULL) {
     // bx_gui has already been filled in.  This happens when you start
@@ -639,69 +731,69 @@ bx_bool load_and_init_display_lib ()
     // plugin_init will install wxWidgets as the bx_gui.
     return true;
   }
-  BX_ASSERT (bx_gui == NULL);
-  bx_param_enum_c *ci_param = SIM->get_param_enum (BXP_SEL_CONFIG_INTERFACE);
-  char *ci_name = ci_param->get_choice (ci_param->get());
-  bx_param_enum_c *gui_param = SIM->get_param_enum(BXP_SEL_DISPLAY_LIBRARY);
-  char *gui_name = gui_param->get_choice (gui_param->get());
+  BX_ASSERT(bx_gui == NULL);
+  bx_param_enum_c *ci_param = SIM->get_param_enum(BXPN_SEL_CONFIG_INTERFACE);
+  char *ci_name = ci_param->get_selected();
+  bx_param_enum_c *gui_param = SIM->get_param_enum(BXPN_SEL_DISPLAY_LIBRARY);
+  char *gui_name = gui_param->get_selected();
   if (!strcmp(ci_name, "wx")) {
     BX_ERROR(("change of the config interface to wx not implemented yet"));
   }
-  if (!strcmp (gui_name, "wx")) {
+  if (!strcmp(gui_name, "wx")) {
     // they must not have used wx as the configuration interface, or bx_gui
     // would already be initialized.  Sorry, it doesn't work that way.
-    BX_ERROR (("wxWidgets was not used as the configuration interface, so it cannot be used as the display library"));
+    BX_ERROR(("wxWidgets was not used as the configuration interface, so it cannot be used as the display library"));
     // choose another, hopefully different!
     gui_param->set (0);
-    gui_name = gui_param->get_choice (gui_param->get());
+    gui_name = gui_param->get_selected();
     if (!strcmp (gui_name, "wx")) {
-      BX_PANIC (("no alternative display libraries are available"));
+      BX_PANIC(("no alternative display libraries are available"));
       return false;
     }
-    BX_ERROR (("changing display library to '%s' instead", gui_name));
+    BX_ERROR(("changing display library to '%s' instead", gui_name));
   }
 #if BX_WITH_AMIGAOS
-  if (!strcmp (gui_name, "amigaos")) 
+  if (!strcmp(gui_name, "amigaos")) 
     PLUG_load_plugin (amigaos, PLUGTYPE_OPTIONAL);
 #endif
 #if BX_WITH_BEOS
-  if (!strcmp (gui_name, "beos")) 
+  if (!strcmp(gui_name, "beos")) 
     PLUG_load_plugin (beos, PLUGTYPE_OPTIONAL);
 #endif
 #if BX_WITH_CARBON
-  if (!strcmp (gui_name, "carbon")) 
+  if (!strcmp(gui_name, "carbon")) 
     PLUG_load_plugin (carbon, PLUGTYPE_OPTIONAL);
 #endif
 #if BX_WITH_MACOS
-  if (!strcmp (gui_name, "macos")) 
+  if (!strcmp(gui_name, "macos")) 
     PLUG_load_plugin (macintosh, PLUGTYPE_OPTIONAL);
 #endif
 #if BX_WITH_NOGUI
-  if (!strcmp (gui_name, "nogui")) 
+  if (!strcmp(gui_name, "nogui")) 
     PLUG_load_plugin (nogui, PLUGTYPE_OPTIONAL);
 #endif
 #if BX_WITH_RFB
-  if (!strcmp (gui_name, "rfb")) 
+  if (!strcmp(gui_name, "rfb")) 
     PLUG_load_plugin (rfb, PLUGTYPE_OPTIONAL);
 #endif
 #if BX_WITH_SDL
-  if (!strcmp (gui_name, "sdl")) 
+  if (!strcmp(gui_name, "sdl")) 
     PLUG_load_plugin (sdl, PLUGTYPE_OPTIONAL);
 #endif
 #if BX_WITH_SVGA
-  if (!strcmp (gui_name, "svga")) 
+  if (!strcmp(gui_name, "svga")) 
     PLUG_load_plugin (svga, PLUGTYPE_OPTIONAL);
 #endif
 #if BX_WITH_TERM
-  if (!strcmp (gui_name, "term")) 
+  if (!strcmp(gui_name, "term")) 
     PLUG_load_plugin (term, PLUGTYPE_OPTIONAL);
 #endif
 #if BX_WITH_WIN32
-  if (!strcmp (gui_name, "win32")) 
+  if (!strcmp(gui_name, "win32")) 
     PLUG_load_plugin (win32, PLUGTYPE_OPTIONAL);
 #endif
 #if BX_WITH_X11
-  if (!strcmp (gui_name, "x")) 
+  if (!strcmp(gui_name, "x")) 
     PLUG_load_plugin (x, PLUGTYPE_OPTIONAL);
 #endif
 
@@ -709,62 +801,80 @@ bx_bool load_and_init_display_lib ()
   // set the flag for guis requiring a GUI sighandler.
   // useful when guis are compiled as plugins
   // only term for now
-  if (!strcmp (gui_name, "term")) {
+  if (!strcmp(gui_name, "term")) {
     bx_gui_sighandler = 1;
-    }
+  }
 #endif
 
-  BX_ASSERT (bx_gui != NULL);
+  BX_ASSERT(bx_gui != NULL);
   return true;
 }
 
-int
-bx_begin_simulation (int argc, char *argv[])
+int bx_begin_simulation (int argc, char *argv[])
 {
+#if BX_SUPPORT_SAVE_RESTORE
+  if (SIM->get_param_bool(BXPN_RESTORE_FLAG)->get()) {
+    if (!SIM->restore_config()) {
+      BX_PANIC(("cannot restore configuration"));
+      SIM->get_param_bool(BXPN_RESTORE_FLAG)->set(0);
+    }
+  }
+#endif
   // deal with gui selection
   if (!load_and_init_display_lib ()) {
     BX_PANIC (("no gui module was loaded"));
     return 0;
   }
-  bx_cpu_count = bx_options.Ocpu_count->get();
+
+  bx_cpu_count = SIM->get_param_num(BXPN_CPU_NPROCESSORS)->get() *
+                 SIM->get_param_num(BXPN_CPU_NCORES)->get() *
+                 SIM->get_param_num(BXPN_CPU_NTHREADS)->get();
+
+  BX_ASSERT(bx_cpu_count < BX_MAX_SMP_THREADS_SUPPORTED);
+  BX_ASSERT(bx_cpu_count > 0);
+
+  bx_init_hardware();
+
+  if (SIM->get_param_enum(BXPN_LOAD32BITOS_WHICH)->get()) {
+    void bx_load32bitOSimagehack(void);
+    bx_load32bitOSimagehack();
+  }
+
+  SIM->set_init_done(1);
+
+  // update headerbar buttons since drive status can change during init
+  bx_gui->update_drive_status_buttons();
+  // iniialize statusbar and set all items inactive
+#if BX_SUPPORT_SAVE_RESTORE
+  if (!SIM->get_param_bool(BXPN_RESTORE_FLAG)->get()) {
+#endif
+    bx_gui->statusbar_setitem(-1, 0);
+#if BX_SUPPORT_SAVE_RESTORE
+  }
+#endif
+
+  // The set handler for mouse_enabled does not actually update the gui
+  // until init_done is set.  This forces the set handler to be called,
+  // which sets up the mouse enabled GUI-specific stuff correctly.
+  // Not a great solution but it works. BBD
+  SIM->get_param_bool(BXPN_MOUSE_ENABLED)->set(SIM->get_param_bool(BXPN_MOUSE_ENABLED)->get());
+
 #if BX_DEBUGGER
   // If using the debugger, it will take control and call
   // bx_init_hardware() and cpu_loop()
   bx_dbg_main(argc, argv);
-#else
+#else 
 #if BX_GDBSTUB
   // If using gdbstub, it will take control and call
   // bx_init_hardware() and cpu_loop()
-  if (bx_dbg.gdbstub_enabled) {
-    bx_gdbstub_init (argc, argv);
-  }
+  if (bx_dbg.gdbstub_enabled) bx_gdbstub_init(argc, argv);
   else
 #endif
   {
-    bx_init_hardware();
-
-    if (bx_options.load32bitOSImage.OwhichOS->get()) {
-      void bx_load32bitOSimagehack(void);
-      bx_load32bitOSimagehack();
-    }
-
-    SIM->set_init_done (1);
-
-    // update headerbar buttons since drive status can change during init
-    bx_gui->update_drive_status_buttons ();
-    // iniialize statusbar and set all items inactive
-    bx_gui->statusbar_setitem(-1, 0);
-
-    // The set handler for mouse_enabled does not actually update the gui
-    // until init_done is set.  This forces the set handler to be called,
-    // which sets up the mouse enabled GUI-specific stuff correctly.
-    // Not a great solution but it works. BBD
-    bx_options.Omouse_enabled->set (bx_options.Omouse_enabled->get());
-
 #if BX_SUPPORT_SMP == 0
     // only one processor, run as fast as possible by not messing with
     // quantums and loops.
-    BX_CPU(0)->cpu_loop(1);
+    BX_CPU(0)->cpu_loop(0);
     // for one processor, the only reason for cpu_loop to return is
     // that kill_bochs_request was set by the GUI interface.
 #else
@@ -772,58 +882,91 @@ bx_begin_simulation (int argc, char *argv[])
     // to another.  Increasing quantum speeds up overall performance, but
     // reduces granularity of synchronization between processors.
     int processor = 0;
-    int quantum = 5;
+    int quantum = SIM->get_param_num(BXPN_SMP_QUANTUM)->get();
     while (1) {
       // do some instructions in each processor
       BX_CPU(processor)->cpu_loop(quantum);
       processor = (processor+1) % BX_SMP_PROCESSORS;
-      if (BX_CPU(0)->kill_bochs_request) 
+      if (bx_pc_system.kill_bochs_request) 
         break;
       if (processor == 0) 
         BX_TICKN(quantum);
     }
 #endif
   }
-#endif /* ! BX_DEBUGGER */
-  BX_INFO (("cpu loop quit, shutting down simulator"));
-  bx_atexit ();
+#endif /* BX_DEBUGGER == 0 */
+  BX_INFO(("cpu loop quit, shutting down simulator"));
+  bx_atexit();
   return(0);
 }
 
+void bx_stop_simulation()
+{
+  // in wxWidgets, the whole simulator is running in a separate thread.
+  // our only job is to end the thread as soon as possible, NOT to shut
+  // down the whole application with an exit.
+  BX_CPU(0)->async_event = 1;
+  bx_pc_system.kill_bochs_request = 1;
+  // the cpu loop will exit very soon after this condition is set.
+}
+
+#if BX_SUPPORT_SAVE_RESTORE
+void bx_sr_after_restore_state(void)
+{
+#if BX_SUPPORT_SMP == 0
+  BX_CPU(0)->after_restore_state();
+#else
+  for (unsigned i=0; i<BX_SMP_PROCESSORS; i++) {
+    BX_CPU(i)->after_restore_state();
+  }
+#endif
+  DEV_after_restore_state();
+}
+#endif
+
 int bx_init_hardware()
 {
   // all configuration has been read, now initialize everything.
 
-  if (SIM->get_param_enum(BXP_BOCHS_START)->get()==BX_QUICK_START) {
+  if (SIM->get_param_enum(BXPN_BOCHS_START)->get()==BX_QUICK_START) {
     for (int level=0; level<N_LOGLEV; level++) {
-      int action = SIM->get_default_log_action (level);
-      io->set_log_action (level, action);
+      int action = SIM->get_default_log_action(level);
+      io->set_log_action(level, action);
     }
   }
 
-  bx_pc_system.init_ips(bx_options.Oips->get());
+  bx_pc_system.initialize(SIM->get_param_num(BXPN_IPS)->get());
 
-  if(bx_options.log.Ofilename->getptr()[0]!='-') {
-    BX_INFO (("using log file %s", bx_options.log.Ofilename->getptr()));
-    io->init_log(bx_options.log.Ofilename->getptr());
+  if (SIM->get_param_string(BXPN_LOG_FILENAME)->getptr()[0]!='-') {
+    BX_INFO (("using log file %s", SIM->get_param_string(BXPN_LOG_FILENAME)->getptr()));
+    io->init_log(SIM->get_param_string(BXPN_LOG_FILENAME)->getptr());
   }
 
-  io->set_log_prefix(bx_options.log.Oprefix->getptr());
+  io->set_log_prefix(SIM->get_param_string(BXPN_LOG_PREFIX)->getptr());
 
   // Output to the log file the cpu and device settings
   // This will by handy for bug reports
   BX_INFO(("Bochs x86 Emulator %s", VER_STRING));
   BX_INFO(("  %s", REL_STRING));
   BX_INFO(("System configuration"));
-  BX_INFO(("  processors: %d",BX_SMP_PROCESSORS));
+  BX_INFO(("  processors: %d (cores=%u, HT threads=%u)", BX_SMP_PROCESSORS,
+    SIM->get_param_num(BXPN_CPU_NCORES)->get(), SIM->get_param_num(BXPN_CPU_NTHREADS)->get()));
   BX_INFO(("  A20 line support: %s",BX_SUPPORT_A20?"yes":"no"));
   BX_INFO(("  APIC support: %s",BX_SUPPORT_APIC?"yes":"no"));
   BX_INFO(("CPU configuration"));
   BX_INFO(("  level: %d",BX_CPU_LEVEL));
-  BX_INFO(("  fpu support: %s",BX_SUPPORT_FPU?"yes":"no"));
   BX_INFO(("  paging support: %s, tlb enabled: %s",BX_SUPPORT_PAGING?"yes":"no",BX_USE_TLB?"yes":"no"));
-  BX_INFO(("  mmx support: %s",BX_SUPPORT_MMX?"yes":"no"));
-  BX_INFO(("  sse support: %s",BX_SUPPORT_SSE==2?"2":BX_SUPPORT_SSE==1?"1":"no"));
+#if BX_SUPPORT_SMP
+  BX_INFO(("  SMP support: yes, quantum=%d", SIM->get_param_num(BXPN_SMP_QUANTUM)->get()));
+#else
+  BX_INFO(("  SMP support: no"));
+#endif
+  BX_INFO(("  FPU support: %s",BX_SUPPORT_FPU?"yes":"no"));
+  BX_INFO(("  MMX support: %s",BX_SUPPORT_MMX?"yes":"no"));
+  if (BX_SUPPORT_SSE == 0)
+    BX_INFO(("  SSE support: no"));
+  else
+    BX_INFO(("  SSE support: %d",BX_SUPPORT_SSE));
   BX_INFO(("  v8086 mode support: %s",BX_SUPPORT_V8086_MODE?"yes":"no"));
   BX_INFO(("  VME support: %s",BX_SUPPORT_VME?"yes":"no"));
   BX_INFO(("  3dnow! support: %s",BX_SUPPORT_3DNOW?"yes":"no"));
@@ -847,12 +990,13 @@ int bx_init_hardware()
            BX_SUPPORT_CLGD54XX?"cirrus":""));
 
   // Check if there is a romimage
-  if (strcmp(bx_options.rom.Opath->getptr(),"") == 0) {
+  if (strcmp(SIM->get_param_string(BXPN_ROM_PATH)->getptr(),"") == 0) {
     BX_ERROR(("No romimage to load. Is your bochsrc file loaded/valid ?"));
   }
 
   // set up memory and CPU objects
-  Bit32u memSize = bx_options.memory.Osize->get()*1024*1024;
+  bx_param_num_c *bxp_memsize = SIM->get_param_num(BXPN_MEM_SIZE);
+  Bit32u memSize = bxp_memsize->get() * 1024*1024;
 
 #if BX_SUPPORT_ICACHE
   pageWriteStampTable.alloc(memSize);
@@ -861,34 +1005,37 @@ int bx_init_hardware()
   BX_MEM(0)->init_memory(memSize);
 
   // First load the BIOS and VGABIOS
-  BX_MEM(0)->load_ROM(bx_options.rom.Opath->getptr(), bx_options.rom.Oaddress->get(), 0);
-  BX_MEM(0)->load_ROM(bx_options.vgarom.Opath->getptr(), 0xc0000, 1);
+  BX_MEM(0)->load_ROM(SIM->get_param_string(BXPN_ROM_PATH)->getptr(),
+                      SIM->get_param_num(BXPN_ROM_ADDRESS)->get(), 0);
+  BX_MEM(0)->load_ROM(SIM->get_param_string(BXPN_VGA_ROM_PATH)->getptr(), 0xc0000, 1);
 
   // Then load the optional ROM images
-  if (strcmp(bx_options.optrom[0].Opath->getptr(), "") !=0)
-    BX_MEM(0)->load_ROM(bx_options.optrom[0].Opath->getptr(), bx_options.optrom[0].Oaddress->get(), 2);
-  if (strcmp(bx_options.optrom[1].Opath->getptr(), "") !=0 )
-    BX_MEM(0)->load_ROM(bx_options.optrom[1].Opath->getptr(), bx_options.optrom[1].Oaddress->get(), 2);
-  if (strcmp(bx_options.optrom[2].Opath->getptr(), "") !=0)
-    BX_MEM(0)->load_ROM(bx_options.optrom[2].Opath->getptr(), bx_options.optrom[2].Oaddress->get(), 2);
-  if (strcmp(bx_options.optrom[3].Opath->getptr(), "") !=0)
-    BX_MEM(0)->load_ROM(bx_options.optrom[3].Opath->getptr(), bx_options.optrom[3].Oaddress->get(), 2);
+  if (strcmp(SIM->get_param_string(BXPN_OPTROM1_PATH)->getptr(), "") !=0)
+    BX_MEM(0)->load_ROM(SIM->get_param_string(BXPN_OPTROM1_PATH)->getptr(), SIM->get_param_num(BXPN_OPTROM1_ADDRESS)->get(), 2);
+  if (strcmp(SIM->get_param_string(BXPN_OPTROM2_PATH)->getptr(), "") !=0)
+    BX_MEM(0)->load_ROM(SIM->get_param_string(BXPN_OPTROM2_PATH)->getptr(), SIM->get_param_num(BXPN_OPTROM2_ADDRESS)->get(), 2);
+  if (strcmp(SIM->get_param_string(BXPN_OPTROM3_PATH)->getptr(), "") !=0)
+    BX_MEM(0)->load_ROM(SIM->get_param_string(BXPN_OPTROM3_PATH)->getptr(), SIM->get_param_num(BXPN_OPTROM3_ADDRESS)->get(), 2);
+  if (strcmp(SIM->get_param_string(BXPN_OPTROM4_PATH)->getptr(), "") !=0)
+    BX_MEM(0)->load_ROM(SIM->get_param_string(BXPN_OPTROM4_PATH)->getptr(), SIM->get_param_num(BXPN_OPTROM4_ADDRESS)->get(), 2);
 
   // Then load the optional RAM images
-  if (strcmp(bx_options.optram[0].Opath->getptr(), "") !=0)
-    BX_MEM(0)->load_RAM(bx_options.optram[0].Opath->getptr(), bx_options.optram[0].Oaddress->get(), 2);
-  if (strcmp(bx_options.optram[1].Opath->getptr(), "") !=0)
-    BX_MEM(0)->load_RAM(bx_options.optram[1].Opath->getptr(), bx_options.optram[1].Oaddress->get(), 2);
-  if (strcmp(bx_options.optram[2].Opath->getptr(), "") !=0)
-    BX_MEM(0)->load_RAM(bx_options.optram[2].Opath->getptr(), bx_options.optram[2].Oaddress->get(), 2);
-  if (strcmp(bx_options.optram[3].Opath->getptr(), "") !=0)
-    BX_MEM(0)->load_RAM(bx_options.optram[3].Opath->getptr(), bx_options.optram[3].Oaddress->get(), 2);
+  if (strcmp(SIM->get_param_string(BXPN_OPTRAM1_PATH)->getptr(), "") !=0)
+    BX_MEM(0)->load_RAM(SIM->get_param_string(BXPN_OPTRAM1_PATH)->getptr(), SIM->get_param_num(BXPN_OPTRAM1_ADDRESS)->get(), 2);
+  if (strcmp(SIM->get_param_string(BXPN_OPTRAM2_PATH)->getptr(), "") !=0)
+    BX_MEM(0)->load_RAM(SIM->get_param_string(BXPN_OPTRAM2_PATH)->getptr(), SIM->get_param_num(BXPN_OPTRAM2_ADDRESS)->get(), 2);
+  if (strcmp(SIM->get_param_string(BXPN_OPTRAM3_PATH)->getptr(), "") !=0)
+    BX_MEM(0)->load_RAM(SIM->get_param_string(BXPN_OPTRAM3_PATH)->getptr(), SIM->get_param_num(BXPN_OPTRAM3_ADDRESS)->get(), 2);
+  if (strcmp(SIM->get_param_string(BXPN_OPTRAM4_PATH)->getptr(), "") !=0)
+    BX_MEM(0)->load_RAM(SIM->get_param_string(BXPN_OPTRAM4_PATH)->getptr(), SIM->get_param_num(BXPN_OPTRAM4_ADDRESS)->get(), 2);
 
 #if BX_SUPPORT_SMP == 0
   BX_CPU(0)->initialize(BX_MEM(0));
   BX_CPU(0)->sanity_checks();
+#if BX_SUPPORT_SAVE_RESTORE
+  BX_CPU(0)->register_state();
+#endif
   BX_INSTR_INIT(0);
-  BX_CPU(0)->reset(BX_RESET_HARDWARE);
 #else
   bx_cpu_array = new BX_CPU_C_PTR[BX_SMP_PROCESSORS];
 
@@ -896,17 +1043,39 @@ int bx_init_hardware()
     BX_CPU(i) = new BX_CPU_C(i);
     BX_CPU(i)->initialize(BX_MEM(0));  // assign local apic id in 'initialize' method
     BX_CPU(i)->sanity_checks();
+#if BX_SUPPORT_SAVE_RESTORE
+    BX_CPU(i)->register_state();
+#endif
     BX_INSTR_INIT(i);
-    BX_CPU(i)->reset(BX_RESET_HARDWARE);
   }
 #endif
 
-#if BX_DEBUGGER == 0
   DEV_init_devices();
-  DEV_reset_devices(BX_RESET_HARDWARE);
-  bx_gui->init_signal_handlers ();
-  bx_pc_system.start_timers();
+#if BX_SUPPORT_SAVE_RESTORE
+  bx_pc_system.register_state();
+  DEV_register_state();
+  if (SIM->get_param_bool(BXPN_RESTORE_FLAG)->get()) {
+    if (!SIM->restore_logopts()) {
+      BX_PANIC(("cannot restore log options"));
+      SIM->get_param_bool(BXPN_RESTORE_FLAG)->set(0);
+    }
+  }
 #endif
+  // will enable A20 line and reset CPU and devices
+  bx_pc_system.Reset(BX_RESET_HARDWARE);
+#if BX_SUPPORT_SAVE_RESTORE
+  if (SIM->get_param_bool(BXPN_RESTORE_FLAG)->get()) {
+    if (SIM->restore_hardware()) {
+      bx_sr_after_restore_state();
+    } else {
+      BX_PANIC(("cannot restore hardware state"));
+      SIM->get_param_bool(BXPN_RESTORE_FLAG)->set(0);
+    }
+  }
+#endif
+  bx_gui->init_signal_handlers();
+  bx_pc_system.start_timers();
+
   BX_DEBUG(("bx_init_hardware is setting signal handlers"));
 // if not using debugger, then we can take control of SIGINT.
 #if !BX_DEBUGGER
@@ -923,7 +1092,7 @@ int bx_init_hardware()
   return(0);
 }
 
-void bx_init_bx_dbg (void)
+void bx_init_bx_dbg(void)
 {
   bx_dbg.floppy = 0;
   bx_dbg.keyboard = 0;
@@ -968,22 +1137,22 @@ int bx_atexit(void)
 
   // in case we ended up in simulation mode, change back to config mode
   // so that the user can see any messages left behind on the console.
-  SIM->set_display_mode (DISP_MODE_CONFIG);
+  SIM->set_display_mode(DISP_MODE_CONFIG);
 
 #if BX_PROVIDE_DEVICE_MODELS==1
   bx_pc_system.exit();
 #endif
 
 #if BX_DEBUGGER == 0
-  if (SIM && SIM->get_init_done ()) {
+  if (SIM && SIM->get_init_done()) {
     for (int cpu=0; cpu<BX_SMP_PROCESSORS; cpu++)
       if (BX_CPU(cpu)) BX_CPU(cpu)->atexit();
   }
 #endif
 
 #if BX_SUPPORT_PCI
-  if (SIM && SIM->get_init_done ()) {
-    if (bx_options.Oi440FXSupport->get()) {
+  if (SIM && SIM->get_init_done()) {
+    if (SIM->get_param_bool(BXPN_I440FX_SUPPORT)->get()) {
       bx_devices.pluginPciBridge->print_i440fx_state();
     }
   }
@@ -991,7 +1160,7 @@ int bx_atexit(void)
 
   // restore signal handling to defaults
 #if !BX_DEBUGGER
-  BX_INFO (("restoring default signal behavior"));
+  BX_INFO(("restoring default signal behavior"));
   signal(SIGINT, SIG_DFL);
 #endif
 
@@ -1012,15 +1181,15 @@ void bx_signal_handler(int signum)
   // Otherwise the BX_PANIC() below can be called in multiple threads at
   // once, leading to multiple threads trying to display a dialog box,
   // leading to GUI deadlock.
-  if (!SIM->is_sim_thread ()) {
+  if (!SIM->is_sim_thread()) {
     BX_INFO (("bx_signal_handler: ignored sig %d because it wasn't called from the simulator thread", signum));
     return;
   }
 #if BX_GUI_SIGHANDLER
   if (bx_gui_sighandler) {
     // GUI signal handler gets first priority, if the mask says it's wanted
-    if ((1<<signum) & bx_gui->get_sighandler_mask ()) {
-      bx_gui->sighandler (signum);
+    if ((1<<signum) & bx_gui->get_sighandler_mask()) {
+      bx_gui->sighandler(signum);
       return;
     }
   }
