@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: memory.cc,v 1.43 2005/10/13 16:22:21 sshwarts Exp $
+// $Id: memory.cc,v 1.57 2006/06/01 20:05:15 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -25,69 +25,94 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
 
+#include "bochs.h"
+#include "cpu/cpu.h"
 #include "iodev/iodev.h"
 #define LOG_THIS BX_MEM_THIS
 
 #if BX_PROVIDE_CPU_MEMORY
 
+//
+// Memory map inside the 1st megabyte:
+//
+// 0x00000 - 0x7ffff    DOS area (512K)
+// 0x80000 - 0x9ffff    Optional fixed memory hole (128K)
+// 0xa0000 - 0xbffff    Standard PCI/ISA Video Mem / SMMRAM (128K)
+// 0xc0000 - 0xdffff    Expansion Card BIOS and Buffer Area (128K)
+// 0xe0000 - 0xeffff    Lower BIOS Area (64K)
+// 0xf0000 - 0xfffff    Upper BIOS Area (64K)
+//
+
   void BX_CPP_AttrRegparmN(3)
-BX_MEM_C::writePhysicalPage(BX_CPU_C *cpu, Bit32u addr, unsigned len, void *data)
+BX_MEM_C::writePhysicalPage(BX_CPU_C *cpu, bx_phy_address addr, unsigned len, void *data)
 {
   Bit8u *data_ptr;
-  Bit32u a20addr;
+  bx_phy_address a20addr = A20ADDR(addr);
+  struct memory_handler_struct *memory_handler = NULL;
 
-  // Note: accesses should always be contained within a single page now.
+  // Note: accesses should always be contained within a single page now
 
+  if (cpu != NULL) {
 #if BX_SUPPORT_IODEBUG
-  bx_iodebug_c::mem_write(cpu, addr, len, data);
+    bx_iodebug_c::mem_write(cpu, a20addr, len, data);
 #endif
 
-  a20addr = A20ADDR(addr);
-  BX_INSTR_PHY_WRITE(cpu->which_cpu(), a20addr, len);
+    BX_INSTR_PHY_WRITE(cpu->which_cpu(), a20addr, len);
 
 #if BX_DEBUGGER
-  // (mch) Check for physical write break points, TODO
-  // (bbd) Each breakpoint should have an associated CPU#, TODO
-  for (int i = 0; i < num_write_watchpoints; i++)
-        if (write_watchpoint[i] == a20addr) {
-	      BX_CPU(0)->watchpoint = a20addr;
-              BX_CPU(0)->break_point = BREAK_POINT_WRITE;
-              break;
-        }
+    // (mch) Check for physical write break points, TODO
+    // (bbd) Each breakpoint should have an associated CPU#, TODO
+    for (int i = 0; i < num_write_watchpoints; i++) {
+      if (write_watchpoint[i] == a20addr) {
+        BX_CPU(0)->watchpoint = a20addr;
+        BX_CPU(0)->break_point = BREAK_POINT_WRITE;
+        break;
+      }
+    }
 #endif
 
-  struct memory_handler_struct *memory_handler = memory_handlers[a20addr >> 20];
+#if BX_SUPPORT_APIC
+    bx_generic_apic_c *local_apic = &cpu->local_apic;
+    if (local_apic->is_selected(a20addr, len)) {
+      local_apic->write(a20addr, (Bit32u *)data, len);
+      return;
+    }
+#endif
+
+    if ((a20addr & 0xfffe0000) == 0x000a0000 && (BX_MEM_THIS smram_available))
+    {
+      // SMRAM memory space
+      if (BX_MEM_THIS smram_enable || (cpu->smm_mode() && !BX_MEM_THIS smram_restricted))
+        goto mem_write;
+    }
+  }
+
+  memory_handler = memory_handlers[a20addr >> 20];
   while (memory_handler) {
     if (memory_handler->begin <= a20addr &&
           memory_handler->end >= a20addr &&
-          memory_handler->write_handler(a20addr, len, data, memory_handler->write_param))
+          memory_handler->write_handler(a20addr, len, data, memory_handler->param))
     {
       return;
     }
     memory_handler = memory_handler->next;
   }
 
+mem_write:
+
+  // all memory access feets in single 4K page 
+  if (a20addr < BX_MEM_THIS len) {
 #if BX_SUPPORT_ICACHE
-  if (a20addr < BX_MEM_THIS len)
     pageWriteStampTable.decWriteStamp(a20addr);
 #endif
-
-#if BX_SUPPORT_APIC
-  bx_generic_apic_c *local_apic = &cpu->local_apic;
-  bx_generic_apic_c *ioapic = bx_devices.ioapic;
-  if (local_apic->is_selected (a20addr, len)) {
-    local_apic->write (a20addr, (Bit32u *)data, len);
-    return;
-  }
-  if (ioapic->is_selected (a20addr, len)) {
-    ioapic->write (a20addr, (Bit32u *)data, len);
-    return;
-  }
-#endif
-
-  if ( (a20addr + len) <= BX_MEM_THIS len ) {
     // all of data is within limits of physical memory
-    if ( (a20addr & 0xfff80000) != 0x00080000 ) {
+    if ((a20addr & 0xfff80000) != 0x00080000 || (a20addr <= 0x0009ffff))
+    {
+      if (len == 8) {
+        WriteHostQWordToLittleEndian(&vector[a20addr], *(Bit64u*)data);
+        BX_DBG_DIRTY_PAGE(a20addr >> 12);
+        return;
+      }
       if (len == 4) {
         WriteHostDWordToLittleEndian(&vector[a20addr], *(Bit32u*)data);
         BX_DBG_DIRTY_PAGE(a20addr >> 12);
@@ -107,14 +132,15 @@ BX_MEM_C::writePhysicalPage(BX_CPU_C *cpu, Bit32u addr, unsigned len, void *data
     }
 
 #ifdef BX_LITTLE_ENDIAN
-  data_ptr = (Bit8u *) data;
+    data_ptr = (Bit8u *) data;
 #else // BX_BIG_ENDIAN
-  data_ptr = (Bit8u *) data + (len - 1);
+    data_ptr = (Bit8u *) data + (len - 1);
 #endif
 
 write_one:
-    if ( (a20addr & 0xfff80000) != 0x00080000 ) {
-      // addr *not* in range 00080000 .. 000FFFFF
+    if ((a20addr & 0xfff80000) != 0x00080000 || (a20addr <= 0x0009ffff))
+    {
+      // addr *not* in range 000A0000 .. 000FFFFF
       vector[a20addr] = *data_ptr;
       BX_DBG_DIRTY_PAGE(a20addr >> 12);
 inc_one:
@@ -129,24 +155,27 @@ inc_one:
       goto write_one;
     }
 
-    // addr in range 00080000 .. 000FFFFF
+    // addr must be in range 000A0000 .. 000FFFFF
 
-    if (a20addr <= 0x0009ffff) {
-      // regular memory 80000 .. 9FFFF
-      vector[a20addr] = *data_ptr;
-      BX_DBG_DIRTY_PAGE(a20addr >> 12);
+    // SMMRAM
+    if (a20addr <= 0x000bffff) {
+      // devices are not allowed to access SMMRAM under VGA memory
+      if (cpu) {
+        vector[a20addr] = *data_ptr;
+        BX_DBG_DIRTY_PAGE(a20addr >> 12);
+      }
       goto inc_one;
     }
+
     // adapter ROM     C0000 .. DFFFF
     // ROM BIOS memory E0000 .. FFFFF
 #if BX_SUPPORT_PCI == 0
     // ignore write to ROM
 #else
     // Write Based on 440fx Programming
-    if ( bx_options.Oi440FXSupport->get () &&
-          ((a20addr & 0xfffc0000) == 0x000c0000) )
+    if (pci_enabled && ((a20addr & 0xfffc0000) == 0x000c0000))
     {
-      switch (DEV_pci_wr_memtype(a20addr & 0xFC000)) {
+      switch (DEV_pci_wr_memtype(a20addr)) {
         case 0x1:   // Writes to ShadowRAM
           BX_DEBUG(("Writing to ShadowRAM: address %08x, data %02x", (unsigned) a20addr, *data_ptr));
           vector[a20addr] = *data_ptr;
@@ -160,88 +189,82 @@ inc_one:
         default:
           BX_PANIC(("writePhysicalPage: default case"));
           goto inc_one;
-        }
       }
+    }
 #endif
     goto inc_one;
   }
   else {
-    // some or all of data is outside limits of physical memory
-
-#ifdef BX_LITTLE_ENDIAN
-    data_ptr = (Bit8u *) data;
-#else // BX_BIG_ENDIAN
-    data_ptr = (Bit8u *) data + (len - 1);
-#endif
-
-    for (unsigned i = 0; i < len; i++) {
-      if (a20addr < BX_MEM_THIS len) {
-        vector[a20addr] = *data_ptr;
-        BX_DBG_DIRTY_PAGE(a20addr >> 12);
-      }
-      // otherwise ignore byte, since it overruns memory
-      addr++;
-      a20addr = (addr);
-#ifdef BX_LITTLE_ENDIAN
-      data_ptr++;
-#else // BX_BIG_ENDIAN
-      data_ptr--;
-#endif
-    }
+    // access outside limits of physical memory, ignore
+    BX_DEBUG(("Write outside the limits of physical memory (ignore)"));
   }
 }
 
   void BX_CPP_AttrRegparmN(3)
-BX_MEM_C::readPhysicalPage(BX_CPU_C *cpu, Bit32u addr, unsigned len, void *data)
+BX_MEM_C::readPhysicalPage(BX_CPU_C *cpu, bx_phy_address addr, unsigned len, void *data)
 {
   Bit8u *data_ptr;
-  Bit32u a20addr;
+  bx_phy_address a20addr = A20ADDR(addr);
+  struct memory_handler_struct *memory_handler = NULL;
 
+  // Note: accesses should always be contained within a single page now
+
+  if (cpu != NULL) {
 #if BX_SUPPORT_IODEBUG
-  bx_iodebug_c::mem_read(cpu, addr, len, data);
+    bx_iodebug_c::mem_read(cpu, a20addr, len, data);
 #endif
  
-  a20addr = A20ADDR(addr);
-  BX_INSTR_PHY_READ(cpu->which_cpu(), a20addr, len);
+    BX_INSTR_PHY_READ(cpu->which_cpu(), a20addr, len);
 
 #if BX_DEBUGGER
-  // (mch) Check for physical read break points, TODO
-  // (bbd) Each breakpoint should have an associated CPU#, TODO
-  for (int i = 0; i < num_read_watchpoints; i++)
-        if (read_watchpoint[i] == a20addr) {
-	      BX_CPU(0)->watchpoint = a20addr;
-              BX_CPU(0)->break_point = BREAK_POINT_READ;
-              break;
-        }
+    // (mch) Check for physical read break points, TODO
+    // (bbd) Each breakpoint should have an associated CPU#, TODO
+    for (int i = 0; i < num_read_watchpoints; i++) {
+      if (read_watchpoint[i] == a20addr) {
+         BX_CPU(0)->watchpoint = a20addr;
+         BX_CPU(0)->break_point = BREAK_POINT_READ;
+         break;
+      }
+    }
 #endif
 
-  struct memory_handler_struct *memory_handler = memory_handlers[a20addr >> 20];
+#if BX_SUPPORT_APIC
+    bx_generic_apic_c *local_apic = &cpu->local_apic;
+    if (local_apic->is_selected (a20addr, len)) {
+      local_apic->read(a20addr, data, len);
+      return;
+    }
+#endif
+
+    if ((a20addr & 0xfffe0000) == 0x000a0000 && (BX_MEM_THIS smram_available))
+    {
+      // SMRAM memory space
+      if (BX_MEM_THIS smram_enable || (cpu->smm_mode() && !BX_MEM_THIS smram_restricted))
+        goto mem_read;
+    }
+  }
+
+  memory_handler = memory_handlers[a20addr >> 20];
   while (memory_handler) {
     if (memory_handler->begin <= a20addr &&
           memory_handler->end >= a20addr &&
-          memory_handler->read_handler(a20addr, len, data, memory_handler->read_param))
+          memory_handler->read_handler(a20addr, len, data, memory_handler->param))
     {
       return;
     }
     memory_handler = memory_handler->next;
   }
 
-#if BX_SUPPORT_APIC
-  bx_generic_apic_c *local_apic = &cpu->local_apic;
-  bx_generic_apic_c *ioapic = bx_devices.ioapic;
-  if (local_apic->is_selected (addr, len)) {
-    local_apic->read (addr, data, len);
-    return;
-  }
-  if (ioapic->is_selected (addr, len)) {
-    ioapic->read (addr, data, len);
-    return;
-  }
-#endif
+mem_read:
 
-  if ( (a20addr + len) <= BX_MEM_THIS len ) {
+  if (a20addr <= BX_MEM_THIS len) {
     // all of data is within limits of physical memory
-    if ( (a20addr & 0xfff80000) != 0x00080000 ) {
+    if ((a20addr & 0xfff80000) != 0x00080000 || (a20addr <= 0x0009ffff))
+    {
+      if (len == 8) {
+        ReadHostQWordFromLittleEndian(&vector[a20addr], * (Bit64u*) data);
+        return;
+      }
       if (len == 4) {
         ReadHostDWordFromLittleEndian(&vector[a20addr], * (Bit32u*) data);
         return;
@@ -254,10 +277,9 @@ BX_MEM_C::readPhysicalPage(BX_CPU_C *cpu, Bit32u addr, unsigned len, void *data)
         * (Bit8u *) data = * ((Bit8u *) (&vector[a20addr]));
         return;
       }
-      // len == 3 case can just fall thru to special cases handling
+      // len == other case can just fall thru to special cases handling
     }
 
-
 #ifdef BX_LITTLE_ENDIAN
     data_ptr = (Bit8u *) data;
 #else // BX_BIG_ENDIAN
@@ -265,7 +287,8 @@ BX_MEM_C::readPhysicalPage(BX_CPU_C *cpu, Bit32u addr, unsigned len, void *data)
 #endif
 
 read_one:
-    if ( (a20addr & 0xfff80000) != 0x00080000 ) {
+    if ((a20addr & 0xfff80000) != 0x00080000 || (a20addr <= 0x0009ffff))
+    {
       // addr *not* in range 00080000 .. 000FFFFF
       *data_ptr = vector[a20addr];
 inc_one:
@@ -280,14 +303,21 @@ inc_one:
       goto read_one;
     }
 
-    // addr in range 00080000 .. 000FFFFF
+    // addr must be in range 000A0000 .. 000FFFFF
+
+    // SMMRAM
+    if (a20addr <= 0x000bffff) {
+      // devices are not allowed to access SMMRAM under VGA memory
+      if (cpu) *data_ptr = vector[a20addr];
+      goto inc_one;
+    }
+
 #if BX_SUPPORT_PCI
-    if ( bx_options.Oi440FXSupport->get () &&
-          ((a20addr & 0xfffc0000) == 0x000c0000) )
+    if (pci_enabled && ((a20addr & 0xfffc0000) == 0x000c0000))
     {
-      switch (DEV_pci_rd_memtype (a20addr)) {
+      switch (DEV_pci_rd_memtype(a20addr)) {
         case 0x0:  // Read from ROM
-          if ( (a20addr & 0xfffe0000) == 0x000e0000 )
+          if ((a20addr & 0xfffe0000) == 0x000e0000)
           {
             *data_ptr = rom[a20addr & BIOS_MASK];
           }
@@ -307,10 +337,10 @@ inc_one:
     else
 #endif  // #if BX_SUPPORT_PCI
     {
-      if ( (a20addr & 0xfffc0000) != 0x000c0000 ) {
+      if ((a20addr & 0xfffc0000) != 0x000c0000) {
         *data_ptr = vector[a20addr];
       }
-      else if ( (a20addr & 0xfffe0000) == 0x000e0000 )
+      else if ((a20addr & 0xfffe0000) == 0x000e0000)
       {
         *data_ptr = rom[a20addr & BIOS_MASK];
       }
@@ -322,7 +352,7 @@ inc_one:
     }
   }
   else
-  {  // some or all of data is outside limits of physical memory
+  {  // access outside limits of physical memory
 
 #ifdef BX_LITTLE_ENDIAN
     data_ptr = (Bit8u *) data;
@@ -331,9 +361,7 @@ inc_one:
 #endif
 
     for (unsigned i = 0; i < len; i++) {
-      if (a20addr < BX_MEM_THIS len)
-        *data_ptr = vector[a20addr];
-      else if (a20addr >= (Bit32u)~BIOS_MASK)
+      if (a20addr >= (bx_phy_address)~BIOS_MASK)
         *data_ptr = rom[a20addr & BIOS_MASK];
       else
         *data_ptr = 0xff;
