@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: osdep.cc,v 1.14.2.1 2004/02/06 22:14:34 danielg4 Exp $
+// $Id: osdep.cc,v 1.17 2006/08/07 17:59:15 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -55,8 +55,15 @@ int bx_snprintf (char *s, size_t maxlen, const char *format, ...)
 
   return done;
 }
+
 #endif  /* !BX_HAVE_SNPRINTF */
 
+#if !BX_HAVE_VSNPRINTF
+int bx_vsnprintf (char *s, size_t maxlen, const char *format, va_list arg)
+{
+  return vsprintf (s, format, arg);
+}
+#endif /* !BX_HAVE_VSNPRINTF*/
 
 #if (!BX_HAVE_STRTOULL && !BX_HAVE_STRTOUQ)
 /* taken from glibc-2.2.2: strtod.c, and stripped down a lot.  There are 
@@ -314,18 +321,12 @@ int fd_stat(struct stat *buf)
 //////////////////////////////////////////////////////////////////////
 
 #if BX_HAVE_REALTIME_USEC
-#  if BX_HAVE_GETTIMEOFDAY
-Bit64u bx_get_realtime64_usec (void) {
-  timeval thetime;
-  gettimeofday(&thetime,0);
-  Bit64u mytime;
-  mytime=(Bit64u)thetime.tv_sec*(Bit64u)1000000+(Bit64u)thetime.tv_usec;
-  return mytime;
-}
-#  elif defined(WIN32)
-Bit64u last_realtime64_top = 0;
-Bit64u last_realtime64_bottom = 0;
-Bit64u bx_get_realtime64_usec (void) {
+#if defined(WIN32)
+static Bit64u last_realtime64_top = 0;
+static Bit64u last_realtime64_bottom = 0;
+
+Bit64u bx_get_realtime64_usec(void)
+{
   Bit64u new_bottom = ((Bit64u) GetTickCount()) & BX_CONST64(0x0FFFFFFFF);
   if(new_bottom < last_realtime64_bottom) {
     last_realtime64_top += BX_CONST64(0x0000000100000000);
@@ -336,5 +337,14 @@ Bit64u bx_get_realtime64_usec (void) {
     (new_bottom          & BX_CONST64(0x00000000FFFFFFFF));
   return interim_realtime64*(BX_CONST64(1000));
 }
-#  endif
+#elif BX_HAVE_GETTIMEOFDAY
+Bit64u bx_get_realtime64_usec(void)
+{
+  timeval thetime;
+  gettimeofday(&thetime,0);
+  Bit64u mytime;
+  mytime=(Bit64u)thetime.tv_sec*(Bit64u)1000000+(Bit64u)thetime.tv_usec;
+  return mytime;
+}
+#endif
 #endif
