@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: plugin.cc,v 1.15 2006/01/21 09:28:49 vruppert Exp $
+// $Id: plugin.cc,v 1.18 2006/05/27 15:54:47 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 // This file defines the plugin and plugin-device registration functions and
@@ -515,6 +515,7 @@ void pluginRegisterDeviceDevmodel(plugin_t *plugin, plugintype_t type, bx_devmod
         // Core devices are present whether or not we are using plugins, so
         // they are managed by the same code in iodev/devices.cc whether
         // plugins are on or off.  
+        free(device);
         return; // Do not add core devices to the devices list.
       case PLUGTYPE_OPTIONAL:
       case PLUGTYPE_USER:
@@ -606,4 +607,33 @@ void bx_reset_plugins(unsigned signal)
     }
 }
 
+#if BX_SUPPORT_SAVE_RESTORE
+/**************************************************************************/
+/* Plugin system: Register device state of all registered plugin-devices  */
+/**************************************************************************/
+
+void bx_plugins_register_state()
+{
+    device_t *device;
+    for (device = devices; device; device = device->next)
+    {
+      pluginlog->info("register state of '%s' plugin device by virtual method",device->name);
+      device->devmodel->register_state();
+    }
+}
+
+/***************************************************************************/
+/* Plugin system: Execute code after restoring state of all plugin devices */
+/***************************************************************************/
+
+void bx_plugins_after_restore_state()
+{
+    device_t *device;
+    for (device = devices; device; device = device->next)
+    {
+      device->devmodel->after_restore_state();
+    }
+}
+#endif
+
 }
